package display;

import uchicago.src.collection.RangeMap;
import uchicago.src.sim.analysis.LocalDataRecorder;
import uchicago.src.sim.engine.*;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;
import cern.jet.random.Gamma;
import cern.jet.random.Normal;
import cern.jet.random.Uniform;
import cern.jet.random.engine.MersenneTwister;

import java.awt.*;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

import units.*;
import health.*;
import infrastructure.*;
import clubs.*;
import economics.*;
import flanagan.math.PsRandom;

public class Model extends SimModelImpl {
	/** Main schedule */
	private Schedule schedule;
	/** Initial action */
	private BasicAction initialAction;
	/**List of agents */
	private ArrayList<Agent> agentList = new ArrayList<Agent>();
	/** List of agents going hungry at current time */
	private ArrayList<Agent> hungryAgents = new ArrayList<Agent>();
	/**List of households */
	private ArrayList<Household> householdList = new ArrayList<Household>();
	/**Maps agents with their IDS*/
	private HashMap<Integer, Agent> agentMap = new HashMap<Integer, Agent>();
	/**List of current HIV Patients */
	private ArrayList<Agent> hivPatients = new ArrayList<Agent>();
	/**List of current HIV Patients who are outside */
	private ArrayList<Agent> hivOutsiderPatients = new ArrayList<Agent>();
	/**List of dying this step */
	private ArrayList<Agent> dead = new ArrayList<Agent>();
	/**Map of households with their IDs */
	private HashMap<Integer, Household> houseMap = new HashMap<Integer, Household>();
	/**List of funeral clubs */
	private ArrayList<FuneralClub> funeralClubs = new ArrayList<FuneralClub>();
	/**List of savings clubs */
	private ArrayList<SavingsClub> savingsClubs = new ArrayList<SavingsClub>();
	/**List of migrated agent */
	private ArrayList<Agent> migratedAgents = new ArrayList<Agent>();
	/**List of unmarried couples */
	private ArrayList<Couple> partnersList = new ArrayList<Couple>();
	/**Agency awarding grants */
	private FinanceAgency agency;
	private ModelGUI gui;
	/**Gets AIDS prevalence */
	private AidsSpread aidSpread = new AidsSpread();
	/**Gives agents' fast health decay */
	private Sigmoid sig = new Sigmoid();
	/**Gives HIV incidence */
	private Weibull weibull = new Weibull();
	private Fermi fermi = new Fermi();
	/**Abstract clinic */
	private Clinic clinic = new Clinic();
	/**Abstract mine */ 
	private Mine mine = new Mine();
	private KS ks = new KS();

	/**Distribution for agents' expected life */
	Normal expectedAgeNormal = new Normal(Settings.MEAN_LIFE_EXPECTANCY,
			Settings.SD_LIFE_EXPECTANCY, new MersenneTwister());

	/**Distribution for female agents' fertility */
	Normal femaleFertilityNormal = new Normal(33, 5, new MersenneTwister());
	PsRandom psr = new PsRandom();
	PsRandom outsiderPsr = new PsRandom();
	/**Distribution for agents' courtship duration */
	PsRandom courtShipDuration = new PsRandom();

	/**seed for empirical distributions */
	private int seed = 123; 
	
	/** for health expenses */
	boolean healthExpenditure = Settings.IS_HEALTH_EXPENDITURE;
	/**Should include HIV/AIDS */
	boolean HIV = Settings.IS_HIV;
	/**Should display Gui */
	boolean showGui = Settings.SHOW_GUI;
	/**Records network at this time interval */
	int networkRecordInterval = Settings.PAJEK_INTERVAL;

	/** network display variables */
	private DisplaySurface householdSurface, agentSurface, 
	savingsSurface, extendedFamilySurface;
	private AbstractGraphLayout agentFriendsGraphLayout, householdGraphLayout,
	savingsGraphLayout, extendedFamilyGraphLayout;
	private Network2DDisplay householdDisplayNetwork, agentDisplayNetwork,
	savingsDisplayNetwork, extendedFamilyDisplayNetwork;
	/**initial number of households */
	int initNumHouseholds = 0, minHouseholdSize = 0,
	maxHouseholdSize = 0, initialFuneralClubs = 0, lastHouseholdID = 0,
	lastAgentID = 0, clusterProportion = 0, lastCoupleID = 0;
	private int initialSteps = 0;	
	/**number of birth at this year */
	private int numBirthsThisYear = 0;
	/**number of infant deaths at this year */
	private int numInfantsDeathThisYear = 0;
	/**number of birth last year */
	private int numBirthsLastYear = 0;
	/**number of possible new infections this year */
	private int numIncidenceThisYear = 0;	
	private int proportionPatients = 0;
	private int currentTick = 0;
	private int incidence = 0;
	private LocalDataRecorder recorder, recorder2, recorder3, recorder4, recorder5, recorder6;

	//recording variables
	public int numberBirths = 0, numDeathsPerYear = 0, numHHReceivingGrants = 0, 
	numHHLackingFood = 0, numHHFuneralClubMembers = 0,
	numHHGracePeriod = 0, numHHStokvelMembers = 0,
	numAgentsPieceWork = 0, numAgentsJobless = 0, numAgentsNotLaborFit = 0,
	numAgentsOnMineJob = 0, numChildrenBornHIVParents = 0,
	numHHDissolved = 0, numHHAccommodated = 0,
	numAdults = 0, numChildren = 0, numSeniors = 0,
	numAgentsOnStreet = 0, numChildHH = 0;
	public int numSingleMales = 0, numSingleFemales = 0, numMarriedAgents = 0,
	numWidowedMales = 0, numWidowedFemales = 0,
	numSexuallyActiveAgents = 0, numUnPairedIndividuals = 0,
	numDyads = 0;
	double meanFemaleDegree = 0, meanMaleDegree = 0, varFemaleDegree = 0, varMaleDegree = 0;
	private ArrayList<Household> currentEmployers = new ArrayList<Household>();

	//for Procedural Stokvels
	private ArrayList<Message> stokvelMessageInbox = new ArrayList<Message>();
	private ArrayList<Message> stokvelMessageOutbox = new ArrayList<Message>();

	private ArrayList<Message> partnershipMessageInbox = new ArrayList<Message>();
	private ArrayList<Message> partnershipMessageOutbox = new ArrayList<Message>();
	/**from Repast JinGirNew Implemenation */ 
	private RangeMap neighborMap = new RangeMap();
	private RangeMap removalMap = new RangeMap();
	/**from Repast JinGirNew Implemenation */	
	private double removeProb = 0.005;
	private double Rsub0 = 0.005;
	private double Rsub1 = 5;

	/**List of outsiders */
	private ArrayList<Agent> outsidersList = new ArrayList<Agent>();
	/**List of pregnant females */
	private ArrayList<Agent> pregnantFemales = new ArrayList<Agent>();
	/**Map for churches */
	private HashMap<Integer, ArrayList<Household>> churches = new HashMap<Integer, ArrayList<Household>>();

	/**
	 * Default constructor	 
	 */
	public Model() {
	}

	/**
	 * Builds the agents, households, churches and family clusters
	 */
	public void buildModel() {
		//earlier version
//		createHouseholds();
		//empirical stuff - from Ruth
		createEmpiricalHouseholds();
		agency = new FinanceAgency(this);
		createHouseholdClusters();
		createChurches();
		//RM 
		assignEducationLevels();
		record();
	}

	/**
	 * Creates churches and assign households to them.
	 *
	 */
	public void createChurches() {
		// choose number of denominations in the village (randomly between 1 and numDenominations)
		int numChurches = Random.uniform.nextIntFromTo(1, Settings.Church.NUMBER_DENOMINATIONS);		
		// create churches
		churches.put(-1, new ArrayList<Household>());		
		for (int i=0; i<numChurches; i++) {
			ArrayList<Household> members = new ArrayList<Household>();
			churches.put(new Integer(i), members);			
		}
		for (int i=0; i < this.householdList.size(); i++) {
			Household household = householdList.get(i);
			if (Random.uniform.nextDouble() < Settings.Church.CHURCH_PARTICIPATION_RATE) {
				int churchID = Random.uniform.nextIntFromTo(0, numChurches-1);
				churches.get(new Integer(churchID)).add(household);
				household.setChurchID(churchID);
			}
			else {
				churches.get(-1).add(household);				
			}
		}
	}

	/** 
	 * Creates funeral clubs and assign some households initially
	 *
	 */
	public void createFuneralClubs() {
		for (int i = 0; i < this.initialFuneralClubs; i++) {
			FuneralClub funeralClub = new FuneralClub(i);
			initializeFuneralClub(funeralClub);
			this.funeralClubs.add(funeralClub);
		}
	}

	/** populate it with some members for a kick-start ... */
	public void initializeFuneralClub(FuneralClub club) {
		SimUtilities.shuffle(this.householdList);
		int initialSize = Random.uniform.nextIntFromTo(3, 5);
		for (Household household : this.householdList) {
			if (!household.isMemberFuneralClub() && (initialSize > 0)) {
				club.addMember(household);
				household.setMemberFuneralClub(true);
				household.setFuneralClub(club);
			}
			initialSize--;
		}
	}

	/**
	 * from Radar:The household size varies from 1 to 19, with a mean of
	 * 6.98. The size distribution can be approximated by a Normal distribution
	 * with mean = 7 and standard deviation = 3 (variance is 9.003).
	 */
	public void createHouseholds() {
		int size = 0, agentCount = 0;
		Normal myNormal = new Normal(7, 3, new MersenneTwister());
		for (int i = 0; i < this.initNumHouseholds; i++) {
			// size = Random.uniform.nextIntFromTo(minHouseholdSize, maxHouseholdSize);
			do {
				size = (int) myNormal.nextInt();
			} while (size <= 5);
			Household household = new Household(i, this);
			lastHouseholdID = i;
			agentCount = createHouseholdMembers(household, size, agentCount);
			// now set head, grants (if any)
			household.initialize();
			int wealth = Random.uniform.nextIntFromTo(Settings.MIN_STARTING_WEALTH, 
					Settings.MAX_STARTING_WEALTH);
			household.addWealth(wealth);
			householdList.add(household);
			houseMap.put(new Integer(household.getId()), household);
		}
		this.lastAgentID = agentCount;
	}

	/**
	 * this is to match the 'anectodal comments: . [in reality there would be a
	 * few hh  possibly 30% with no seniors, there would also be some with no
	 * children 5%, and some with seniors and no adults but still children, 5%]
	 * this is just to improve towards it ...
	 */
	public int createHouseholdMembers(Household household, int size, int curCount) {
		// this is a speical case ... one female and one male
		Agent husband = new Agent(curCount, Settings.GENDER.MALE, Settings.AGE_GROUP.ADULT, 
				Settings.HEALTH_STATUS.WELL, household, this);
		curCount++;
		household.addAgent(husband);
		Agent wife = new Agent(curCount, Settings.GENDER.FEMALE,Settings.AGE_GROUP.ADULT, 
				Settings.HEALTH_STATUS.WELL, household, this);
		curCount++;
		household.addAgent(wife);

		Couple couple = new Couple (this.lastCoupleID, husband, wife);
		couple.setCoupleBeginTick(0);
		this.lastCoupleID++;

		couple.setMarried(true);
		household.addCouple(couple);
		wife.setMaternalHouseholdID(household.getId());

		husband.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
		husband.setSpouseID(wife.getId());
		wife.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
		wife.setSpouseID(husband.getId());

		// minus husband and wife
		size -= 2;

		Agent agent;
		int gender = -1, ageGroup = -1, numSeniors = Random.uniform.nextIntFromTo(0, 2);
		ageGroup = Settings.AGE_GROUP.SENIOR;
		while ((numSeniors > 0) && (size > 0)) {
			gender = Math.random() < 0.50 ? Settings.GENDER.FEMALE
					: Settings.GENDER.MALE;
			agent = new Agent(curCount, gender, ageGroup,
					Settings.HEALTH_STATUS.OLD_AGE_SICKNESS, household, this);
			curCount++;
			household.addAgent(agent);
			size--;
			numSeniors--;
		}

		int numOtherAdults = Random.uniform.nextIntFromTo(0, 3);
		ageGroup = Settings.AGE_GROUP.ADULT;

		while ((numOtherAdults > 0) && (size > 0)) {
			gender = Math.random() < 0.50 ? Settings.GENDER.FEMALE
					: Settings.GENDER.MALE;
			agent = new Agent(curCount, gender, Settings.AGE_GROUP.ADULT,
					Settings.HEALTH_STATUS.WELL, household, this);
			curCount++;
			household.addAgent(agent);
			size--;
			numOtherAdults--;
		}

		for (int i = 0; i < size; i++) {
			gender = Math.random() < 0.50 ? Settings.GENDER.FEMALE
					: Settings.GENDER.MALE;
			ageGroup = Settings.AGE_GROUP.CHILD;
			agent = new Agent(curCount, gender, ageGroup,
					Settings.HEALTH_STATUS.WELL, household, this);
			if (ageGroup == Settings.AGE_GROUP.CHILD) {
				agent.setFatherID(husband.getId());
				agent.setMotherID(wife.getId());
			}
			household.addAgent(agent);
			curCount++;
		}

		/*
		 * this is just an idea to assign the seniors as parents of adults ...
		 * which could be it can be optimized a lot of times ... this is just
		 * for testinng purposes
		 */
		for (Agent senior : household.getSeniors()) {
			int seniorAge = senior.getAge(), seniorID = senior.getId(), 
			seniorGender = senior.getGender();
			for (Agent adult : household.getAdults()) {
				if (seniorAge - adult.getAge() > 15) {
					if (seniorGender == Settings.GENDER.FEMALE) {
						adult.setMotherID(seniorID);
					} else {
						adult.setFatherID(seniorID);
					}
				}
			}
		}
		if (household.getMembers().size() == 0) {
			size++;
		}
		return curCount;
	}

	/**
	 * this function is called when child agent, if any, is to be created during
	 * the simulation run
	 */
	public void createChildAgent (int _id, Agent mother, Agent father, Household household) {
		int gender = Math.random() > 0.5 ? Settings.GENDER.FEMALE
				: Settings.GENDER.MALE;
		Agent newBorn = new Agent(_id, gender, Settings.HEALTH_STATUS.WELL,
				household, this);						
		newBorn.setFatherID(mother.getFatherOfMyChildID());
		newBorn.setMotherID(mother.getId());
		household.addAgent(newBorn);
		this.lastAgentID = _id;
		if (Math.random() < Settings.INNOVATOR_PROBABILITY) {
			newBorn.setInnovator(true);
		}						
		if (mother.hasHIV()) {
			if (motherToChildAIDSTransfer(mother, newBorn)) {
				MTCT(newBorn);
			}
		}
		if (father != null) {
			father.getOffSprings().add(newBorn);
		}
		if (mother.getId() != -1) {
			mother.getOffSprings().add(newBorn);
		}
		mother.setPregnant(false);		
		this.pregnantFemales.remove(mother);
		this.numBirthsThisYear--;
		this.numberBirths++;
		this.numBirthsLastYear++;
	}	

	/**
	 * Gives the chances for MTCT transfer for the newborn
	 * @param newBorn
	 */
	public void MTCT(Agent newBorn) {
		setHIVInfection(newBorn);
		double chance = Math.random();
		if (chance < 0.325) {
			newBorn.setDeathTick(currentTick + Uniform.staticNextIntFromTo(2, 13));
			return;
		}
		else if (chance < 0.525) {
			newBorn.setDeathTick(currentTick + Uniform.staticNextIntFromTo(13, 29));
			return;
		}
	}

	/**	
	 * from avert.org: "Without treatment, around 15-30% of babies born to
	 * HIV positive women will become infected with HIV during pregnancy and
	 * delivery." here the assumption is that their is no nevirapine or
	 * Antiretroviral drugs are available ... which can reduce the risk to
	 * as low (perhaps ideally) to a below 2% we're making it 30% chance of
	 * getting HIV from mother; mother to child transmission (MTCT)
	 */	
	public boolean motherToChildAIDSTransfer(Agent mother, Agent child) {
		double chance = 0;
		if (mother.hasHIV()) {
			if (this.clinic.getPatients().contains(mother)) {
				chance = this.clinic.returnChanceOfMTC(this.clinic.returnProfile(mother));
			}
			else {
				chance = Settings.MTCT_RATE;
			}
		}	
		return Math.random() < chance ? true : false;				
	}
	
	/**
	 * Create moieties in the village
	 *
	 */
	public void createHouseholdClusters() {
		SimUtilities.shuffle(householdList);
		int totalHouses = householdList.size();
		int num = totalHouses / clusterProportion, count = 0;
		ArrayList<Household> tempHouses = new ArrayList<Household>(
				householdList);
		for (int i = 0; i < num; i++) {
			Cluster cluster = new Cluster(i);
			while ((count < clusterProportion) && (!tempHouses.isEmpty())) {
				Household household = (Household) tempHouses.get(0);
				cluster.addHousehold(household);
				tempHouses.remove(0);
				count++;
			}
			count = 0;
			populateRelatives(cluster);
		}
	}

	/**
	 * Populate extended families in the given moiety
	 * @param cluster
	 */
	public void populateRelatives(Cluster cluster) {
		for (Household household : cluster.households) {
			for (Household relative : cluster.households) {
				if (household.getId() != relative.getId()) {
					household.getExtendedFamilyIDList().add(new Integer(relative.getId()));
				}
			}
		}
	}

	/**
	 * Creates layouts and displays for the networks
	 *
	 */
	public void buildDisplay() {
		int xSize = 1280, ySize = 1024;
		householdGraphLayout = new CircularGraphLayout(gui
				.getHouseholdNodeList(), xSize, ySize);
		agentFriendsGraphLayout = new PositionPreservingGraphLayout(gui
				.getAgentFriendsNodeList(), xSize, ySize);
		savingsGraphLayout = new PositionPreservingGraphLayout(gui
				.getSavingsNodeList(), xSize, ySize);
		extendedFamilyGraphLayout = new CircularGraphLayout(gui
				.getExtendedFamilyNodeList(), xSize, ySize);
		householdDisplayNetwork = new Network2DDisplay(householdGraphLayout);
		agentDisplayNetwork = new Network2DDisplay(agentFriendsGraphLayout);
		savingsDisplayNetwork = new Network2DDisplay(savingsGraphLayout);
		extendedFamilyDisplayNetwork = new Network2DDisplay(
				extendedFamilyGraphLayout);
		updateGraphLayoutAll();
		surfaceSettingsAll();
	}

	/**
	 * Initial action called before the first time step
	 *
	 */
	public void initialAction() {
		SimUtilities.shuffle(this.agentList);
		for (Agent agent : this.agentList) {
			agent.initializeFriends();
		}
		SimUtilities.shuffle(this.agentList);	
		for (Agent agent : this.agentList) {
			gui.makeFriendsLinks(agent);
		}
		randomMeeting();
		neighborMeeting();
		initializeHIV();
		initializeMigrants();		
		createFuneralClubs();
		updateGraphLayoutAll();
		SimUtilities.shuffle(this.householdList);
	}

	/** 
	 * Agents aare assigned HIV infection at the start
	 */
	public void initializeHIV () {
		SimUtilities.shuffle(this.agentList);
		for (Agent agent : this.agentList) {
			if (aidSpread.returnAgentPrevalence(agent)) {
				if (!this.hivPatients.contains(agent)) {					
					setInitalizeHIVInfection(agent);
				}				
			}
		}
	}

	/**
	 * Function to determin an agents' HIV->AIDS stage
	 * @param agent
	 */
	private void setInitalizeHIVInfection(Agent agent) {
		//Pinkerton
		double r = Math.random();
		/*		if (r < 0.08) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.EARLY_HIV);						
		}
		else if (r >= 0.08 && r < 0.16) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.MIDDLE_HIV);
		}
		else if (r >= 0.16 && r < 0.768) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.LATE_HIV);
		}
		else if (r >= 0.768) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.AIDS);
		}*/

		//adpated from Wawer
		if (r < 0.024) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.EARLY_HIV);						
		}
		else if (r >= 0.024 && r < 0.096) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.MIDDLE_HIV);
		}
		else if (r >= 0.096 && r < 0.768) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.LATE_HIV);
		}
		else if (r >= 0.768) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.AIDS);
		}

		/*if (r < 0.2) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.EARLY_HIV);						
		}
		else if (r >= 0.2 && r < 0.5) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.MIDDLE_HIV);
		}
		else if (r >= 0.5 && r < 0.85) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.LATE_HIV);
		}
		else if (r >= 0.85) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.AIDS);
		}*/

		agent.setInfectedTick(currentTick);
		this.hivPatients.add(agent);
		switch (agent.getHealthStatus()) {
		case Settings.HEALTH_STATUS.EARLY_HIV:
			agent.setDeathTick(currentTick 
					+ Uniform.staticNextIntFromTo(1, 3)
					+ agent.getInfectedLag() 
					+ Uniform.staticNextIntFromTo(10, 13));							
			break;
		case Settings.HEALTH_STATUS.MIDDLE_HIV:
			agent.setMidLag(Uniform.staticNextIntFromTo(1, 3));
			agent.setDeathTick(currentTick
					+ agent.getInfectedLag() 
					+ Uniform.staticNextIntFromTo(10, 13));								
			break;
		case Settings.HEALTH_STATUS.LATE_HIV:
			agent.setLateLag(Uniform.staticNextIntFromTo(4, 15));
			int lag = agent.getInfectedLag() - agent.getLateLag();
			agent.setDeathTick(currentTick
					+ lag 
					+ Uniform.staticNextIntFromTo(10, 13));
			break;
		case Settings.HEALTH_STATUS.AIDS:
			agent.setDeathTick(Uniform.staticNextIntFromTo(10, 13));
			break;
		default: break;			
		}
	}

	/**
	 * Initializes some agents with migration
	 *
	 */
	public void initializeMigrants() {
		int countMigration = (int)(Settings.MIGRATION.PROPORTION_INITIAL_MIGRANTS
				* this.agentList.size());
		SimUtilities.shuffle(this.agentList);
		for (Agent agent : this.agentList) {
			if (countMigration == 0) {
				break;
			}
			//magic number
			if ((Math.random() < 0.45 && agent.getGender() == Settings.GENDER.MALE)
					|| (Math.random() < 0.05 && agent.getGender() == Settings.GENDER.FEMALE)) {
//				if (agent.getAgeGroup() != Settings.AGE_GROUP.CHILD) {
				if (agent.getAgeGroup() == Settings.AGE_GROUP.ADULT) {
					agent.setMigrated(true);
					this.migratedAgents.add(agent);
					countMigration--;					
				}
			}

			/*if ((Math.random() < 0.45 && agent.getGender() == Settings.GENDER.MALE)) {
				if (agent.getAgeGroup() != Settings.AGE_GROUP.CHILD) {
					agent.setMigrated(true);
					this.migratedAgents.add(agent);
					countMigration--;					
				}
			}	*/
		}		
	}

	/** 
	 * function for spread of HIV at the current month
	 */
	public void AIDS() {
		migratoryIncidence();
/*		if (currentTick > 60) {			
			migratoryIncidence();
		}
*/		sexualHIVTransmission();
		printAIDSProfile();	
	}
	
	
	/**
	 * Migration of agents occur; also, agents visit home 
	 * and resume migration again.
	 */
	public void migration() {
		ArrayList<Agent> temp = new ArrayList<Agent>(this.migratedAgents);
		for (Agent agent : temp) {
			if (agent.stopMigration(currentTick)) {
				this.migratedAgents.remove(agent);
			}
		}
		for (Agent agent : this.agentList) {
			agent.attemptMigration(currentTick);
		}		
		/* shah Jan032008 - right now, agents visit for only a month
		 * in RM's they take a random duration of (1-8 wks)
		 */
		for (Agent agent : this.migratedAgents) {
			agent.visitHome(currentTick);
			if (agent.isOnVisit()) {
				agent.resumeMigration(currentTick);
			}
		}				
	}

	/**
	 * Application of the agents and the outsiders for mine jobs
	 *
	 */
	public void mineJobApplication() {
		if (currentTick < mine.getStartTick()) {
			return;
		}
		ArrayList<Agent> potentialApplicants = new ArrayList<Agent>();		
		for (Agent agent : this.agentList) {
			//there is an assumption that only males work at 
			// the mines
			if (agent.getGender() == Settings.GENDER.MALE
					&& agent.getAgeGroup() == Settings.AGE_GROUP.ADULT
					//reusing the code
					&& agent.availableAtVillage()
					&& agent.isLaborFit()
					&& !agent.isMineJob()) {
				potentialApplicants.add(agent);
			}			
		}
		for (Agent outsider : this.outsidersList) {
			if (outsider.getGender() == Settings.GENDER.MALE
					&& outsider.getAgeGroup() == Settings.AGE_GROUP.ADULT
					&& outsider.isLaborFit()
					&& !outsider.isMineJob()
					) {
				potentialApplicants.add(outsider);
			}
			
		}
				
		SimUtilities.shuffle(potentialApplicants);		
		mine.handleJobApplications(currentTick, potentialApplicants);	
	}
	/**
	 * Controls the start/stop of the mine.
	 *
	 */
	public void mining() {
		//remove mine after 20 years
		if (currentTick == mine.getStopTick()) {
				mine.stop();
		}
		
		else if (currentTick == mine.getStartTick()) {
			mine.setFunctioning(true);
		}

		if (mine.isFunctioning()
				&& currentTick % 6 == 0
				) {
			mineJobApplication();
		}
			
/*			if (currentTick > mine.getTrainingStartTick()) {
				mine.laborTraining();
			}			*/
	
	}

	/** main action of the simulation */
	public void mainAction() {		
		refreshVariables();
		currentTick = (int) getTickCount();
		SimUtilities.shuffle(this.agentList);
		SimUtilities.shuffle(this.householdList);
		if ((currentTick % 12 == 0) || (currentTick == 1)) {
			double size = (double) this.agentList.size();
			this.numBirthsThisYear = (int)((Settings.BIRTH_RATE * size)/1000.0);
			this.numInfantsDeathThisYear = (int)((Settings.INFANT_MORTALITY_RATE 
					* ((double) this.numBirthsLastYear))/1000.0);			
			this.numBirthsLastYear = 0;			
		}
		updateAgents();
		updateOutsiderAgents();
		updateHouseholds();
		updateGraphLayoutAll();

		if (mine.isFunctioning()) {
			createOutsiders();
		}

		mining();
		
		migration();
		pieceworkJobs();
		// allocate grant
		agency.step();	

		if (HIV) {
			AIDS();
		}

		for (Household household : this.householdList) {
			household.step();
			if (household.isMemberFuneralClub()) {
				this.numHHFuneralClubMembers++;
			}
		}

		householdBorrowFood();
		stokvels();
		updateFriendshipNetwork();
		updateCouples();
		searchPartner();		
		marriages();	
		newHouses();		
		pregnancyFemale();
		birth();

		schedule.scheduleActionAt(Settings.NUM_RECORDS - 1,	this, "pause", Schedule.LAST);
		if (currentTick % 100 == 0) {
			System.gc();
		}
		if (showGui) {
			updateDisplayAll();
		}

		calculateUtilization();
		calcuDescStat();
		if (currentTick % networkRecordInterval == 0 && Settings.RECORD_NETWORK) {
//			recordNetworks();
//			recordSexDegree("sexDegreeDist-" + currentTick);
//			sexualNetworkPajek("sex-" + currentTick);
			sexualNetworkPajekAIDS("sexAIDS-" + currentTick);
		}	
	}

	/**
	 * Based on the Repast implementation for JinGirNew Model
	 */
	public void updateFriendshipNetwork() {
		randomMeeting();
		neighborMeeting();
		removeRandomFriendship();
	}

	/**
	 * Creating of new stokvels and also payment of the dues for
	 * the existing ones.
	 *
	 */
	public void stokvels() {
		stokvelPostOffice();
		ArrayList<SavingsClub> temp = new ArrayList<SavingsClub>(this.savingsClubs);
		for (SavingsClub club : temp) {
			if (club.isViable()) {
				club.step();
				numHHStokvelMembers += club.getMembers().size();
			}
			else {
				this.savingsClubs.remove(club);
			}
		}
	}
	
	/**
	 * Searching for a sexual partners by the male agents
	 * to the potential female agents.
	 *
	 */
	public void searchPartner() {		
		SimUtilities.shuffle(this.agentList);			
		for (Agent agent : this.agentList) {
			agent.updateAspirationLevel();
			agent.getProposers().clear();
			agent.getPotentialFemales().clear();
			agent.setPotentialPartner(null);
			agent.getSexualPartnershipMessageInbox().clear();
			agent.getSexualPartnershipMessageOutbox().clear();
		}		
		ArrayList<Agent> males = new ArrayList<Agent>(), females = new ArrayList<Agent>();
		for (Agent agent : this.agentList) {
			if (agent.getGender() == Settings.GENDER.MALE
					&& agent.getAge() >= Settings.CHILD_AGE_LIMIT
					&& agent.getAge() <= Settings.MALE_SEX_AGE_LIMIT
					&& agent.getSexualPartners().size() < agent.getMaxSexualPartners()) {
				males.add(agent);
			}
			else if (agent.getGender() == Settings.GENDER.FEMALE
					&& agent.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
					&& agent.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
					&& agent.getMaritalStatus() != Settings.MARITAL_STATUS.MARRIED
					&& agent.getSexualPartners().size() < agent.getMaxSexualPartners()) {
				females.add(agent);
			}
		}	
		
		for (Agent outsider : this.outsidersList) {
			if (outsider.getSexualPartners().size() < outsider.getMaxSexualPartners()
					&& 
					//magic number 
					Math.random() < 0.05
					) {
				males.add(outsider);
			}
		}
		
		SimUtilities.shuffle(males);
		SimUtilities.shuffle(females);	
		for (Agent male : males) {
			male.processSexualPartnershipOutbox(females);
			if (!male.getSexualPartnershipMessageOutbox().isEmpty()) {
				this.partnershipMessageOutbox.addAll(male.getSexualPartnershipMessageOutbox());
				male.getSexualPartnershipMessageOutbox().clear();
			}
		}

		do {
			ArrayList<Agent> receivers = new ArrayList<Agent>();
			for (Message message : this.partnershipMessageOutbox) {
				Agent receiver = (Agent) message.getReceiver();
				receiver.getSexualPartnershipMessageInbox().add(message);
				if (!receivers.contains(receiver)) {
					receivers.add(receiver);	
				}				
			}
			this.partnershipMessageOutbox.clear();
			for (Agent agent : receivers) {
				agent.processSexualPartnershipInbox();
			}    		
		} while (!this.partnershipMessageOutbox.isEmpty());

		for (Agent female : females) {
			if (female.getPotentialPartner() != null) {
				Agent male = female.getPotentialPartner();
				if ((male != null && male.getSexualPartners().size() <= male.getMaxSexualPartners())
						&& (female != null && male.getSexualPartners().size() <= female.getMaxSexualPartners())) {
					this.partnersList.add(makePartners(female.getPotentialPartner(), female));		
				}

			}
		}
	}

	/**
	 * Makes a unmarried couple for the male and the female agent
	 * @param male
	 * @param female
	 * @return
	 */
	public Couple makePartners(Agent male, Agent female) {
		Couple couple = new Couple(this.lastCoupleID, male,	female);
		couple.setCoupleBeginTick((int)getTickCount());		
		couple.setNumMonthsDating(1);		
		this.lastCoupleID++;	
		return couple;
	}

	/**
	 * Checks if the couple should breakup or should marry.
	 *
	 */
	public void updateCouples() {
		ArrayList<Couple> temp = new ArrayList<Couple>(this.partnersList);
		for (Couple couple : temp) {
			if (!couple.getMale().getSexualPartners().contains(couple.getFemale())
					|| (!couple.getFemale().getSexualPartners().contains(couple.getMale()))) {
				if (couple.getFemale().isAlive()
						&& couple.getFemale().isPregnant()
						&& couple.getMale().getId() == couple.getFemale().getFatherOfMyChildID()) {
					couple.getFemale().setSingleMother(true);				
				}
				removeCouple(couple);
			}
		}
		temp = new ArrayList<Couple>(this.partnersList);
		SimUtilities.shuffle(temp);		
		for (Couple couple : temp) {
			if (!couple.isMarried()) {
				couple.update();	
			}			
			if (couple.isBroken() 
					|| !couple.exists()) {
				if (couple.getFemale().isAlive()
						&& couple.getFemale().isPregnant()
						&& couple.getMale().getId() == couple.getFemale().getFatherOfMyChildID()) {
					couple.getFemale().setSingleMother(true);					
				}
				removeCouple(couple);
			}  
		}
	}

	/**
	 * Removes the unmarried couple.
	 * @param couple
	 */
	public void removeCouple(Couple couple) {
		couple.getFemale().getSexualPartners().remove(couple.getMale());
		couple.getMale().getSexualPartners().remove(couple.getFemale());
		this.partnersList.remove(couple);		
	}

	/**
	 * Handles brith to pregnant female agents for whom birth is now possible
	 *
	 */
	public void birth() {				
		ArrayList<Agent> temp = new ArrayList<Agent>();		
		for (Agent agent : this.agentList) {
			if (agent.getGender() == Settings.GENDER.FEMALE
					&& agent.isAlive()
					&& agent.birthPossible(currentTick)) {
				temp.add(agent);
			}
		}
		SimUtilities.shuffle(temp);
		for (Agent female : temp) {
			if (this.numBirthsThisYear <= 0) {
				break;
			}						
			if (female.isSingleMother()) {
				birthToSingleMother(female);
			}
			else {
				Couple couple = (Couple) returnBirthCouple(female);
				if (couple != null) {
					childBornToCouple(couple);	
				}

			}			
		}	
	}
	
	/**
	 * Returns the female which will give birth
	 * @param female
	 * @return
	 */
	public Couple returnBirthCouple(Agent female) {
		Couple myCouple = null;		
		if (female.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED
				&& female.getSpouse() != null
		) {
			Household husbandHousehold = (Household) female.getSpouse().getHousehold();
			for (Couple couple : husbandHousehold.getCoupleList()) {
				if (couple.getMale().getId() == female.getSpouse().getId()
						&& couple.getFemale().getId() == female.getId()) {
					myCouple = couple;
					break;
				}				
			}
			return myCouple;
		}	
		SimUtilities.shuffle(this.partnersList);		
		for (Couple couple : this.partnersList) {
			if (couple.getFemale().getId() == female.getId()
					&& couple.getMale().getId() == female.getFatherOfMyChildID()) {
				myCouple = couple;				
				break;		
			}
		}		
		if (myCouple != null) {
			if (myCouple.exists()) {
				return myCouple;	
			}
		}				
		return null;
	}

	/**
	 * Birth to a single mother who is not linked with the father of the new born anymore.
	 * @param mother
	 */
	public void birthToSingleMother(Agent mother) {
		this.lastAgentID++;
		Household household = mother.getHousehold();	
		createChildAgent(this.lastAgentID, mother, null, household);	
//		vulnerable children
		if (mother.hasHIV()) {
			this.numChildrenBornHIVParents++;	
		}		
	}

	/**
	 * Returns the couple for the female that gets pregnant at this step.
	 * @param female
	 * @return
	 */
	public Couple getPregnancyCouple(Agent female) {
		Couple myCouple = null;
		if (female.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED
				&& female.getSpouse() != null
		) {
			Household husbandHousehold = (Household) female.getSpouse().getHousehold();
			for (Couple couple : husbandHousehold.getCoupleList()) {
				if (couple.getMale().getId() == female.getSpouse().getId()
						&& couple.getFemale().getId() == female.getId()) {
					myCouple = couple;
					break;
				}				
			}
			return myCouple;
		}	
		SimUtilities.shuffle(this.partnersList);		
		for (Couple couple : this.partnersList) {
			if (couple.getFemale().getId() == female.getId()) {				
				if ((couple.getMale().isMigrated() && couple.getMale().isOnVisit())
						|| !couple.getMale().isMigrated()) {
					myCouple = couple;
					break;
				}		
			}
		}		
		if (myCouple != null) {
			if (myCouple.exists()) {
				return myCouple;	
			}
		}				
		return null;		
	}

	/**
	 * Female agents who can get pregnant are filtered out. 
	 *
	 */
	public void pregnancyFemale() {		
		ArrayList<Agent> temp = new ArrayList<Agent>();		
		for (Agent agent : this.agentList) {
			if (agent.getAgeGroup() == Settings.AGE_GROUP.ADULT
					&& agent.getGender() == Settings.GENDER.FEMALE) {
				temp.add(agent);
			}
		}
		SimUtilities.shuffle(temp);
		for (Agent female : temp) {	
			if (female.pregnancyPossible(currentTick)) {				 
				//need to adjust agents' visiting home 
				//for the deomgraphic output
				//try for making migration difficutl			
				Couple couple = getPregnancyCouple(female);
				if (couple != null
//						&& ((couple.getMale().isMigrated() && couple.getMale().isOnVisit())
//						|| !couple.getMale().isMigrated())
				) {
					female.makePregnanct(currentTick);
					female.setFatherOfMyChildID(couple.getMale().getId());
				}				
			}			
		}		
	}

	/**
	 * Marriage occurs for an existing unmarried couple. 
	 *
	 */
	public void marriages() {
		SimUtilities.shuffle(this.partnersList);
		ArrayList<Couple> temp = new ArrayList<Couple>(this.partnersList);
		ArrayList<Couple> marriedCouples = new ArrayList<Couple>();
		for (Couple couple : temp) {
			if (	couple.shouldMarry()
					&& couple.getMale().getHousehold().hasLebola()
					&& Math.random() < Settings.MARRIAGE_PROBABILITY					
			) {
				couple.getMale().setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
				couple.getFemale().setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
				couple.setMarried(true);
				marriedCouples.add(couple);
			}
		}			
		for (Couple marriedCouple : marriedCouples) {
			marriage(marriedCouple);
			this.partnersList.remove(marriedCouple);
		}
	}

	/** 
	 * Create new house for the couple.
	 *
	 */
	public void newHouses() {
		ArrayList<Household> temp = new ArrayList<Household>(this.householdList);
		for (Household household : temp) {
			ArrayList<Couple> temp2 = new ArrayList<Couple>(household.getCoupleList());
			for (Couple couple : temp2) {
				if (newHousePossible(household, couple)) {
					createNewHouse(couple);
				}
			}
		}		
	}

	/**
	 * Households offering piecewise jobs
	 *
	 */
	public void pieceworkJobs() {
		//households issuing piecework
		this.currentEmployers.clear();		
		for (Household household : this.householdList) {
			if (household.announceJobs()) {
				currentEmployers.add(household);
			}
		}		

		SimUtilities.shuffle(this.agentList);
		for (Agent agent : this.agentList) {
			if (!this.currentEmployers.contains(agent.getHousehold())
					&& agent.isLaborFit() 
					&& !agent.isMigrated()
					&& !agent.isMineJob()) {
//				magic number
//				&& Math.random() < 0.35) {
//				agent.getPieceWork(currentTick);
//				magic number INSIDE
				agent.applyForPieceWork(currentTick);
			}			
		}
		SimUtilities.shuffle(this.currentEmployers);
		for (Household household : this.currentEmployers) {
			household.allocateJob();
		}
	}

	/** Using child birth rate, infant mortality rate,
	 * and maternal mortality rate for child birth 
	 */

	public void childBornToCouple(Couple couple) {
		this.lastAgentID++;
		Household household;
		if (couple.isMarried()) {
			household = (Household) couple.getMale().getHousehold();
		}
		else {
			household = (Household) couple.getFemale().getHousehold();
		}
		createChildAgent(this.lastAgentID, couple.getFemale(), couple.getMale(), household);
		couple.setLastBirthTick((int) getTickCount());
//		vulnerable children
		if (couple.getFemale().hasHIV() || (couple.getMale().hasHIV())) {
			this.numChildrenBornHIVParents++;	
		}
	}
	
	/**
	 * Infects the agent with the HIV infection and its stage
	 * @param agent
	 */
	public void setHIVInfection(Agent agent) {
		//sets the current month as the agent's infected month
		if (!this.hivPatients.contains(agent) 
				&& !agent.hasHIV()) {
			agent.setHealthStatus(Settings.HEALTH_STATUS.EARLY_HIV);
			agent.setInfectedTick(currentTick);		
//			agent.setInfectedTick(Uniform.staticNextIntFromTo(currentTick, currentTick+5));
			/*			agent.setDeathTick(currentTick 
					+ Uniform.staticNextIntFromTo(1, 3)
					+ agent.getInfectedLag() 
					+ Uniform.staticNextIntFromTo(10, 13));					
			 */			
			agent.setDeathTick(currentTick 
					+ Uniform.staticNextIntFromTo(1, 3)
					+ agent.getInfectedLag() 
					+ Uniform.staticNextIntFromTo(10, 13));
			this.hivPatients.add(agent);
			if (agent.isOutsider()) {
				hivOutsiderPatients.add(agent);
			}
		}				
	}

	/**
	 * Incidence to the migrant workers is introduced this month.
	 *
	 */
	public void migratoryIncidence() {					
		if (mine.isFunctioning()) {
			proportionPatients = (int) weibull.returnIncidenceMine();
		}
		else {
			proportionPatients = (int) weibull.returnIncidence();
		}
		if (proportionPatients == 0) {
			return; 
		}
//		SimUtilities.shuffle(agentList);
		int count = 0;	
/*		SimUtilities.shuffle(this.migratedAgents);
		for (Agent agent : this.migratedAgents) {
			if (count >= proportionPatients) {
				break;
			}
			if (!agent.isOnVisit()
					&& !agent.hasHIV()) {
				int num = Uniform.staticNextIntFromTo(7, 13);
				if (Math.random() < 0.001*num) {
					setHIVInfection(agent);
					this.numIncidenceThisYear++;
					count++;
				}				
			}
		}*/
		
		ArrayList<Agent> temp = new ArrayList<Agent>(this.migratedAgents);
		temp.addAll(this.outsidersList);
		SimUtilities.shuffle(temp);
		for (Agent agent : temp) {
			if (count >= proportionPatients) {
				break;
			}			
			if (!agent.isOnVisit()
					&& !agent.hasHIV()) {
				int num = Uniform.staticNextIntFromTo(7, 13);
				if (Math.random() < 0.001*num) {
					setHIVInfection(agent);
					this.numIncidenceThisYear++;
					count++;
				}				
			}
		}
	}

	/**
	 * Possibility of sexual transmission of HIV among the couples.
	 *
	 */
	public void sexualHIVTransmission() {
		SimUtilities.shuffle(this.hivPatients);
		ArrayList<Agent> temp = new ArrayList<Agent>(this.hivPatients);
		for (Agent hivPatient : temp) {
			for (Agent sexPartner : hivPatient.getSexualPartners()) {
				if (!sexPartner.hasHIV()
						&& hivPatient.availableAtVillage() 
						&& sexPartner.availableAtVillage()
				) {
					double coitalActs = Uniform.staticNextDoubleFromTo(7, 13);
					if (Math.random() < coitalActs *
							aidSpread.returnTransmissionProbabilityWawerPinkerton(hivPatient)) {								 
//							aidSpread.returnTransmissionProbabilityWawer(hivPatient)) {
						setHIVInfection(sexPartner);
					}
				}				 
			}			
		}
	}

	/**
	 * currently, this is women specific treatment to prevent Mother-to-Child-Transfer...
	 * in the the next stage, we'll extend that to all males and females 
	 * receiving treatments
	 */
	public void femaleTreatment() {		
		for (Agent patient : this.hivPatients) {
			if (patient.getGender() == Settings.GENDER.FEMALE
					&& patient.getInfectedTick() == currentTick) {
			}
			patient.hivTreatment(currentTick);
		}
	}

	/**
	 * Marriage means the female in the couple moves to the male's house
	 * and the lobola is paid.
	 * @param couple
	 */
	public void marriage(Couple couple) {		 
		Agent husband = couple.getMale(), wife = couple.getFemale();
		Household husbandHousehold = (Household) husband.getHousehold();
		Household wifeHousehold = (Household) wife.getHousehold();

		int lobola = Uniform.staticNextIntFromTo(Settings.LOBOLA_AMOUNT/2, Settings.LOBOLA_AMOUNT);
		husbandHousehold.deductWealth((double)lobola);
		wifeHousehold.addWealth((double)lobola);
		int id = couple.getId();
		wife.resetMyCouples();
		if (!wife.getSexualPartners().contains(husband)) {
			wife.getSexualPartners().add(husband);
		}
		if (!husband.getSexualPartners().contains(wife)) {
			husband.getSexualPartners().add(wife);	 
		}
		husband.removeFriend(wife);
		husband.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
		husband.setSpouseID(wife.getId());
		wife.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
		wife.setSpouseID(husband.getId());
		husbandHousehold.addAgent(wife);
		wifeHousehold.removeAgent(wife);
		husbandHousehold.getExtendedFamilyIDList().add(wifeHousehold.getId());
		wifeHousehold.getExtendedFamilyIDList().add(husbandHousehold.getId());
		husbandHousehold.addCouple(new Couple(id, husband, wife));
		ArrayList<Agent> temp = new ArrayList<Agent> (wife.getOffSprings());
		for (Agent child : temp) {
			if (child.getMotherID() == wife.getId()) {
				husbandHousehold.addAgent(child);
				wifeHousehold.removeAgent(child);
			}
		}
	}

	/** Checks if a new house needs and can be created or not */
	public boolean newHousePossible(Household household, Couple couple) {
		if (	 
//				!couple.getMale().isYoungestSon()
//				&&  
				couple.getFemale().getAgeGroup() == Settings.AGE_GROUP.ADULT
				&& couple.getMale().getAgeGroup() == Settings.AGE_GROUP.ADULT
				//magic number
//				&& household.getWealth() > 5 * Settings.MAX_STARTING_WEALTH				 
//				&& household.getWealth() > 3 * Settings.MAX_STARTING_WEALTH
				&& household.getWealth() > Settings.MAX_STARTING_WEALTH
//				magic number
				&& household.getCoupleList().size() > 1
//				//magic number
				&& household.getCurrentMembers().size() > 7
		) {
			return true;
		}
		return false;
	}

	/**
	 * Creates a new house for a couple/nuclear family
	 * @param couple
	 */
	public void createNewHouse(Couple couple) {
		Agent husband = couple.getMale();
		Agent wife = couple.getFemale();
		Household husbandHousehold = (Household) husband.getHousehold();
		Household wifeHousehold = (Household) wife.getHousehold();	 
		this.lastHouseholdID++;
		Household household = new Household(this.lastHouseholdID, this);
		household.addAgent(wife);
		household.addAgent(husband);		 
		if (Random.uniform.nextDouble() < Settings.Church.CHURCH_PARTICIPATION_RATE) {
			if (wifeHousehold.getChurchID() != husbandHousehold.getChurchID()) {
				//adapted from RADAR
				if (Math.random() < 0.70) {
					household.setChurchID(husbandHousehold.getChurchID());
				}
				else {
					household.setChurchID(wifeHousehold.getChurchID());
				}
			}
			else {
				household.setChurchID(husbandHousehold.getChurchID());
			}
		}		 
		churches.get(household.getChurchID()).add(household);		 
		// initialize households
		household.addCouple(couple);
		household.initialize();
		husbandHousehold.removeCouple(couple);
		husbandHousehold.removeAgent(husband);
		husbandHousehold.removeAgent(wife);
		ArrayList<Agent> temp = new ArrayList<Agent>();
		for (Agent child : temp) {
			if (child.getFatherID() == couple.getMale().getId()
					|| child.getMotherID() == couple.getFemale().getId()) {
				household.addAgent(child);
				husbandHousehold.removeAgent(child);
			}
		}
		householdList.add(household);
		this.houseMap.put(new Integer(household.getId()), household);
		household.populateExtendedFamily(husbandHousehold, wifeHousehold);
		// now add to the gui - network ... and inherit all the links of all
		this.gui.inheritSocialLinks(household, husbandHousehold, wifeHousehold);
		/*if (this.householdList.contains(wifeHousehold)) {
			this.gui.inheritLinks(household, husbandHousehold, wifeHousehold);
		} else {
			this.gui.inheritLinks(household, husbandHousehold, null);
		}*/

		// household.addWealth(Settings.MIN_STARTING_WEALTH);
		// int wealth =
		// Random.uniform.nextIntFromTo(Settings.MIN_STARTING_WEALTH,
		// Settings.MAX_STARTING_WEALTH);
		//magic number
		double wealth = Math.min(husbandHousehold.getWealth() / 2,
				Settings.MIN_STARTING_WEALTH);
		household.addWealth(wealth);
		husbandHousehold.deductWealth(wealth);
		this.gui.addExtendedFamilyNode(household);
		updateGraphLayoutAll();
	}

	/**
	 * Purge the household; dissolve it and accommodate orphans, if any.
	 * @param household
	 */
	public void purgeHousehold(Household household) {
		//recording
		this.numHHDissolved++;
		if (household.getMembers().isEmpty() || household == null) {
			removeHousehold(household);	 			 
			return;
		}
		Household accommodator = (Household) household.returnAccommodator();
		if (accommodator != null) {		
//			HouseholdNode node = (HouseholdNode) this.gui.getHouseholdNode(household);
			HouseholdNode accommodatorNode = (HouseholdNode) this.gui.getHouseholdNode(accommodator);
			if (!getHouseholdGraphLayout().getNodeList().contains(accommodatorNode)) {
				return;
			}
			accommodator.accommodateNewMembers(household);
			accommodatorNode.setColor(Color.BLUE);
			updateGraphLayoutAll();
			removeHousehold(household);
			//recording
			this.numHHAccommodated++;
		}
		// we couldn't find a neighbour or an extended family
		else if (accommodator == null) {
			if (!unAccommodated(household)) {				
				agentsOnStreetCase(household);
				removeHousehold(household);
			}
		}		
	}
	
	/**
	 * Handle agents who do not have a household to live anymore
	 * @param household
	 */
	public void agentsOnStreetCase(Household household) {
		ArrayList<Agent> temp = new ArrayList<Agent>(household.getMembers());
		for (Agent agent : temp) {
			// streetAgentList.add(agent);
			numAgentsOnStreet++;
			purgeAgent(agent);
		}
	}

	/**
	 * Handles unaccommodated household
	 * @param household
	 * @return
	 */
	public boolean unAccommodated(Household household) {
		for (Agent agent : household.getMembers()) {
			if (this.migratedAgents.contains(agent)
					&& household.getMembers().contains(agent)) {
				agent.setMigrated(false);
				household.setHead(agent);
				this.migratedAgents.remove(agent);
				print(" -------- migrated agent back ---------");
				return true;
			}
		}
		for (Agent agent : household.getMembers()) {
			// magic number
			if (agent.getAge() > 10
					&& agent.getAgeGroup() == Settings.AGE_GROUP.CHILD) {
				household.setHead(agent);
				print(" -------- Child household head ---------");
				numChildHH++;
				return true;
			}
		}
		return false;
	}

	/**
	 * Removes the household from the networks and data structures.
	 * @param household
	 */
	public void removeHousehold(Household household) {
		// remove from the gui - node and network ...
		this.gui.removeHousholdNode(household);
		householdList.remove(household);
		this.houseMap.remove(household);
		updateGraphLayoutAll();
	}

	/**
	 * Purge the agent from the simulation, once it is dead.
	 * @param agent
	 */
	public void purgeAgent(Agent agent) {
		if (this.hivPatients.contains(agent)) {
			this.hivPatients.remove(agent);			
		}	 		 	 
		Household household = (Household) agent.getHousehold();
		/*
		 * this gets more complex when an adult dies because we then might have
		 * to move the household to another if there's no more adult left ...
		 */ 
		if (this.migratedAgents.contains(agent)) {
			this.migratedAgents.remove(agent);
		}
		if (agent.isMineJob()) {
			mine.removeMineWorker(agent);
		}
		agent.clearAllFriends();
//		agent.clearAllSexualPartners();		
		Agent spouse = agentMap.get(new Integer(agent.getSpouseID()));
		if (spouse != null) {
			spouse.getSexualPartners().remove(agent);
			agent.getSexualPartners().remove(spouse);		
		}	
		agent.resetMyCouples();
		household.removeAgent(agent);
		this.gui.removeAgentNode(agent);
		updateGraphLayoutAll();
		this.agentList.remove(agent);
	}
	

	/** this does the bookkeeping actually for the stokvel */
	public void createSavingsClub(SavingsClub club) {
		this.savingsClubs.add(club);
		ArrayList<Agent> temp = new ArrayList<Agent>(club.getMembers());
		for (Agent agent1 : temp) {
			agent1.setMemberSavings(true);
			for (Agent agent2 : temp) {
				if ((agent1.getId() != agent2.getId())
						&& (!agent1.getFriends().contains(agent2))
						&& agent1.getAgeGroup() != Settings.AGE_GROUP.CHILD
						&& agent2.getAgeGroup() != Settings.AGE_GROUP.CHILD
				) {
//					agent1.makeFriend(agent2);
					makeAcquaintances(agent1, agent2);
				}

			}
		}
		// draw edges
		gui.makeSavingsLinks(club);
		updateGraphLayoutAll();
	}
	
	/**
	 * Make agent1 and agent2 as acquaintances to each other.
	 * @param agent1
	 * @param agent2
	 */
	public void makeAcquaintances(Agent agent1, Agent agent2) {
		if (!agent1.getAcquaintances().contains(agent2)) {
			agent1.getAcquaintances().add(agent2);
		}
		if (!agent2.getAcquaintances().contains(agent1)) {
			agent2.getAcquaintances().add(agent1);
		}
	}
	
	/**
	 * Agents who go hungry borrow food from their friends' households
	 *
	 */
	public void borrowFood() {
		// borrow food ...
		int counter = 0;
		for (Agent agent : this.hungryAgents) {
			for (Agent friend : agent.getFriends()) {
				if (agent.askFood(friend) && friend.feedFriend(agent)) {
					agent.setFeedStatus(Settings.FEED_STATUS.HALF_FED);
					counter++;
					break;
				}
			}
		}
	}
	
	/**
	 * Households who go hungry borrow food from their neighbours
	 *
	 */
	public void householdBorrowFood() {
		SimUtilities.shuffle(this.householdList);
		ArrayList<Household> temp = new ArrayList<Household>(this.householdList);
		for (Household household : temp) {
			if (household.isHungry()) {
				household.borrowFood();
			}
		}
	}

	/**
	 * Incorporating an infant mortality rate
	 *using 5% chance for infant mortality ...tested
	 *it's 37 deaths per 1000 infants for Limpopo
	 *source: http://www.hst.org.za/healthstats/7/data/geo
	 *also checking for firth 24 months ... (magic number)
	 */
	public boolean checkChildMortality(Agent agent) {
		if (this.numInfantsDeathThisYear > 0
				&& agent.getAgeGroup() == Settings.AGE_GROUP.CHILD
				&& agent.getAge() <= 1
				&& currentTick - agent.getBirthTick() <= 12) {
			agent.setAlive(false);
			this.numInfantsDeathThisYear--;
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * Updates all agents at each time step. 
	 *
	 */
	public void updateAgents() {
		SimUtilities.shuffle(this.agentList);
		this.dead.clear();
		for (Agent agent : this.agentList) {
			checkChildMortality(agent);
			agent.updateProfile(currentTick);
			if (!agent.isAlive()) {
				dead.add(agent);
				//recording
			}
			else {
				int ageGroup = agent.getAgeGroup();
				if (ageGroup == Settings.AGE_GROUP.CHILD) {
					this.numChildren++;				
				}
				else if (ageGroup == Settings.AGE_GROUP.ADULT) {
					this.numAdults++;				
				} 
				else if (ageGroup == Settings.AGE_GROUP.SENIOR) {
					this.numSeniors++;				
				}			 				
			}
		}	
		numDeathsPerYear += this.dead.size();
		for (Agent agent : this.dead) {
			purgeAgent(agent);
		}
	}

	/**
	 * Updates all outsider agents at each time step. 
	 *
	 */
	public void updateOutsiderAgents() {
		SimUtilities.shuffle(this.outsidersList);		
		ArrayList<Agent> temp = new ArrayList<Agent>(this.outsidersList);
		for (Agent outsider : temp) {
			outsider.updateOutsiderProfile(currentTick);
			if (outsider.getNumMonthsOutsiderArrival() > outsider.getMaxOutsiderWaitingMonths()				
					|| !outsider.isAlive() 
					|| outsider.getAgeGroup() == Settings.AGE_GROUP.SENIOR
					|| !outsider.isLaborFit()					
					) {
				if (outsider.isMineJob()) {
					mine.removeMineWorker(outsider);
				}
				outsider.resetMyCouples();
				this.outsidersList.remove(outsider);
				this.hivPatients.remove(outsider);
				this.hivOutsiderPatients.remove(outsider);
				
			}
		}
	}

	/**
	 * Updates all households at each time step. 
	 *
	 */
	public void updateHouseholds() {
		for (Household hh : this.householdList) {
			if (hh.getHead() != null) {
				if (!hh.getHead().isAlive()) {
					hh.updateHead(hh.getHead());					 
				}
			}					
		}
		ArrayList<Household> noGuardians = new ArrayList<Household>();
		for (Household household : this.householdList) {
			if (household.getNoGuardians() || household.getHead() == null
					|| household.getMembers().size() <= 1) {
				noGuardians.add(household);
			}
		}
		for (Household household : noGuardians) {
			purgeHousehold(household);
		}

	}
	
	public void removeInitialAction() {
		schedule.removeAction(initialAction);
	}

	protected void buildSchedule() {
		initialAction = schedule.scheduleActionAt(0, this, "initialAction");
		schedule.scheduleActionAt(initialSteps, this, "removeInitialAction",
				Schedule.LAST);
		schedule.scheduleActionBeginning(initialSteps + 1, this, "mainAction");
		schedule.scheduleActionAtInterval(1, new BasicAction() {
			public void execute() {

//				recorder2.record();
				recorder3.record();
				recorder4.record();
				recorder5.record();
//				recorder6.record();
			}
		}, Schedule.LAST);
		schedule.scheduleActionAtInterval(12, new BasicAction() {
			public void execute() {				
				recorder.record();
				if (currentTick % 12 == 0) {
					numberBirths = 0;
					numDeathsPerYear = 0;
					numChildrenBornHIVParents = 0;
					numIncidenceThisYear = 0;
				}

			}
		}, Schedule.LAST);
		schedule.scheduleActionAtEnd(new BasicAction() {
			public void execute() {
				recorder.write();
//				recorder2.write();
				recorder3.write();
				recorder4.write();
				recorder5.write();
//				recorder6.write();
//				ks.it();
			}
		});
	}

	public void begin() {
		buildModel();
		gui = new ModelGUI(this);
		buildDisplay();
		buildSchedule();
		if (showGui) {
			displaySurfaceAll();	
		}						
	}

	public void displaySurfaceAll() {
		householdSurface.display();
		agentSurface.display();
		savingsSurface.display();
		extendedFamilySurface.display();
	}

	public void setup() {
		schedule = null;
		System.gc();
		schedule = new Schedule();
		agentList = new ArrayList<Agent>();
		householdList = new ArrayList<Household>();
		Random.createUniform();

		householdSurface = null;
		agentSurface = null;
		savingsSurface = null;
		System.gc();

		householdSurface = new DisplaySurface(this,	"Households Network Surface");
		agentSurface = new DisplaySurface(this, "Agents Network Surface");
		savingsSurface = new DisplaySurface(this, "Savings Club Surface");
		extendedFamilySurface = new DisplaySurface(this, "Extended Family Surface");

		registerDisplaySurface("Household Display", householdSurface);
		this.registerDisplaySurface("Friendship Display", agentSurface);
		this.registerDisplaySurface("Savings Display", savingsSurface);
		this.registerDisplaySurface("Extended Family Display", extendedFamilySurface);
		setInitNumHouseholds(233);
//		initNumHouseholds = 50;
//		initNumHouseholds = 111;
		setInitialFuneralClubs(11);		
//		initialFuneralClubs = 7;
//		initialFuneralClubs = 19;
		setClusterProportion(7);
//		clusterProportion = 11;
		setNetworkRecordInterval(Settings.PAJEK_INTERVAL);
		setHealthExpenditure(Settings.IS_HEALTH_EXPENDITURE);
		setHIV(Settings.IS_HIV);
		setShowGui(Settings.SHOW_GUI);
		Controller.CONSOLE_ERR = false;
		Controller.CONSOLE_OUT = false;
		setRngSeed(new Long(1199995797250l));
	}

	public String[] getInitParam() {
		/*String[] params = { "isFuneralClub, initNumHouseholds, minHouseholdSize,"
				+ "maxHouseholdSize, initialFuneralClubs, clusterProportion, proportionInitialMigrated" };*/
		String[] params = {"initNumHouseholds", "initialFuneralClubs", "clusterProportion",
				"healthExpenditure", "hiv", "showGui",	"networkRecordInterval"				
				};
		return params;
	}

	public Schedule getSchedule() {
		return schedule;
	}

	public String getName() {
		return "Modelling the Socioecnomic Impact of HIV/AIDS in a South African Village";
	}

	public static void main(String[] args) {
		uchicago.src.sim.engine.SimInit init = new uchicago.src.sim.engine.SimInit();
		Model model = new Model();
		init.loadModel(model, null, false);
	}	

	public ArrayList<Agent> getAgentList() {
		return agentList;
	}

	public int getClusterProportion() {
		return clusterProportion;
	}

	public ArrayList<Household> getHouseholdList() {
		return householdList;
	}

	public HashMap getHouseMap() {
		return houseMap;
	}

	public ModelGUI getGui() {
		return gui;
	}

	public HashMap<Integer, Agent> getAgentMap() {
		return agentMap;
	}

	public Network2DDisplay getHouseholdDisplayNetwork() {
		return householdDisplayNetwork;
	}

	public DisplaySurface getHouseholdSurface() {
		return householdSurface;
	}

	public Sigmoid getSig() {
		return sig;
	}

	public ArrayList<FuneralClub> getFuneralClubs() {
		return funeralClubs;
	}

	public ArrayList<SavingsClub> getSavingsClubs() {
		return savingsClubs;
	}

	public int getInitNumHouseholds() {
		return initNumHouseholds;
	}

	public void setInitNumHouseholds(int _initNumHouseholds) {
		this.initNumHouseholds = _initNumHouseholds;
	}

	public boolean isFuneralClub() {
		return Settings.IS_FUNERAL_CLUB;
	}

	public Normal getExpectedAgeNormal() {
		return expectedAgeNormal;
	}

	public class Cluster {
		public int id = -1;
		public ArrayList<Household> households = new ArrayList<Household>();
		Cluster(int _id) {
			this.id = _id;		
		}
		public void addHousehold(Household household) {
			this.households.add(household);
		}
	}

	public AbstractGraphLayout getAgentFriendsGraphLayout() {
		return agentFriendsGraphLayout;
	}

	public void setAgentFriendsGraphLayout(AbstractGraphLayout agentGraphLayout) {
		this.agentFriendsGraphLayout = agentGraphLayout;
	}

	public AbstractGraphLayout getHouseholdGraphLayout() {
		return householdGraphLayout;
	}

	public void setHouseholdGraphLayout(AbstractGraphLayout householdGraphLayout) {
		this.householdGraphLayout = householdGraphLayout;
	}

	/**
	 * Registers a DisplaySurface with this model and associates it with a
	 * particular name.
	 */
	public void registerDisplaySurface(String name, DisplaySurface surface) {
		ProducerNamePair p = new ProducerNamePair(name, surface);
		this.getMediaProducers().add(p);
	}

	public DisplaySurface getAgentSurface() {
		return agentSurface;
	}

	public DisplaySurface getSavingsSurface() {
		return savingsSurface;
	}

	public AbstractGraphLayout getSavingsGraphLayout() {
		return savingsGraphLayout;
	}

	public AbstractGraphLayout getExtendedFamilyGraphLayout() {
		return extendedFamilyGraphLayout;
	}

	/**
	 * Updates all the graph layouts during the simulation.
	 *
	 */
	public void updateGraphLayoutAll() {
		householdGraphLayout.updateLayout();
		agentFriendsGraphLayout.updateLayout();
		savingsGraphLayout.updateLayout();
		extendedFamilyGraphLayout.updateLayout();
	}

	public void surfaceSettingsAll() {
		agentSurface.addDisplayableProbeable(agentDisplayNetwork, "Agent Links");
		agentSurface.setBackground(java.awt.Color.white);
		agentSurface.addZoomable(agentDisplayNetwork);
		addSimEventListener(agentSurface);

		householdSurface.addDisplayableProbeable(householdDisplayNetwork,"Social links of Households");
		householdSurface.setBackground(java.awt.Color.white);
		householdSurface.addZoomable(householdDisplayNetwork);
		addSimEventListener(householdSurface);

		savingsSurface.addDisplayableProbeable(savingsDisplayNetwork,
		"Savings Club Memberships");
		savingsSurface.setBackground(java.awt.Color.white);
		savingsSurface.addZoomable(savingsDisplayNetwork);
		addSimEventListener(savingsSurface);

		extendedFamilySurface.addDisplayableProbeable(
				extendedFamilyDisplayNetwork, "Extended Family Network");
		extendedFamilySurface.setBackground(java.awt.Color.white);
		extendedFamilySurface.addZoomable(extendedFamilyDisplayNetwork);
		addSimEventListener(extendedFamilySurface);
	}

	public void updateDisplayAll() {
		householdSurface.updateDisplay();
		agentSurface.updateDisplay();
		savingsSurface.updateDisplay();
		extendedFamilySurface.updateDisplay();
	}

	public ArrayList<Agent> getHungryAgents() {
		return hungryAgents;
	}

	public ArrayList<Agent> getMigratedAgents() {
		return migratedAgents;
	}

	public int getNumMigratedAgents() {
		int size = migratedAgents.size();
		return size;
	}

	public Fermi getFermi() {
		return fermi;
	}

	public ArrayList<Couple> getPartnersList() {
		return partnersList;
	}

	public int getAIDSPatients() {
		return this.hivPatients.size();
	}

	public int getProportion() {
		return this.proportionPatients; 
	}

	public void record() {
		recorder = new LocalDataRecorder("Population_related-.txt", this);
		recorder.createNumericDataSource("Population", this, "getPopulation");
		recorder.createNumericDataSource("IncidencePerYear", this, "getNumIncidence");
		recorder.createNumericDataSource("BirthsPerYear", this, "getNumberBirths");
		recorder.createNumericDataSource("DeathsPerYear", this, "getNumDeathsPerYear");
//		recorder.createNumericDataSource("numChildrenBornHIVParents", this, "getNumChildrenBornHIVParents");
//		recorder.createNumericDataSource("numHHDissolved", this, "getNumHHDissolved");
//		recorder.createNumericDataSource("numHHAccommodated", this, "getNumHHAccommodated");
//		recorder.createNumericDataSource("numAgentsOnStreet", this, "getNumAgentsOnStreet");
//		recorder.createNumericDataSource("numChildHH", this, "getNumChildHH");

		recorder2 = new LocalDataRecorder("Household_related-.txt", this);
		recorder2.createNumericDataSource("NumHouseholds", this, "getNumHH");
//		recorder2.createNumericDataSource("Percentage_receiving_grants", this, "getProportionHHReceivingGrants");
//		recorder2.createNumericDataSource("Percentage_lacking_food", this, "getProportionHHLackingFood");

		recorder3 = new LocalDataRecorder("Distribution-.txt", this);
		recorder3.createNumericDataSource("femaleMean", this, "getMeanFemaleDegree");
		recorder3.createNumericDataSource("femaleVariance", this, "getVarFemaleDegree");
		recorder3.createNumericDataSource("maleMean", this, "getMeanMaleDegree");
		recorder3.createNumericDataSource("maleVariance", this, "getVarMaleDegree");

		/*		recorder3.createNumericDataSource("numHHFuneralClubMembers", this, "getNumHHFuneralClubMembers");
		recorder3.createNumericDataSource("numHHGracePeriod", this, "getNumHHGracePeriod");
		recorder3.createNumericDataSource("numOfStokvels", this, "getNumStokvels");
		recorder3.createNumericDataSource("numHHStokvelMembers", this, "getNumHHStokvelMembers");
		 */
//		recorder4 = new LocalDataRecorder("Education_Work_Related-.txt", this);
		/*//		recorder4.createNumericDataSource("numChildrenLeavingSchoolsWithoutDegree", this, "");
		recorder4.createNumericDataSource("numMigrants", this, "getNumMigrants");
		recorder4.createNumericDataSource("NumAgentsPiecework", this, "getNumAgentsPieceWork");
//		recorder4.createNumericDataSource("NumAgentsMine", this, "getNumAgentsOnMineJob");
//		recorder4.createNumericDataSource("NumAgentsJobless", this, "getNumAgentsJobless");
//		recorder4.createNumericDataSource("NumAgentsNotLaborFit", this, "getNumAgentsNotLaborFit");
		 */
		recorder5 = new LocalDataRecorder("AIDS_related-.txt", this);
		recorder5.createNumericDataSource("Population", this, "getPopulation");
		recorder5.createNumericDataSource("numHIVPatients", this, "getNumHIVPatients");
		recorder5.createNumericDataSource("Prevalence", this, "getPrevalence");
//		recorder5.createNumericDataSource("numSingles", this, "getNumSingles");
		recorder5.createNumericDataSource("NumHouseholds", this, "getNumHH");
		recorder5.createNumericDataSource("MarrCouples", this, "getMarriedCoupleCounts");		 
		recorder5.createNumericDataSource("NumPartners", this, "getPartners");


		recorder4 = new LocalDataRecorder("Sexually_Active-.txt", this);
		recorder4.createNumericDataSource("#SingleMales", this, "getNumSingleMales");
		recorder4.createNumericDataSource("#SingleFemales", this, "getNumSingleFemales");
		recorder4.createNumericDataSource("#MarriedAgents", this, "getNumMarriedAgents");
		recorder4.createNumericDataSource("#WidowedMales", this, "getNumWidowedMales");
		recorder4.createNumericDataSource("#WidowedFemales", this, "getNumWidowedFemales");
		recorder4.createNumericDataSource("#SexActiveAgents", this, "getNumSexuallyActiveAgents");
		recorder4.createNumericDataSource("#UnpairedAgents", this, "getNumUnPairedIndividuals");
		recorder4.createNumericDataSource("#Dyads", this, "getNumDyads");

//		recorder5.createNumericDataSource("NumAgentsNotLaborFit", this, "getNumAgentsNotLaborFit");
//		recorder5.createNumericDataSource("numChildrenBornHIVParents", this, "getNumChildrenBornHIVParents");
//		recorder5.createNumericDataSource("numHHDissolved", this, "getNumHHDissolved");
//		//includes both relatives and possibly neighbors
//		recorder5.createNumericDataSource("numHHAccommodated", this, "getNumHHAccommodated");
//		recorder5.createNumericDataSource("numAgentsOnStreet", this, "getNumAgentsOnStreet");
//		recorder5.createNumericDataSource("numChildHH", this, "getNumChildHH");

		recorder6 = new LocalDataRecorder("Age_Groups-.txt", this);
		recorder6.createNumericDataSource("numChildren", this, "getNumChildren");
		recorder6.createNumericDataSource("numAdults", this, "getNumAdults");
		recorder6.createNumericDataSource("numSeniors", this, "getNumSeniors");
	}

	public double getPercentage() {
		return (double)this.hivPatients.size()/this.agentList.size()*100;
	}

	public int getIncidence() {
		return this.incidence;
	}

	public Clinic getClinic() {
		return clinic;
	}

	public ArrayList<Household> getCurrentEmployers() {
		return currentEmployers;
	}

	public void setCurrentEmployers(ArrayList<Household> currentEmployers) {
		this.currentEmployers = currentEmployers;
	}

	/**
	 * Function to handle the messages between the agents regarding
	 * creating new stokvels.
	 *
	 */
	public void stokvelPostOffice() {
		/* this can be improved as currently only household heads are processing the messages
	    	but we leave it to the entire list, as we intend to have further agents' interactions
	    	and not just those between the household heads. 
		 */    	
		for (Agent agent : this.agentList) {
			agent.processStokvelOutbox();
			if (!agent.getStovelMessageOutbox().isEmpty()) {
				this.stokvelMessageOutbox.addAll(agent.getStovelMessageOutbox());
				agent.getStovelMessageOutbox().clear();
			}
		}

		do {
			ArrayList<Agent> receivers = new ArrayList<Agent>();
			for (Message message : stokvelMessageOutbox) {
				Agent receiver = (Agent) message.getReceiver();
				receiver.getStovelMessageInbox().add(message);
				receivers.add(receiver);
			}
			this.stokvelMessageOutbox.clear();
			for (Agent agent : receivers) {
				agent.processStokvelInbox();
			}    		
		} while (!this.stokvelMessageOutbox.isEmpty());   	    	

		for (Household household : this.householdList) {
			Agent agent = (Agent) household.getHead();
			if ((agent != null) && (agent.getAcceptances().contains(agent))){
				agent.createStokvel();
			}    		
		}
	}  	   

	/**
	 * author: ruth meyer - assigns education levels
	 */	
	private void assignEducationLevels(){
		// use distributions extracted from RADAR data
		EmpiricalDiscreteDistArray femaleLevels = new EmpiricalDiscreteDistArray(
				new double[]{5,6,13,18,28,37,64,120}, 
				new double[][]{{1,0,0,0,0,0},{0.2,0.8,0,0,0,0},{0.02,0.93,0.02,0.04,0.00,0.00},
						{0.01,0.17,0.12,0.69,0.01,0.00},{0.01,0.06,0.04,0.70,0.12,0.07},
						{0.06,0.19,0.06,0.48,0.16,0.05},{0.45,0.37,0.05,0.10,0.02,0.01},
						{0.93,0.06,0.00,0.01,0.00,0.00}},
						new MersenneTwister(seed));
		EmpiricalDiscreteDistArray maleLevels = new EmpiricalDiscreteDistArray(
				new double[]{5,6,13,18,33,41,51,120}, 
				new double[][]{{1,0,0,0,0,0},{0.2,0.8,0,0,0,0},{0.02,0.98,0,0,0,0},{0,0.2,0.1,0.7,0,0},
						{0.01,0.10,0.05,0.58,0.18,0.08},{0.03,0.33,0.05,0.33,0.18,0.08},
						{0.20,0.42,0.10,0.20,0.05,0.03},{0.46,0.37,0.03,0.13,0.01,0.00},
						{0.83,0.10,0.01,0.06,0.00,0.00}},
						new MersenneTwister(seed));
		EmpiricalDiscreteDistArray femaleAttendance = new EmpiricalDiscreteDistArray(
				new double[]{5,6,13,16,18,20,24,28,120}, 
				new double[][]{{1.0, 0.0},{0.2, 0.8},{0.02, 0.98},{0.06, 0.94},{0.22, 0.78},{0.5, 0.5},{0.85, 0.15},{0.95, 0.05},{1.0, 0.0}},
				new MersenneTwister(seed));
		EmpiricalDiscreteDistArray maleAttendance = new EmpiricalDiscreteDistArray(
				new double[]{5,6,16,18,21,23,28,120}, 
				new double[][]{{1.0, 0.0},{0.2, 0.8},{0.02, 0.98},{0.17, 0.83},{0.5, 0.5},{0.75, 0.25},{0.95,0.05},{1.0, 0.0}},
				new MersenneTwister(seed));

		// have to make sure that
		// (a) persons with government jobs have high education level (5 or at least 4)
		// --> this is now taken care of in assigningEmployments()!
		// (b) young migrants are still at school and have correct education level (3, 4, 5)
		// (c) all kids of school-age are at school
		for (Agent agent : agentList){
			int level = 0;
			boolean attendance = false;
			if (agent.getGender() == Settings.GENDER.FEMALE){
				level = femaleLevels.nextInt(agent.getAge());
				attendance = (femaleAttendance.nextInt(agent.getAge()) > 0);
			}
			else {
				level = maleLevels.nextInt(agent.getAge());
				attendance = (maleAttendance.nextInt(agent.getAge()) > 0);
			}
			agent.setAchievedEducation(Settings.educationLevels[level]);
			agent.setAtSchool(attendance);
			if (agent.isMigrated() && agent.getAge() <= 23){ // see pickMigrants()
				agent.setAtSchool(true);
//				// adjust level if necessary
				adjustEducationLevel(agent);
			}
		}
	}

	/**
	 * author: ruth meyer - assigne education levels
	 */	
	private void adjustEducationLevel(Agent agent){
		// agent still goes to school so education level has to be matched to age group
		int level = 0;
		if (agent.getAge() >= 18) level = 5; // in tertiary education
		else if (agent.getAge() >= 14) level = 3; // in secondary education
		else level = 1; // in primary education
		if (!agent.getAchievedEducation().toString().equals(Settings.educationLevels[level])){
			agent.setAchievedEducation(Settings.educationLevels[level]);
		}
	}

	public ArrayList<Message> getStokvelMessageInbox() {
		return stokvelMessageInbox;
	}

	public ArrayList<Message> getStokvelMessageOutbox() {
		return stokvelMessageOutbox;
	}

	public void print (String printString) {
		if (Settings.PRINT_OUTPUT) {
			System.out.println(printString);
		}		
	}

	/**
	 * Prints the incidence and prevalence in the population.
	 *
	 */
	public void printAIDSProfile() {
		print("Time: " + currentTick);		
		print("-----HIV Incidence " + proportionPatients);
		print("-----HIVAIDS Patients: " + hivPatients.size());
		print("-----Persons on Migration : " + this.migratedAgents.size());
		print("-----Population size: " + (this.agentList.size() + this.outsidersList.size()));
		print("-----Outsiders' size: " + this.outsidersList.size());
		print("-----Percentage : " 
				+ ((double)(this.hivPatients.size() 
//						- this.hivOutsiderPatients.size()
						)
						/(this.agentList.size()
					+ this.outsidersList.size()
					
				)*100));
		print("Total number of households: " + this.householdList.size());
		print("Number of partners: " + this.partnersList.size());
	}

	public Normal getFemaleFertilityNormal() {
		return femaleFertilityNormal;
	}

	/** Adapted from Repast JinGirNew Impplementation */
	public void randomMeeting() {
		int numNodes = agentFriendsGraphLayout.getNodeList().size();
		int nPairs = (numNodes * (numNodes - 1)) / 2;
		int numPicks = (int) Math.round(nPairs * Rsub0);		
		for (int n = 0; n < numPicks; n++) {
			int index = Random.uniform.nextIntFromTo(0, numNodes - 1);
			AgentNode node = (AgentNode) agentFriendsGraphLayout.getNodeList().get(index);
			node.meetRandom(agentFriendsGraphLayout.getNodeList());
		}
	}

	/** Adapted from Repast JinGirNew Impplementation */
	private void neighborMeeting() {
		neighborMap.clear();
		double zeeSum = 0;
		int numNodes = getAgentFriendsGraphLayout().getNodeList().size();
		for (int n = 0; n < numNodes; n++) {
			AgentNode node = (AgentNode) getAgentFriendsGraphLayout().getNodeList().get(n);
			int degree = node.getOutDegree();
			if (degree > 1) {
				neighborMap.put(zeeSum, node);
				zeeSum += degree * (degree - 1);
			}
		}
		int nM = (int) (zeeSum / 2);   //will be even cause #edges even
		int numPicks = (int) Math.round(nM * Rsub1);
		for (int n = 0; n < numPicks; n++) {
			double pick = Random.uniform.nextDoubleFromTo(0, zeeSum);
			AgentNode node = (AgentNode) neighborMap.get(pick);
			if (node.getOutDegree() > 1) node.meetNeighbor();
		}
	}

	/** Adapted from Repast JinGirNew Impplementation */
	private void removeRandomFriendship() {
		int numNodes = getAgentFriendsGraphLayout().getNodeList().size();
		removalMap.clear();
		double zeeSum = 0;
		int nEdges = 0;
		for (int i = 0; i < numNodes; i++) {
			AgentNode node = (AgentNode) getAgentFriendsGraphLayout().getNodeList().get(i);
			int degree = node.getOutDegree();
			if (degree > 1) {
				removalMap.put(zeeSum, node);
				zeeSum += degree * (degree - 1);
			}
			nEdges += degree;
		}
		nEdges = nEdges / 2;
		int numPicks = (int) Math.round(nEdges * removeProb);
		for (int n = 0; n < numPicks; n++) {
			double pick = Random.uniform.nextDoubleFromTo(0, zeeSum);
			AgentNode node = (AgentNode) removalMap.get(pick);
			node.removeFriend();
		}
	}

	/**WHO Life Tables for calculating death probabilities  */
	public class WHOLifeTable{
		int[] ageIntervals;  // the age range classes, given as (inclusive) lower bounds 
		double[] deathProbabilities;  // the probability to die within the current age range (nqx)

		public WHOLifeTable(int[] ageIntervals, double[] deathProbabilities){
			this.ageIntervals = ageIntervals;
			this.deathProbabilities = deathProbabilities;
		}

		public double getDeathProbability(int age){
			// lookup age in ageIntervals, then use that index to find corresponding death probability
			int index = findAgeInterval(age);
			if (index >= 0 ) return this.deathProbabilities[index];
			return 1.0;
		}

		public int getAgeIntervalSize(int age){
			int i = findAgeInterval(age);
			if (i == this.ageIntervals.length-1) return 3; // assume last interval has size 3
			else if (i >= 0) return (this.ageIntervals[i+1] - this.ageIntervals[i]);
			return -1;
		}

		public boolean isLowerBound(int age){
			int i = findAgeInterval(age);
			if (i > 0 && this.ageIntervals[i] == age) return true;
			return false;
		}

		private int findAgeInterval(int age){
			if (age < 0) return -1;
			// linear search
			int index = 0;
			while (index < this.ageIntervals.length 
					&& this.ageIntervals[index] <= age){
				index++;
			}
			return index-1;
		}
	}

	public WHOLifeTable malesTable = new WHOLifeTable(
			new int[]{0, 1, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 
					70, 75, 80, 85, 90, 95, 100},
					new double[]{
					0.05769,//0-1
					0.01474,//1-5
					0.01903,//
					0.00694,
					0.01451,
					0.037,
					0.05493,
					0.11001,
					0.1725,
					0.14059,
					0.15916,
					0.167,
					0.16182,
					0.19377,
					0.23759,
					0.31431,
					0.4283,
					0.57377,
					0.73448,
					0.82419,
					0.87064,
					1});

	public WHOLifeTable femalesTable = new WHOLifeTable(
			new int[]{0, 1, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 
					70, 75, 80, 85, 90, 95, 100},
					new double[]{
					0.0502,
					0.01273,
					0.01813,
					0.00632,
					0.01167,
					0.04169,
					0.08878,
					0.13015,
					0.14072,
					0.13819,
					0.10376,
					0.09637,
					0.10613,
					0.1262,
					0.16889,
					0.24338,
					0.35132,
					0.49935,
					0.66916,
					0.78066,
					0.84565,
					1});

	public int getLastAgentID() {
		return lastAgentID;
	}

	/** Adapted from Repast JinGirNew Impplementation */
	public int getNumInfantsDeathThisYear() {
		return numInfantsDeathThisYear;
	}

	public PsRandom getPsr() {
		return psr;
	}

	public PsRandom getCourtShipDuration() {
		return courtShipDuration;
	}

	public Mine getMine() {
		return mine;
	}

	public PsRandom getOutsiderPsr() {
		return outsiderPsr;
	}

	/** Returns Population size */
	public int getPopulation() {
		return this.agentList.size();
	}

	/** Returns birth occuring this step*/
	public int getNumberBirths() {
		return numberBirths;
	}

	/** Returns number deaths this year */
	public int getNumDeathsPerYear() {
		return numDeathsPerYear;
	}

	/** Returns percentage of households receiving grants */
	public double getProportionHHReceivingGrants() {
		return (double) (this.numHHReceivingGrants / this.householdList.size());
	}
	
	/** Returns percentage of households lacking food */
	public double getProportionHHLackingFood() {
		return (double) (this.numHHLackingFood / this.householdList.size());
	}

	/** Returns number of households which are members of funeral clubs */
	public int getNumHHFuneralClubMembers() {
		return numHHFuneralClubMembers;
	}

	public int getNumHHGracePeriod() {
		return numHHGracePeriod;
	}

	/** Returns number of stokvels*/
	public int getNumStokvels() {
		return this.savingsClubs.size();
	}

	/** Returns total number of stokvel members*/
	public int getNumHHStokvelMembers() {
		return this.numHHStokvelMembers;
	}

	/** Returns number of stokvels*/
	public int getNumMigrants() {
		return this.migratedAgents.size();
	}

	/** Returns number of stokvels*/
	public int getNumAgentsJobless() {
		return numAgentsJobless;
	}

	/** Returns number of unfit agents*/
	public int getNumAgentsNotLaborFit() {
		return numAgentsNotLaborFit;
	}

	/** Returns number of agents on piecework*/
	public int getNumAgentsPieceWork() {
		return numAgentsPieceWork;
	}

	/** Returns number of agents on mine job*/
	public int getNumAgentsOnMineJob() {
		return numAgentsOnMineJob;
	}

	/** Returns number of HIV/AIDS patients*/
	public int getNumHIVPatients() {
		return this.hivPatients.size();
	}

	/** Returns prevalence in population*/
	public double getPrevalence() {
		double p = 0;
		p = ((double) this.hivPatients.size()/this.agentList.size());
		p *= 100;
		return p;
	}

	/** Returns children born to HIV parents this step.*/
	public int getNumChildrenBornHIVParents() {
		return numChildrenBornHIVParents;
	}

	public int getNumHHAccommodated() {
		return numHHAccommodated;
	}

	public int getNumHHDissolved() {
		return numHHDissolved;
	}

	public int getNumAdults() {
		return numAdults;
	}

	public int getNumChildren() {
		return numChildren;
	}

	public int getNumSeniors() {
		return numSeniors;
	}

	public int getNumAgentsOnStreet() {
		return this.numAgentsOnStreet;
	}

	public int getNumChildHH() {
		return this.numChildHH;
	}

	public int getNumHH() {
		return this.householdList.size();
	}

	/**
	 * Records the sexual network 
	 * 
	 */
	public HashMap<String, String[]> recordSexualNetwork(){
		HashMap<String, String[]> network = new HashMap<String, String[]>();
		for (Agent p : this.agentList){
			ArrayList<String> partners = new ArrayList<String>();
			if (p.getSexualPartners().size() > 0){
				for (Agent cp : p.getSexualPartners()){
					partners.add("" + cp.getId());
				}
			}
			if (partners.size() > 0){
				String[] list = new String[partners.size()];
				list = partners.toArray(list);
				network.put("" + p.getId(), list);
			}
		}
		return network;
	}

	/**
	 * Ruth - Records the sexual network into pajek with a prefix string
	 * @param prefix
	 */
	public void sexualNetworkPajek(String prefix) {
		PrintWriter netRecorder;
		HashMap<Integer, ArrayList<Agent>> sexualNetwork = new  HashMap<Integer, ArrayList<Agent>>();
		HashMap<Integer, Boolean> verticesMale = new HashMap<Integer, Boolean>();
		HashMap<Integer, Boolean> verticesFemale = new HashMap<Integer, Boolean>();

		ArrayList<Agent> temp = new ArrayList<Agent>();

		for (Agent agent : this.agentList) {
			if (!agent.getSexualPartners().isEmpty()
//					&& agent.getAgeGroup() != Settings.AGE_GROUP.CHILD
			) {
				if (!(agent.getSexualPartners().size() == 1
						&& agent.getSexualPartners().get(0)
						.getSexualPartners().size() == 1)) {
					temp.add(agent);
				}
				else {
					numDyads++;
				}
			}
		}

		for (Agent agent : temp) {
			sexualNetwork.put(new Integer(agent.getId()), 
					new ArrayList<Agent>(agent.getSexualPartners()));
			if (agent.getGender() == Settings.GENDER.MALE) {
				if (agent.hasHIV()) {
					verticesMale.put(new Integer(agent.getId()), true);
				}
				else {
					verticesMale.put(new Integer(agent.getId()), false);
				}
			}
			else {
				if (agent.hasHIV()) {
					verticesFemale.put(new Integer(agent.getId()), true);
				}
				else {
					verticesFemale.put(new Integer(agent.getId()), false);
				}						
			}			 
		}

		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".net")));
			HashMap<Integer, Integer> vertices = new HashMap<Integer, Integer>();
			int counter = 1;
			netRecorder.println("*Vertices " + (verticesMale.size() + verticesFemale.size()));		
			for (Iterator<Integer> it = verticesMale.keySet().iterator(); it.hasNext(); ){
				Integer id = it.next();
				boolean infected = verticesMale.get(id);
//				netRecorder.println(counter + " \"" + id.intValue() + " ic " + ((infected)? "Purple" : "RoyalBlue"));				 
//				netRecorder.println(counter + " person-" + id.intValue() + " ic " + ((infected)? "Purple" : "RoyalBlue"));
				netRecorder.println(counter + " person-" + id.intValue() + " ic " + "RoyalBlue");
//				netRecorder.println(counter + " " + id.intValue() + " ic " + ((infected)? "Purple" : "RoyalBlue"));
//				netRecorder.println(counter + " \"" + id.intValue() + "\" 1 1");
				vertices.put(id, new Integer(counter));
				counter++;


			}
			for (Iterator<Integer> it = verticesFemale.keySet().iterator(); it.hasNext(); ){
				Integer id = it.next();
				boolean infected = verticesFemale.get(id);
//				netRecorder.println(counter + " \"" + id.intValue() + " ic " + ((infected)? "BrickRed" : "Dandelion"));
//				netRecorder.println(counter + " person-" + id.intValue() + " ic " + ((infected)? "BrickRed" : "Dandelion"));
				netRecorder.println(counter + " person-" + id.intValue() + " ic " + "Dandelion");
//				netRecorder.println(counter + " " + id.intValue() + " ic " + ((infected)? "BrickRed" : "Dandelion"));
//				netRecorder.println(counter + " \"" + id.intValue() + "\" 1 1");
				vertices.put(id, new Integer(counter));
				counter++;

			}
			netRecorder.println("*Edges");		 

			for (Iterator<Integer> it = sexualNetwork.keySet().iterator(); it.hasNext(); ){
				Integer id = it.next();
				ArrayList<Agent> partners = sexualNetwork.get(id);
				for (Agent p : partners) {
//					if (vertices.containsKey(new Integer(p.getId()))) {
//					netRecorder.println(vertices.get(id) + " " + vertices.get(new Integer(p.getId())));
//					}
					netRecorder.println(vertices.get(id) + " " + vertices.get(new Integer(p.getId())));
				}				 				 
			}

			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public ArrayList<Agent> getPregnantFemales() {
		return pregnantFemales;
	}

	private void refreshVariables() {
		this.numHHLackingFood = 0;
		this.numHHReceivingGrants = 0;
		this.numHHFuneralClubMembers = 0;
		this.numHHGracePeriod = 0;
		this.numHHStokvelMembers = 0;
		this.numAgentsPieceWork = 0;
		this.numAgentsJobless = 0;
		this.numAgentsNotLaborFit = 0;
		this.numAgentsOnMineJob = 0;
		this.numChildrenBornHIVParents = 0;
		this.numHHDissolved = 0;
		this.numHHAccommodated = 0;
		this.numAdults = 0;
		this.numChildren = 0;
		this.numSeniors = 0;
		this.numberBirths = 0;
		numAgentsOnStreet = 0;
		numChildHH = 0;	
		numSingleMales = 0;
		numSingleFemales = 0;
		numMarriedAgents = 0;
		numWidowedMales = 0;
		numWidowedFemales = 0;
		numSexuallyActiveAgents = 0;
		numUnPairedIndividuals = 0;
		numDyads = 0;
		meanFemaleDegree = 0; 
		meanMaleDegree = 0;
		varFemaleDegree = 0; 
		varMaleDegree = 0;
	}
	
	/**
	 * RM-SJA: Creates empirical households based on RADAR Survey
	 *
	 */
	private void createEmpiricalHouseholds() {
		// (according to RADAR data, size distribution follows Normal dist with mean = 7, sd = 3)
		Normal hhSizeDist = new Normal(7.0, 3.0, new MersenneTwister(seed));  
		// (age distribution of hh heads follows Normal dist with mean = 56.2, sd = 12.2)
		Normal hhHeadAgeDist = new Normal(56.2, 12.2, new MersenneTwister(seed));
		// age difference between married spouses (man-woman), approx. by Normal dist with mean = 8.43, sd = 6.576
		Normal ageDiffDist = new Normal(8.43, 6.576, new MersenneTwister(seed));
		// female hh heads are 58% widowed, 22.4% single, 16.2% divorced/separated, 3.4% married
		EmpiricalDiscreteDist femaleStatusDist = new EmpiricalDiscreteDist(new double[]{0.224, 0.034, 0.162, 0.58}, new MersenneTwister(seed));
		// male hh heads are 0.8% single, 96.4% married, 0.8% divorced, 2% widowed
		EmpiricalDiscreteDist maleStatusDist = new EmpiricalDiscreteDist(new double[]{0.008, 0.964, 0.008, 0.02}, new MersenneTwister(seed));
		// type of hh member (except head / spouse): child 0.653, grandchild 0.238, other 0.109
		EmpiricalDiscreteDist memberTypeDist = new EmpiricalDiscreteDist(new double[]{0.653, 0.238, 0.109}, new MersenneTwister(seed));
		// general age distribution, can be approximated with Gamma distribution, with alpha = 2.4 and lambda = 0.086)
		Gamma ageDist = new Gamma(2.4, 0.086, new MersenneTwister(seed));

		int size = 0;
		int i = 0;
		for (i = this.lastHouseholdID; i < initNumHouseholds; i++){
			Household household = new Household(i, this);
			// determine size of hh 
			do {
				size = (int)hhSizeDist.nextDouble();
			} while (size < 5);
			// create members according to empirical data from RADAR
			populateHousehold(size, household, hhHeadAgeDist, ageDiffDist, ageDist, femaleStatusDist, maleStatusDist, memberTypeDist);
			// give some random cash
			household.addWealth(Random.uniform.nextDoubleFromTo(Settings.MIN_STARTING_WEALTH, 
					Settings.MAX_STARTING_WEALTH));
			householdList.add(household);
			houseMap.put(new Integer(household.getId()), household);
		}
		this.lastHouseholdID = i;
	}

	/**
	 * Assign parents to the created child agnet
	 * @param child
	 * @param head
	 * @param spouse
	 */
	public void assignParents(Agent child, Agent head, Agent spouse) {
		if (head.getGender() == Settings.GENDER.MALE) {
			child.setFatherID(head.getId());
			if (spouse != null) {
				child.setMotherID(spouse.getId());
				spouse.setFatherOfMyChildID(head.getId());
			}
		}
		else {
			child.setMotherID(head.getId());			 
			if (spouse != null) {
				child.setFatherID(spouse.getId());
				head.setFatherOfMyChildID(spouse.getId());
			}
		}
		head.getOffSprings().add(child);
		if (spouse != null) {
			spouse.getOffSprings().add(child);
		}
	}

	/**
	 * Populate the household with members and distributions.
	 * @param numMembers
	 * @param household
	 * @param hhHeadAgeDist
	 * @param ageDiffDist
	 * @param ageDist
	 * @param femaleStatusDist
	 * @param maleStatusDist
	 * @param memberTypeDist
	 */
	public void populateHousehold(int numMembers, Household household, Normal hhHeadAgeDist, Normal ageDiffDist, Gamma ageDist,
			EmpiricalDiscreteDist femaleStatusDist, EmpiricalDiscreteDist maleStatusDist, EmpiricalDiscreteDist memberTypeDist){
		int id = getLastAgentID();
		int numCreated = 0;
		Agent head = new Agent(id, household, this);
		id++;
		head.setBirthTick(Random.uniform.nextIntFromTo(0,11));
		int gender = Math.random() > 0.5 ? Settings.GENDER.FEMALE : Settings.GENDER.MALE;
		head.setGender(gender);
		head.setMaritalStatus(Settings.MARITAL_STATUS.SINGLE);
		head.setAge((int)hhHeadAgeDist.nextDouble());
		if (gender == Settings.GENDER.FEMALE) {
			setAsHead(head, femaleStatusDist);
		}
		else {
			setAsHead(head, maleStatusDist);
		}
		head.initializeAgeGroup();
		head.initialize();		 
		household.addAgent(head);

		numCreated++;
		Agent spouse = null;
		//if head married, give head a spouse
		if (head.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED) {
			Agent husband = null, wife = null;
			spouse = new Agent(id, household, this);
			spouse.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
			id++; numCreated++;
			int ageDiff = (int)ageDiffDist.nextDouble();
			int headGender = head.getGender(); 
			if (headGender == Settings.GENDER.FEMALE){
				spouse.setGender(Settings.GENDER.MALE);
				spouse.setAge(head.getAge() + ageDiff);
				wife = head; 
				husband = spouse;
			}
			else {
				spouse.setGender(Settings.GENDER.FEMALE);
				spouse.setAge(head.getAge() - ageDiff);
				husband = head;
				wife = spouse;
			}
			spouse.initializeAgeGroup();
			spouse.initialize();
			household.addAgent(spouse);			 			 
			head.setSpouseID(spouse.getId());			 
			spouse.setSpouseID(head.getId());
			Couple couple = new Couple (this.lastCoupleID, husband, wife);
			this.lastCoupleID++;
			couple.setMarried(true);
			household.addCouple(couple);
		}


		// create rest of members
		// check for very rare case that hhsize is 1 and head is married --> change hhsize to 2
		if (numCreated > numMembers) {
			numMembers = numCreated;
		}

		Agent[] members = new Agent[numMembers-numCreated];
		int[] memberType = new int[members.length];
		for (int i = 0; i < memberType.length; i++){
			memberType[i] = memberTypeDist.nextInt();
		}

		memberType = sortTypes(memberType);

		int eldestChildAge = 0;

		for (int i = 0; i < members.length; i++){
			members[i] = new Agent(id, household, this);
			id++;
			members[i].setMaritalStatus(Settings.MARITAL_STATUS.SINGLE);
			if (Random.uniform.nextIntFromTo(0,1) == 0){
				members[i].setGender(Settings.GENDER.FEMALE);
			}
			else {
				members[i].setGender(Settings.GENDER.MALE);
			}
			int age = (int)ageDist.nextDouble();			
			int compAge = head.getAge();
			if ((head.getGender() == Settings.GENDER.MALE) 
					&& head.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED
					&& spouse != null) {
				compAge = spouse.getAge();
			}
			if (memberType[i] == Settings.MEMBERS_TYPE.CHILD){
				// determine age according to hh head (resp. spouse): females can give birth between 14 and 45
				// --> child must be at least 14 years younger, at most 45 years younger
				while (age > (compAge-14) || age < (compAge-45)){
					age = (int)ageDist.nextDouble();
				}
				members[i].setAge(age);
				members[i].initializeAgeGroup();
				members[i].initialize();
				assignParents(members[i], head, spouse);
				household.addAgent(members[i]);

				if (age > eldestChildAge) {
					eldestChildAge = age;
				}
//				System.out.println("child aged " + age);
			}
			else if (memberType[i] == Settings.MEMBERS_TYPE.GRANDCHILD) {
				if (eldestChildAge < 15){
					// change member type to CHILD
					memberType[i] = Settings.MEMBERS_TYPE.CHILD;
//					System.out.println("-- changed type from grandchild to child");
					while (age > (compAge-14) || age < (compAge-45)){
						age = (int)ageDist.nextDouble();
						assignParents(members[i], head, spouse);
					}					
				}
				else {
					// determine age according to eldest child: females can give birth between 14 and 45
					// --> grandchild must be at least 14 years younger, at most 45 years younger
					while (age > (eldestChildAge-14) || age < Math.max((eldestChildAge-45),0)){
						age = (int)ageDist.nextDouble();
					}
				}
				members[i].setAge(age);
				members[i].initializeAgeGroup();
				members[i].initialize();
				household.addAgent(members[i]);
//				System.out.println("grandchild aged " + age);
			}
			else if (memberType[i] == Settings.MEMBERS_TYPE.OTHER){
				members[i].setAge(age);
				members[i].initializeAgeGroup();
				members[i].initialize();
//				System.out.println("other aged " + age);
				household.addAgent(members[i]);
			}

		}
		this.lastAgentID = id;
//		populateFamilyLinks(household, memberTypeDist);

	}

	private int[] sortTypes(int[] types){
		int[] sorted = new int[types.length];
		int index = 0;
		for (int i = 0; i < types.length; i++){
			// copy all the CHILD types
			if (types[i] == Settings.MEMBERS_TYPE.CHILD) {
				sorted[index] = types[i];
				index++;
			}
		}
		for (int i = 0; i < types.length; i++){
			// copy all the GRANDCHILD types
			if (types[i] == Settings.MEMBERS_TYPE.GRANDCHILD){
				sorted[index] = types[i];
				index++;
			}
		}
		for (int i = 0; i < types.length; i++){
			// copy all OTHER types
			if (types[i] == Settings.MEMBERS_TYPE.OTHER){
				sorted[index] = types[i];
				index++;
			}
		}
		return sorted;
	}

	/**
	 * Sets the person as head of household
	 * @param person
	 * @param statusDistribution
	 */
	private void setAsHead(Agent person, EmpiricalDiscreteDist statusDistribution){
		person.setMaritalStatus(statusDistribution.nextInt());
		person.setHead(true);
		person.getHousehold().setHead(person);
	}

	public Weibull getWeibull() {
		return weibull;
	}

	public int getPartners() {
		int count = this.partnersList.size();
		return count;
	}
	public int getMarriedCoupleCounts() {
		int count = 0;
		for (Household hh : this.householdList) {
			count += hh.getCoupleList().size();
		}
		return count;
	}


	/**
	 * Calculates the mean and variance of the degree of sexual network
	 *
	 */
	public void calcuDescStat() {
		double count = 0;
		double x = 0, x2 = 0;

		for (Agent agent : this.agentList) {
			if (agent.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
					&& agent.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
					&& agent.getGender() == Settings.GENDER.FEMALE
			) {
				x += agent.getSexualPartners().size();
				x2 += Math.pow(agent.getSexualPartners().size(), 2);
				count++;					
			}
		}


		double meanX = x/count;
		double meanX2 = x2/count;		
		double dev = meanX2 - Math.pow(meanX, 2);

		this.meanFemaleDegree = meanX;
		this.varFemaleDegree = dev;


		meanX = 0; meanX2 = 0; dev = 0; x = 0; x2 = 0; count = 0;

		for (Agent agent : this.agentList) {
			if (	agent.getAge() >= Settings.CHILD_AGE_LIMIT					
					&& agent.getAge() <= Settings.MALE_SEX_AGE_LIMIT
					&& agent.getGender() == Settings.GENDER.MALE
			) {
				x += agent.getSexualPartners().size();
				x2 += Math.pow(agent.getSexualPartners().size(), 2);
				count++;								
			}
		}

		meanX = x/count;
		meanX2 = x2/count;		
		dev = meanX2 - Math.pow(meanX, 2);

		this.meanMaleDegree = meanX;
		this.varMaleDegree = dev;


	}
	public void recordSexDegree(String prefix) {		
		PrintWriter netRecorder;

		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
					"females-" + prefix + ".txt")));
			for (Agent agent : this.agentList) {
				if (agent.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE 
						&& agent.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
						&& agent.getGender() == Settings.GENDER.FEMALE) {
					netRecorder.println(agent.getSexualPartners().size() + ", ");
				}
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();					
		}

		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
					"males-" + prefix + ".txt")));
			for (Agent agent : this.agentList) {
				if (agent.getAge() >= Settings.CHILD_AGE_LIMIT						
						&& agent.getAge() <= Settings.MALE_SEX_AGE_LIMIT
						&& agent.getGender() == Settings.GENDER.MALE
				) {
					netRecorder.println(agent.getSexualPartners().size() + ", ");
				}				
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();					
		}		


	}


	public void recordNetworks() {
//		gui.saveList("socialneighborhood-" + currentTick,
//		getHouseholdGraphLayout().getNodeList());
//		gui.saveList("friend-" + currentTick, getAgentFriendsGraphLayout().getNodeList());
//		gui.saveList("family-" + currentTick,
//		getExtendedFamilyGraphLayout().getNodeList());
//		gui.saveList("savings-" + currentTick,
//		getSavingsGraphLayout().getNodeList());
//		ks.addList(currentTick, getAgentFriendsGraphLayout().getNodeList());
//		ks.addList(currentTick, getExtendedFamilyGraphLayout().getNodeList());
//		ks.addList(currentTick, getHouseholdGraphLayout().getNodeList());
	}

	public ArrayList<Message> getPartnershipMessageInbox() {
		return partnershipMessageInbox;
	}

	public ArrayList<Message> getPartnershipMessageOutbox() {
		return partnershipMessageOutbox;
	}
	public int getNumSingleMales() {
		return numSingleMales;
	}

	public int getNumSingleFemales() {
		return numSingleFemales;
	}

	public int getNumMarriedAgents() {
		return numMarriedAgents;
	}

	public int getNumWidowedMales() {
		return numWidowedMales;
	}

	public int getNumWidowedFemales() {
		return numWidowedFemales;
	}

	public int getNumSexuallyActiveAgents() {
		return numSexuallyActiveAgents;
	}

	public int getNumUnPairedIndividuals() {
		return numUnPairedIndividuals;
	}

	/**
	 * Calcualtes the number of dyads in the sexual network
	 * @return
	 */
	public double getNumDyads() {
		for (Agent agent : this.agentList) {
			if (!agent.getSexualPartners().isEmpty()
			) {
				if (!(agent.getSexualPartners().size() == 1
						&& agent.getSexualPartners().get(0)
						.getSexualPartners().size() == 1)) {					
				}
				else {
					numDyads++;
				}
			}
		}
		return (double)(numDyads/2);		
	}

	public void sexualNetworkPajekAIDS(String prefix) {
		PrintWriter netRecorder;
		HashMap<Integer, ArrayList<Agent>> sexualNetwork = new  HashMap<Integer, ArrayList<Agent>>();
		HashMap<Integer, Boolean> verticesMale = new HashMap<Integer, Boolean>();
		HashMap<Integer, Boolean> verticesFemale = new HashMap<Integer, Boolean>();

		ArrayList<Agent> temp = new ArrayList<Agent>();

		for (Agent agent : this.agentList) {
			if (!agent.getSexualPartners().isEmpty()
//					&& agent.getAgeGroup() != Settings.AGE_GROUP.CHILD
			) {
				temp.add(agent);
				/*if (!(agent.getSexualPartners().size() == 1
						&& agent.getSexualPartners().get(0)
						.getSexualPartners().size() == 1)) {
					temp.add(agent);
				}*/
			}
		}

		for (Agent agent : temp) {
			sexualNetwork.put(new Integer(agent.getId()), 
					new ArrayList<Agent>(agent.getSexualPartners()));
			if (agent.getGender() == Settings.GENDER.MALE) {
				if (agent.hasHIV()) {
					verticesMale.put(new Integer(agent.getId()), true);
				}
				else {
					verticesMale.put(new Integer(agent.getId()), false);
				}
			}
			else {
				if (agent.hasHIV()) {
					verticesFemale.put(new Integer(agent.getId()), true);
				}
				else {
					verticesFemale.put(new Integer(agent.getId()), false);
				}						
			}			 
		}


		for (Agent agent : temp) {
			if (!agent.getSexualPartners().isEmpty()
			) {
				sexualNetwork.put(new Integer(agent.getId()), 
						new ArrayList<Agent>(agent.getSexualPartners()));
				if (agent.getGender() == Settings.GENDER.MALE) {
					if (agent.hasHIV()) {
						verticesMale.put(new Integer(agent.getId()), true);
					}
					else {
						verticesMale.put(new Integer(agent.getId()), false);
					}
				}
				else {
					if (agent.hasHIV()) {
						verticesFemale.put(new Integer(agent.getId()), true);
					}
					else {
						verticesFemale.put(new Integer(agent.getId()), false);
					}						
				}

			}
		}

		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".net")));
			HashMap<Integer, Integer> vertices = new HashMap<Integer, Integer>();
			int counter = 1;
			netRecorder.println("*Vertices " + (verticesMale.size() + verticesFemale.size()));		
			for (Iterator<Integer> it = verticesMale.keySet().iterator(); it.hasNext(); ){
				Integer id = it.next();
				boolean infected = verticesMale.get(id);
//				netRecorder.println(counter + " \"" + id.intValue() + " ic " + ((infected)? "Purple" : "RoyalBlue"));				 
				netRecorder.println(counter + " person-" + id.intValue() + " ic " + ((infected)? "Purple" : "RoyalBlue"));
//				netRecorder.println(counter + " person-" + id.intValue() + " ic " + "RoyalBlue");
//				netRecorder.println(counter + " " + id.intValue() + " ic " + ((infected)? "Purple" : "RoyalBlue"));
//				netRecorder.println(counter + " \"" + id.intValue() + "\" 1 1");
				vertices.put(id, new Integer(counter));
				counter++;


			}
			for (Iterator<Integer> it = verticesFemale.keySet().iterator(); it.hasNext(); ){
				Integer id = it.next();
				boolean infected = verticesFemale.get(id);
//				netRecorder.println(counter + " \"" + id.intValue() + " ic " + ((infected)? "BrickRed" : "Dandelion"));
				netRecorder.println(counter + " person-" + id.intValue() + " ic " + ((infected)? "BrickRed" : "Dandelion"));
//				netRecorder.println(counter + " person-" + id.intValue() + " ic " + "Dandelion");
//				netRecorder.println(counter + " " + id.intValue() + " ic " + ((infected)? "BrickRed" : "Dandelion"));
//				netRecorder.println(counter + " \"" + id.intValue() + "\" 1 1");
				vertices.put(id, new Integer(counter));
				counter++;

			}
			netRecorder.println("*Edges");		 

			for (Iterator<Integer> it = sexualNetwork.keySet().iterator(); it.hasNext(); ){
				Integer id = it.next();
				ArrayList<Agent> partners = sexualNetwork.get(id);
				for (Agent p : partners) {
//					if (vertices.containsKey(new Integer(p.getId()))) {
//					netRecorder.println(vertices.get(id) + " " + vertices.get(new Integer(p.getId())));
//					}
					netRecorder.println(vertices.get(id) + " " + vertices.get(new Integer(p.getId())));
				}				 				 
			}

			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Calculates the percentage of sexually active agents linked to a partner
	 *
	 */
	public void calculateUtilization() {	
		for (Agent agent : this.agentList) {
			if (agent.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
					&& ((agent.getGender() == Settings.GENDER.FEMALE
							&& agent.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE 
					)
					|| (agent.getGender() == Settings.GENDER.MALE
							&& agent.getAge() >= Settings.CHILD_AGE_LIMIT
							&& agent.getAge() <= Settings.MALE_SEX_AGE_LIMIT))) {
				numSexuallyActiveAgents++;
				if (agent.getMaritalStatus() == Settings.MARITAL_STATUS.SINGLE) {
					if (agent.getGender() == Settings.GENDER.FEMALE) {
						numSingleFemales++;
					}
					else {
						numSingleMales++;
					}
				}
				if (agent.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED) {
					numMarriedAgents++;
				}
				if (agent.getMaritalStatus() == Settings.MARITAL_STATUS.WIDOWED) {
					if (agent.getGender() == Settings.GENDER.FEMALE) {
						numWidowedFemales++;
					}
					else {
						numWidowedMales++;
					}
				}
				if (agent.getSexualPartners().isEmpty()) {
					numUnPairedIndividuals++;
				}				
			}
		}

		double c = 0, ut = 0;
		for (Agent agent : this.agentList) {
			if (agent.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
					&& ((agent.getGender() == Settings.GENDER.FEMALE
							&& agent.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE)
							|| (agent.getGender() == Settings.GENDER.MALE
									&& agent.getAge() >= Settings.CHILD_AGE_LIMIT
									&& agent.getAge() <= Settings.MALE_SEX_AGE_LIMIT
							))) {
				c++;
				if (agent.getSexualPartners().isEmpty()) {
					ut++;
				}
			}
		}
		/*
		print("********** total sexually active agents:"  + c);
		print("********** total under utilized agents:"  + ut);*/
		print ("---------------------- Percentage utilization: " + (double) ((c-ut)/c)*100);		
	}

	public int getNumIncidence() {
		return this.numIncidenceThisYear;
	}

	public double getMeanFemaleDegree() {
		return meanFemaleDegree;
	}

	public double getMeanMaleDegree() {
		return meanMaleDegree;
	}

	public double getVarFemaleDegree() {
		return varFemaleDegree;
	}

	public double getVarMaleDegree() {
		return varMaleDegree;
	}

	/**
	 * outsiders: migrant workers
	 * 
	 * outsiders have no houses and are assumed to be living in mine(s)' hostels.
	 * they could rent a place at a village (rented by a household; a shelter)
	 * ... can have partners from the village; can as well marry, if their courtship
	 * stays longers and build a new 'house' in the village. 
	 * We are also not assuming any women outsider as a miner.
	 * They could be both skilled and unskilled workers
	 *
	 */	 
	private void createOutsiders() {
		//first five years
		double proportion = 0;
		if (currentTick - mine.getStartTick() < 60) {
			proportion = Uniform.staticNextDoubleFromTo(0.010, 0.015);	
		}
		else if (currentTick - mine.getStartTick() >= 60
				&& 
				currentTick - mine.getStartTick() < 120) {			
			proportion = Uniform.staticNextDoubleFromTo(0.005, 0.010);
		}
		else  if (currentTick - mine.getStartTick() >= 120) {
			proportion = Uniform.staticNextDoubleFromTo(0.003, 0.005);
		}
		
		int numOutsiders = (int) (proportion * this.agentList.size());
		for (int i=0; i<numOutsiders; i++) {			
			this.lastAgentID++;
			Agent outsider = new Agent(this.lastAgentID, this);
			this.outsidersList.add(outsider);
//			if (Math.random() < 0.40) {
//				setInitalizeHIVInfection(outsider);
//			}

			if (aidSpread.returnAgentPrevalence(outsider)) {
				setInitalizeHIVInfection(outsider);
			}
			//TODO: Need to initialize outsiders friendship links
		}		
	}

	public int getInitialFuneralClubs() {
		return initialFuneralClubs;
	}

	public void setInitialFuneralClubs(int initialFuneralClubs) {
		this.initialFuneralClubs = initialFuneralClubs;
	}

	public void setClusterProportion(int clusterProportion) {
		this.clusterProportion = clusterProportion;
	}

	public int getNetworkRecordInterval() {
		return networkRecordInterval;
	}

	public void setNetworkRecordInterval(int pajekInterval) {
		this.networkRecordInterval = pajekInterval;
	}

	public boolean isHealthExpenditure() {
		return healthExpenditure;
	}

	public void setHealthExpenditure(boolean healthExpenditure) {
		this.healthExpenditure = healthExpenditure;
	}

	public boolean isHIV() {
		return HIV;
	}

	public void setHIV(boolean hiv) {
		this.HIV = hiv;
	}

	public boolean isShowGui() {
		return showGui;
	}

	public void setShowGui(boolean showGui) {
		this.showGui = showGui;
	}	
}