package display;

import java.awt.Color;
import java.io.*;
import java.util.*;

import cern.jet.random.Uniform;
import clubs.*;
import uchicago.src.sim.analysis.DataSource;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.network.NetworkFactory;
import uchicago.src.sim.util.SimUtilities;
import units.*;

/**
 * Class that contains methods concerning networks and their displays.
 */
public class ModelGUI {
	private Model model;
	/**Nodelist for agents in Model */
	private ArrayList<AgentNode> agentNodeList = new ArrayList<AgentNode>();
	/**Nodelist for agents friendship network */
	private ArrayList<AgentNode> agentFriendsNodeList = new ArrayList<AgentNode>();
	/**Nodelist for agents stokvel network */
	private ArrayList<AgentNode> savingsNodeList = new ArrayList<AgentNode>();
	/**Nodelist for Household neighbor network */
	private ArrayList<HouseholdNode> neighborhoodNodeList = new ArrayList<HouseholdNode>();
	/**Nodelist for Household extended family network */
	private ArrayList<HouseholdNode> extendedFamilyNodeList = new ArrayList<HouseholdNode>();
	private HashMap<Integer, AgentNodeMap> agentNodeMap = new HashMap<Integer, AgentNodeMap>(),
	agentFriendNodeMap = new HashMap<Integer, AgentNodeMap>(),
	savingsNodeMap = new HashMap<Integer, AgentNodeMap>();
	private HashMap<Integer, HouseholdNodeMap> householdNodeMap = new HashMap<Integer, HouseholdNodeMap>(),
	extendedFamilyNodeMap = new HashMap<Integer, HouseholdNodeMap>();
	/**
	 * Constructor
	 * @param _model
	 */
	public ModelGUI(Model _model) {
		this.model = _model;
		initialize();
	}
	/**
	 * Initialize networks' lists
	 *
	 */
	public void initialize() {
		initializeHouseholdNodes();
		initializeAgentFriendsNodes();
		initializeSavingsNodes();
		initializeExtendedFamilyNodes();
	}
	/** social nodes at the agent's level */
	public void initializeAgentNodes() {
		agentNodeList = (ArrayList<AgentNode>) NetworkFactory
		.createUnlinkedNetwork(model.getAgentList().size(),
				AgentNode.class);
		int i = 0;
		for (AgentNode node : agentNodeList) {
			node.setAgent(model.getAgentList().get(i));
			node.setColor(Color.GREEN);
			node.setNodeLabel(" " + node.getAgent().getId());
			node.setID(model.getAgentList().get(i).getId());
			agentNodeMap.put(new Integer(node.getAgent().getId()),
					new AgentNodeMap(node.getAgent(), node));
			i++;
		}
	}	
	/**
	 * Initialize savings nodes list
	 *
	 */
	public void initializeSavingsNodes() {
		ArrayList<Agent> temp = new ArrayList<Agent>();
		for (Agent agent : model.getAgentList()) {
			temp.add(agent);
		}
		savingsNodeList = (ArrayList<AgentNode>) NetworkFactory
		.createUnlinkedNetwork(temp.size(), AgentNode.class);
		int i = 0;
		for (AgentNode node : savingsNodeList) {
			node.setAgent(temp.get(i));
			if (node.getAgent().isInnovator()) {
				node.setColor(Color.BLACK);
			}
			node.setNodeLabel(" " + node.getAgent().getId());
			node.setID(node.getAgent().getId());
			savingsNodeMap.put(new Integer(node.getAgent().getId()),
					new AgentNodeMap(node.getAgent(), node));
			i++;
		}
	}
	/**
	 * Initialize friendships nodes list
	 *
	 */
	public void initializeAgentFriendsNodes() {
		// each friend has no friend at the moment ...
		ArrayList<Agent> temp = new ArrayList<Agent>(model.getAgentList());
		agentFriendsNodeList = (ArrayList<AgentNode>) NetworkFactory
		.createUnlinkedNetwork(temp.size(), AgentNode.class);	
		int i=0;
		for (AgentNode node : agentFriendsNodeList) {
			Agent agent = temp.get(i);
			node.setAgent(agent);
			node.setColor(Color.GREEN);
			node.setNodeLabel("" + agent.getId());
			node.setID(agent.getId());			
			agentFriendNodeMap.put(new Integer(agent.getId()),
					new AgentNodeMap(agent, node));
			i++;
		}
	}
	/**
	 * Remove agent from all networks
	 * @param agent
	 */
	public void removeAgentNode(Agent agent) {
		removeAgentFriendNode(agent);
		removeSavingsNode(agent);
	}
	/**
	 * Make friendship links
	 * @param agent
	 */
	public void makeFriendsLinks(Agent agent) {
		AgentNode node = (AgentNode) this.agentFriendNodeMap.get(new Integer(
				agent.getId())).node;
		for (Agent friend : agent.getFriends()) {
			if (this.agentFriendNodeMap.containsKey(new Integer(friend.getId()))) {
				AgentNode friendNode = (AgentNode) this.agentFriendNodeMap
				.get(new Integer(friend.getId())).node;
				node.makeEdgeToFrom(friendNode);
			}
		}
	}	
	/**
	 * Make savings links among club members
	 * @param club
	 */
	public void makeSavingsLinks(SavingsClub club) {
		ArrayList<AgentNode> clubNodes = new ArrayList<AgentNode>();
		// get the nodes for the members
		for (Agent agent : club.getMembers()) {
			if (model.getAgentList().contains(agent)) {
				AgentNode node = savingsNodeMap.get(new Integer(agent.getId())).node;
				if (agent.getId() == club.getModerator().getId()) {
					node.setColor(Color.BLACK);
				} else {
					node.setColor(Color.BLUE);
				}
				node.setNodeLabel(" " + node.getAgent().getId());
				node.clearEdges();
				clubNodes.add(node);
			}
		}
		// now make edges between all the nodes
		for (AgentNode node : clubNodes) {
			for (AgentNode otherNode : clubNodes) {
				if (node != otherNode) {
					node.makeEdgeToFrom(otherNode);
				}
			}
		}
	}
	/**
	 * Remove savings members from the club
	 * @param club
	 */
	public void removeSavingsMemberAll(SavingsClub club) {
		for (Agent agent : club.getMembers()) {
			removeSavingsMember(agent);
		}
	}
	/**
	 * Remove the agent from the savings clubs
	 * @param agent
	 */
	public void removeSavingsMember(Agent agent) {
		if (!savingsNodeMap.containsKey(new Integer(agent.getId()))) {
			return;
		}
		AgentNode node = (AgentNode) this.savingsNodeMap.get(new Integer(agent
				.getId())).node;
		node.clearEdges();
		model.getSavingsGraphLayout().getNodeList().remove(node);
//		savingsNodeMap.remove(new Integer(agent.getId()));
	}
	/**
	 * Initialize extended family nodes
	 *
	 */
	public void initializeExtendedFamilyNodes() {
		ArrayList<Household> temp = new ArrayList<Household>(model
				.getHouseholdList());
		extendedFamilyNodeList = (ArrayList<HouseholdNode>) NetworkFactory
		.createUnlinkedNetwork(temp.size(), HouseholdNode.class);
		int i = 0;
		for (HouseholdNode node : extendedFamilyNodeList) {
			node.setHouse(model.getHouseholdList().get(i));
			node.setID(node.getHousehold().getId());
			node.setColor(Color.BLUE);
			node.setNodeLabel(" " + node.getHousehold().getId());
			extendedFamilyNodeMap.put(new Integer(node.getHousehold().getId()),
					new HouseholdNodeMap(node.getHousehold(), node));
			i++;
		}

		for (HouseholdNode node : extendedFamilyNodeList) {
			makeExtendedFamilyLinks(node);
		}
	}
	/**
	 * Create extened family links for a newly created household
	 * @param h1: husband's household
	 * @param h2: wife's household
	 */
	public void createExtendedFamilyLink(Household h1, Household h2) {
		ArrayList<HouseholdNode> temp = new ArrayList<HouseholdNode>
		(model.getExtendedFamilyGraphLayout().getNodeList());

		HouseholdNode h1_node = null, h2_node = null;
		for (HouseholdNode node : temp) {
			if (h1_node != null && h2_node != null) {
				break;
			}
			if (node.getID() == h1.getId()) {
				h1_node = node;
			}
			else if (node.getID() == h2.getId()) {
				h2_node = node;
			}
		}		
//		HouseholdNode h1_node = (HouseholdNode) this.extendedFamilyNodeMap
//		.get(h1.getId()).node;
//		HouseholdNode h2_node = (HouseholdNode) this.extendedFamilyNodeMap
//		.get(h2.getId()).node;
		if (h1_node != null && h2_node != null) {
			h1_node.makeEdgeToFrom(h2_node);
		}
	}

	
	public void makeExtendedFamilyLinks(HouseholdNode node) {
		Household household = (Household) node.getHousehold();
		for (Integer id : household.getExtendedFamilyIDList()) {
			if (this.extendedFamilyNodeMap.containsKey(id)) {
				HouseholdNode rel_node = (HouseholdNode) this.extendedFamilyNodeMap
				.get(id).node;
				node.makeEdgeToFrom(rel_node);
			}
		}
	}
	/**
	 * Initialize household social links
	 *
	 */
	public void initializeHouseholdNodes() {
		// houseNodeList = (ArrayList<HouseholdNode>)
		// NetworkFactory.createUnlinkedNetwork(model.getHouseholdList().size(),
		// HouseholdNode.class);
//		neighborhoodNodeList = (ArrayList<HouseholdNode>) NetworkFactory
//		.createWattsStrogatzNetwork(model.getHouseholdList().size(), 2,
//		swProb, HouseholdNode.class, SAEdge.class);

		neighborhoodNodeList = (ArrayList<HouseholdNode>) NetworkFactory.createRandomDensityNetwork(model.getHouseholdList().size(), 0.10, 
				false, true, HouseholdNode.class, SAEdge.class);

		// householdNodeList = (ArrayList<HouseholdNode>)
		// NetworkFactory.createRandomDensityNetwork(model.getHouseholdList().size(),
		// 0.02, false, true, HouseholdNode.class, SAEdge.class);
		int i = 0;
		for (HouseholdNode node : neighborhoodNodeList) {					
			node.setHouse(model.getHouseholdList().get(i));
			node.setColor(Color.GREEN);
			node.setNodeLabel(" " + node.getHousehold().getId());
			householdNodeMap.put(new Integer(node.getHousehold().getId()),
					new HouseholdNodeMap(node.getHousehold(), node));
			node.setID(node.getHousehold().getId());
			i++;
		}				
	}

	/**
	 * TODO: Need to give the source from where it is adapted.
	 *
	 */
	public void createSquareLattic() {
		int nCols = model.getHouseholdGraphLayout().getNodeList().size();
		int nRows = 1;
//		int nNodes = nCols * nRows;
		int nNodes = model.getHouseholdGraphLayout().getNodeList().size();	    

		int radius = 2;

		SimUtilities.shuffle(model.getHouseholdGraphLayout().getNodeList());

		for (int i = 0; i < nNodes; i++)
		{
			HouseholdNode iNode = (HouseholdNode) model.getHouseholdGraphLayout().getNodeList().get(i);
			iNode.clearEdges();
		}
		for (int i = 0; i < nNodes; i++)
		{
			HouseholdNode iNode = (HouseholdNode) model.getHouseholdGraphLayout().getNodeList().get(i);

			int x = i%nCols;          //col "grid" coords for index i
			int y = (int)Math.floor(i/nCols);   //row
			int j; //index of other node
			int xJ;
			int yJ;
			for (int r = 1; r <= (radius); r++)	{
				//"east" connection ("west" will be set by previous/last node)
				xJ = (x+r)%nCols;
				yJ = y;
				j =yJ*nCols+xJ;
				if (i != j) {

					HouseholdNode jNode = (HouseholdNode)model.getHouseholdGraphLayout().getNodeList().get(j);
					iNode.makeEdgeToFrom(jNode);
				}

				//"south" connection ("north" will be set by previous/last node)
				xJ = x;
				yJ = (y+r)%nRows;
				j =yJ*nCols+xJ;
				if (i != j)  //traps 1 by n "ring"
				{
					HouseholdNode jNode = (HouseholdNode)model.getHouseholdGraphLayout().getNodeList().get(j);
					iNode.makeEdgeToFrom(jNode);
				}
			}
		}

//		model.getHouseholdGraphLayout().setList(neighborhoodNodeList);
		randomRewire(0.1);

		model.updateGraphLayoutAll();
		model.updateDisplayAll();
	}

	/** 
	 * Need to give the source from where it is adapted
	 */
	public static int getIJTie (Node iNode, Node jNode) {
		return iNode.hasEdgeTo(jNode) ? 1 : 0;
	}

	/**
	 * Repast's adaptation of random rewire, because households keep coming 
	 * in the network
	 * @param rewireProb
	 */

	public void randomRewire(double rewireProb) {
		ArrayList<HouseholdNode> temp = new ArrayList<HouseholdNode> (model.getHouseholdGraphLayout().getNodeList());
		ArrayList<HouseholdNode> temp2 = new ArrayList<HouseholdNode> (model.getHouseholdGraphLayout().getNodeList());
		ArrayList<SAEdge> edges = new ArrayList<SAEdge>();
		for (HouseholdNode node : temp) {
			edges.addAll(node.getOutEdges());			
		}

		for (int e = 0; e < edges.size (); e++) {
			if (rewireProb > uchicago.src.sim.util.Random.uniform.nextDoubleFromTo (0, 1)) {
				SAEdge edge = (SAEdge) edges.get (e);
				int i = temp2.indexOf (edge.getFrom ());
				int j = temp2.indexOf (edge.getTo ());
				HouseholdNode iNode = (HouseholdNode) edge.getFrom();
				HouseholdNode jNode = (HouseholdNode) edge.getTo();

				int newJ = uchicago.src.sim.util.Random.uniform.nextIntFromTo (0, temp.size () - 1);

				if ((i != newJ) && (getIJTie ((Node) temp2.get (i),
						(Node) temp2.get (newJ)) < 1)) {

					HouseholdNode kNode = (HouseholdNode) temp.get(newJ);
					iNode.removeNode(jNode);
					iNode.makeEdgeToFrom(kNode);

//					((Node)model.getHouseholdGraphLayout().getNodeList() .get (j)).removeInEdge (edge);
//					edge.setTo ((Node) temp2.get (newJ));
//					((Node) model.getHouseholdGraphLayout().getNodeList().get (newJ)).addInEdge (edge);
				}
			}
		}
	}

	
	public void initializeFuneralClubLinks() {
		for (FuneralClub funeralClub : model.getFuneralClubs()) {
			for (Household member1 : (ArrayList<Household>) funeralClub
					.getMembers()) {
				HouseholdNode node1 = householdNodeMap.get(new Integer(member1
						.getId())).node;
				ArrayList nodeList = new ArrayList();
				for (Household member2 : (ArrayList<Household>) funeralClub
						.getMembers()) {
					if (member1 != member2) {
						HouseholdNode node2 = householdNodeMap.get(new Integer(
								member2.getId())).node;
						nodeList.add(node2);
					}
				}
				node1.constructEdges(nodeList, Color.blue);
			}
		}
	}

	/**
	 * Remove household from all netwokrs
	 * @param household
	 */
	public void removeHousholdNode(Household household) {
		removeHouseholdNetworkNode(household);
		removeExtendedFamilyNode(household);
	}
	/**
	 * Remove household from social neighborhood
	 * @param household
	 */
	public void removeHouseholdNetworkNode(Household household) {
		HouseholdNode node = (HouseholdNode) this.householdNodeMap
		.get(new Integer(household.getId())).node;
		node.clearEdges();
		model.getHouseholdGraphLayout().getNodeList().remove(node);
		householdNodeMap.remove(new Integer(household.getId()));
	}
	/**
	 * Remove household from extended family network
	 * @param household
	 */
	public void removeExtendedFamilyNode(Household household) {
		if (this.extendedFamilyNodeMap.containsKey(new Integer(household
				.getId()))) {
			HouseholdNode node = (HouseholdNode) this.extendedFamilyNodeMap
			.get(new Integer(household.getId())).node;
			node.clearEdges();
			model.getExtendedFamilyGraphLayout().getNodeList().remove(node);
			extendedFamilyNodeMap.remove(new Integer(household.getId()));
		}
	}
	/**
	 * Remove agent from friendship network
	 * @param agent
	 */
	public void removeAgentFriendNode(Agent agent) {
		if (!agentFriendNodeMap.containsKey(new Integer(agent.getId()))) {
			return;
		}
		AgentNode node = (AgentNode) this.agentFriendNodeMap.get(new Integer(
				agent.getId())).node;
		node.clearEdges();
		if (model.getAgentFriendsGraphLayout().getNodeList().contains(node)) {
			model.getAgentFriendsGraphLayout().getNodeList().remove(node);
		}
		agentFriendNodeMap.remove(new Integer(agent.getId()));
	}
	/**
	 * Add agent to savings network
	 * @param agent
	 */
	public void addSavingsNode(Agent agent) {
		if (savingsNodeMap.containsKey(new Integer(agent.getId()))) {
			return;
		}
		AgentNode node = new AgentNode(agent);
		savingsNodeMap.put(new Integer(agent.getId()), new AgentNodeMap(agent,
				node));
		savingsNodeList.add(node);
		model.getSavingsGraphLayout().getNodeList().add(node);
		if (agent.isInnovator())
			node.setColor(Color.LIGHT_GRAY);
		else
			node.setColor(Color.BLUE);
		node.setNodeLabel(" " + agent.getId());
		node.setID(agent.getId());
	}
	/**
	 * Remove agent from savings network
	 * @param agent
	 */
	public void removeSavingsNode(Agent agent) {
		if (savingsNodeMap.containsKey(new Integer(agent.getId()))) {
			AgentNode node = (AgentNode) this.savingsNodeMap.get(new Integer(
					agent.getId())).node;
			node.clearEdges();
			model.getSavingsGraphLayout().getNodeList().remove(node);
			savingsNodeMap.remove(new Integer(agent.getId()));
			// ### also update the (redundant) savingsNodeList!
			savingsNodeList.remove(node);
		}
	}
	/**
	 * Add agent to all networks
	 * @param agent
	 */
	public void addAgentNode(Agent agent) {
		addAgentFriendshipNode(agent);
		addSavingsNode(agent);
	}
	/**
	 * Add agent to friendship network
	 * @param agent
	 */
	public void addAgentFriendshipNode(Agent agent) {
		AgentNode node = new AgentNode(agent);
		if (!agentFriendNodeMap.containsKey(new Integer(agent.getId()))) {
			AgentNodeMap map = new AgentNodeMap(agent, node);
			agentFriendNodeMap.put(new Integer(agent.getId()), map);			
		}
		model.getAgentFriendsGraphLayout().getNodeList().add(node);
		node.setNodeLabel(" " + agent.getId());
		node.setID(agent.getId());
		makeFriendsLinks(agent);
	}
	/**
	 * Add household to all netwokrs
	 * @param household
	 */
	public void addHouseholdNode(Household household) {
		addHouseholdNetworkNode(household);
		addExtendedFamilyNode(household);
	}
	/**
	 * Add household to social neighborhood
	 * @param household
	 */
	public void addHouseholdNetworkNode(Household household) {
		HouseholdNode node = new HouseholdNode(household);
		householdNodeMap.put(new Integer(household.getId()),
				new HouseholdNodeMap(household, node));
		model.getHouseholdGraphLayout().getNodeList().add(node);
		node.setColor(Color.DARK_GRAY);
		node.setNodeLabel(" " + node.getHousehold().getId());
		node.setID(node.getHousehold().getId());
	}

	public void addExtendedFamilyNode(Household household) {
		HouseholdNode node = new HouseholdNode(household);
		extendedFamilyNodeMap.put(new Integer(household.getId()),
				new HouseholdNodeMap(household, node));
		model.getExtendedFamilyGraphLayout().getNodeList().add(node);
		node.setColor(Color.DARK_GRAY);
		node.setNodeLabel(" " + node.getHousehold().getId());
		node.setID(node.getHousehold().getId());
		makeExtendedFamilyLinks(node);		
	}

	/**
	 * Inherit social links for this new household from the parents' households
	 * @param household
	 * @param p1
	 * @param p2
	 */
	public void inheritSocialLinks(Household household, Household p1, Household p2) {
		addHouseholdNetworkNode(household);
		HouseholdNode householdNode = (HouseholdNode) this
		.getHouseholdNodeMap().get(new Integer(household.getId())).node;

		HouseholdNode p1Node = null, p2Node = null;
		ArrayList<Node> links = new ArrayList<Node>();

		if (p1 != null) {
			p1Node = (HouseholdNode) this.getHouseholdNode(p1);
			links.add(p1Node);
		}
		if (p2 != null) {
			p2Node = (HouseholdNode) this.getHouseholdNode(p2);
			links.add(p2Node);
		}		
		ArrayList<HouseholdNode> temp = new ArrayList<HouseholdNode>();
		//magic number
		if (Math.random() < 0.50 && p1Node != null) {
			temp = new ArrayList<HouseholdNode>(p1Node.getToNodes());		
		}	
		else if (p2Node != null){
			temp = new ArrayList<HouseholdNode>(p2Node.getToNodes());
		}

		SimUtilities.shuffle(temp);
		if (!temp.isEmpty() && temp.get(0) != null) {
			links.add(temp.get(0));			
		}		

		int size = 0;
		if (temp.size() > 0) {
//			magic number
			size = Uniform.staticNextIntFromTo(1, 3);
		}
		else {
//			magic number
			size = Uniform.staticNextIntFromTo(2, 5);
		}

		ArrayList<Agent> friends = new ArrayList<Agent>(household.getHead().getFriends());
		SimUtilities.shuffle(friends);
		for (Agent friend : friends) {
			if (size <= 0) {
				break;
			}
			Household friendHousehold = friend.getHousehold();
			HouseholdNode friendHouseholdNode = 
				(HouseholdNode) this.getHouseholdNode(friendHousehold);
			if (friendHouseholdNode != null) {
				links.add(friendHouseholdNode);
				size--;				
			}
		}		

		if (!links.isEmpty()) {
			householdNode.constructEdges(links);	
		}		
	}


	/**
	 * inherit parents' links in this new household ...
	 */
	public void inheritLinks(Household household, Household p1, Household p2) {
		addHouseholdNetworkNode(household);
		HouseholdNode householdNode = (HouseholdNode) this
		.getHouseholdNodeMap().get(new Integer(household.getId())).node;
		// this is an undirected relation ... so any would do ...
		if (p1 != null) {
//			HouseholdNode p1Node = (HouseholdNode) this.getHouseholdNodeMap()
//			.get(new Integer(p1.getId())).node;
			HouseholdNode p1Node = (HouseholdNode) this.getHouseholdNode(p1);
			if (p1Node != null) {
				householdNode.constructEdges(p1Node.getToNodes());
			}						
		}
		if (p2 != null) {
//			HouseholdNode p2Node = (HouseholdNode) this.getHouseholdNodeMap()
//			.get(new Integer(p2.getId())).node;
			HouseholdNode p2Node = (HouseholdNode) this.getHouseholdNode(p2);
			if (p2Node != null) {
				householdNode.constructEdges(p2Node.getToNodes());
			}
		}
		normalizeHouseholdNodes(householdNode);
	}
	/**
	 * Remove some social neighborhood links randomly
	 * @param householdNode
	 */
	public void normalizeHouseholdNodes(HouseholdNode householdNode) {		
		int size = householdNode.getToNodes().size();
		for (int i = 0; i < size; i++) {
			if (Math.random() < 0.10) {
				householdNode.removeRandomLink();
			}
		}
		if (size < 2) {
			size = Uniform.staticNextIntFromTo(2, 5);
		}
		ArrayList<Agent> friends = new ArrayList<Agent>(householdNode.getHousehold().getHead().getFriends());
		SimUtilities.shuffle(friends);
		for (Agent friend : friends) {
			if (size <= 0) {
				break;
			}
			Household friendHousehold = friend.getHousehold();
			HouseholdNode friendHouseholdNode = 
				(HouseholdNode) this.getHouseholdNode(friendHousehold);
			if (friendHouseholdNode != null) {
				householdNode.makeEdgeToFrom(friendHouseholdNode);
				size--;				
			}
		}		
	}

	/**
	 * Return list of neighbors for this household
	 * @param household
	 * @return
	 */
	public ArrayList<Household> returnNeighbors(Household household) {
		HouseholdNode householdNode = (HouseholdNode) this.getHouseholdNodeMap()
		.get(new Integer(household.getId())).node;
		ArrayList<Household> neighbors = new ArrayList<Household>();		
		for (HouseholdNode neighborNode : new ArrayList<HouseholdNode>(householdNode.getToNodes())) {
			if (neighborNode != null) {
				neighbors.add(neighborNode.getHousehold());
			}
		}
		return neighbors;
	}
	/**
	 * get agents node list
	 * @return
	 */
	public ArrayList<AgentNode> getAgentNodeList() {
		return agentNodeList;
	}
	/**
	 * get households node list
	 * @return
	 */
	public ArrayList<HouseholdNode> getHouseholdNodeList() {
		return neighborhoodNodeList;
	}
	/**
	 * Maps agents to nodes
	 */
	public class AgentNodeMap {
		public Agent agent;
		public AgentNode node;
		public AgentNodeMap(Agent _agent, AgentNode _node) {
			this.agent = _agent;
			this.node = _node;
		}
	}
	/**
	 * Maps households to nodes
	 *
	 */
	public class HouseholdNodeMap {
		public Household household;
		public HouseholdNode node;
		public HouseholdNodeMap(Household _household, HouseholdNode _node) {
			this.household = _household;
			this.node = _node;
		}
	}

	public HashMap<Integer, AgentNodeMap> getAgentNodeMap() {
		return agentNodeMap;
	}

	public HashMap<Integer, HouseholdNodeMap> getHouseholdNodeMap() {
		return householdNodeMap;
	}

	public HouseholdNode getHouseholdNode(Household household) {
		HouseholdNode node = null;
		if (household == null) {
			return null;
		}
		if (householdNodeMap.containsKey(new Integer(household.getId()))) {
			node = (HouseholdNode) (householdNodeMap.get(new Integer(household
					.getId()))).node;
		}
		return node;
	}

	public void saveList(String prefix, ArrayList<Node> nodeList) {
//		saveNodeListPajek(model.getExtendedFamilyGraphLayout().getNodeList(), prefix);
//		saveNodeListPajek(model.getHouseholdGraphLayout().getNodeList(), prefix);
		saveNodeListPajek(nodeList, prefix);
		saveListKS(nodeList, prefix);

		/*ArrayList<AgentNode> nodeList = 
			new ArrayList<AgentNode>(model.getAgentFriendsGraphLayout().getNodeList());

		PrintWriter netRecorder;		
		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".txt")));
			for (Node n : nodeList) {
				netRecorder.println(n.getNumOutEdges() + ", ");
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();					
		}	

		 */		

//		MultiAffiliation multi = new MultiAffiliation();
//		ArrayList<ArrayList> actorsCollection = new ArrayList<ArrayList>();
		// multi.setActors(this.agentFriendsNodeList);
		// actorsCollection.add(this.agentFriendsNodeList);
		// actorsCollection.add(this.savingsNodeList);
		// multi.setActorsCollection(actorsCollection);
		// multi.saveAffiliationNetworkPajek(prefix);

		// SJA: oct 02. 2006
		/*
		 * multi.setActors(this.householdNodeList);
		 * actorsCollection.add(this.householdNodeList);
		 * actorsCollection.add(this.extendedFamilyNodeList);
		 * multi.setEvents(actorsCollection);
		 */

		// multi.saveAffiliationNetworkPajek(prefix);
		// SJA: oct 02. 2006
		/* multi.saveAffiliationNetworkUCI(prefix); */
	}

	/** Original version from Friedrich. Adapted for our case. Saves network in Pajek format
	 */
//	public void saveNodeListPajek(ArrayList<HouseholdNode> nodeList, String prefix) {
	public void saveNodeListPajek(ArrayList<Node> nodeList, String prefix) {
		PrintWriter netRecorder;
		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".net")));
			netRecorder.println("*Vertices " + nodeList.size());			

			for (Node n : nodeList) {
				netRecorder.println(nodeList.indexOf(n) + 1 + " \""
						+ n.getNodeLabel() + "\" 1 1");
			}
			netRecorder.println("*Arcs");
			for (Node n : nodeList) {
				ArrayList<Node> toNodeList = n.getOutNodes();
				for (Node tn : toNodeList) {
					netRecorder.println((nodeList.indexOf(n) + 1) + " "
							+ (nodeList.indexOf(tn) + 1) + " 1");
				}
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	/**Original version from Friedrich. Adapted for our case. Saves network for KS calculation */
	public void saveListKS(ArrayList<Node> nodeList, String prefix) {
		PrintWriter netRecorder;		
		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".txt")));
			for (Node n : nodeList) {
				netRecorder.println(n.getNumOutEdges() + ", ");
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();					
		}		
	}
	/**
	 * Original version from Friedrich. Adapted for our case. Saves network for motif analysis
	 * @param nodeList
	 * @param prefix
	 */
	public void saveNodeListMotif(ArrayList<Node> nodeList, String prefix) {
		PrintWriter netRecorder;
		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".net")));
			for (Node n : nodeList) {
				ArrayList<AgentNode> toNodeList = n.getToNodes();
				for (Node tn : toNodeList) {
					if (nodeList.indexOf(n) > nodeList.indexOf(tn)) {
						netRecorder.println((nodeList.indexOf(n) + 1) + " "
								+ (nodeList.indexOf(tn) + 1) + " 1");
					}
				}
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public ArrayList<AgentNode> getAgentFriendsNodeList() {
		return agentFriendsNodeList;
	}

	public ArrayList<AgentNode> getSavingsNodeList() {
		return savingsNodeList;
	}

	public HashMap<Integer, AgentNodeMap> getSavingsNodeMap() {
		return savingsNodeMap;
	}

	public HashMap<Integer, AgentNodeMap> getAgentFriendNodeMap() {
		return agentFriendNodeMap;
	}

	public HashMap<Integer, HouseholdNodeMap> getExtendedFamilyNodeMap() {
		return extendedFamilyNodeMap;
	}

	public ArrayList<HouseholdNode> getExtendedFamilyNodeList() {
		return extendedFamilyNodeList;
	}
	/**
	 * Returns the wealth statistics for all households
	 * @author shah
	 *
	 */
	class SingleHouseholdStatistics implements DataSource, Sequence {
		private Household household;
		public SingleHouseholdStatistics(Household _household) {
			this.household = _household;
		}
		public Object execute() {
			return new Double(getSValue());
		}
		public double getSValue() {
			HouseholdNode node = (HouseholdNode) household
			.returnHouseholdNode();
			return node != null ? node.getInDegree() : 0;
		}
	};
}