package health;

import infrastructure.Settings;

import java.util.ArrayList;
import java.util.HashMap;

import cern.jet.random.Uniform;

import units.Agent;

/**
 * The total cost of providing the drugs to everybody needing them will be
 *  between US $1 billion and $1.09 billion by 2005, according to the findings of a 
 *  joint health and treasury task team. Their report, on the costing of a national treatment plan, 
 *  led to the cabinet's decision on 8 August to go ahead with the programme.
 * <http://www.scienceinafrica.co.za/2003/september/aids.htm>
 *
 * WHO - Antiretroviral Therapy http://www.who.int/hiv/topics/arv/en/
 * Antiretroviral drugs inhibit the replication of HIV. When antiretroviral drugs are given 
 * in combination,  HIV replication  and immune deterioration can be delayed, 
 * and survival and quality of life improved.
 * 
 * From Avert.org/Pregnancy: "It is worth noting that someone is less likely to transmit HIV 
 * if they are receiving effective antiretroviral treatment, and also if neither they nor their 
 * partner has any other sexually transmitted infections."
 *
 * SJA: we're assuming a health unit - which is a hospital and not a mobile health unti
 */

public class Clinic {
	ArrayList<Agent> patients = new ArrayList<Agent>();
	HashMap<Integer, AgentClinicalProfile> patientsProfile = new HashMap<Integer, 
																AgentClinicalProfile>();
	
	public ArrayList<Agent> getPatients() {
		return patients;
	}

	public void setPatients(ArrayList<Agent> patients) {
		this.patients = patients;
	}
	
	/**
	 * Enrol the patient to the clinic
	 * @param patient
	 * @param currentTick
	 */
	public void addPatient(Agent patient, int currentTick) {
		if (this.patients.contains(patients)) {
			return;
		}
		this.patients.add(patient);		
		AgentClinicalProfile profile = new AgentClinicalProfile(patient, currentTick);
		
		if (!patientsProfile.containsKey(new Integer(patient.getId()))) {
			patientsProfile.put(new Integer(patient.getId()), profile);
		}		
		startTreatment(patient, currentTick);				
	}
	
	/**
	 * Returns the profile for this agent
	 * @param patient
	 * @return
	 */
	public AgentClinicalProfile returnProfile(Agent patient) {
		AgentClinicalProfile profile = null;
		if (patientsProfile.containsKey(new Integer(patient.getId()))) {
			profile = (AgentClinicalProfile) patientsProfile.get(new Integer(patient.getId()));
		}
		return profile;		
	}
	
	public void startTreatment(Agent patient, int currentTick) {
		AgentClinicalProfile profile = returnProfile(patient);
		if (profile == null) {
			return;
		}
		
		if (patient.getGender() == Settings.GENDER.FEMALE
			&& patient.hasHIV()) {
			//health is critical ... the patient needs ARV Treatment now!
			if 	(patient.getHealthStatus() == Settings.HEALTH_STATUS.AIDS
				//that is the patient is not critical but requires
				|| (patient.getPregnancyTick() > 4)) {
//				profile.setTypeOfTreatment(Settings.ARV_TREATMENT.AZT_WITH_NEVIRAPINE);				
//				profile.setTypeOfTreatment(Settings.ARV_TREATMENT.SINGLE_DOSE_NEVIRAPINE);
				profile.setTypeOfTreatment(Settings.ARV_TREATMENT.NO_TREATMENT_CURRENTLY);
			}
			else {
//				profile.setTypeOfTreatment(Settings.ARV_TREATMENT.INFANT_TREATMENT_ONLY);
				profile.setTypeOfTreatment(Settings.ARV_TREATMENT.NO_TREATMENT_CURRENTLY);
			}
		}		
		//TODO: set scenarios for Nevirapine ... 
		profile.setChanceofMTC(returnChanceOfMTC(profile));		
	}
	
	/**
	 * Removes the patient from the clinic
	 * @param patient
	 */
	public void removePatient(Agent patient) {
		if (!this.patients.contains(patients)) {
			return;
		}
		this.patients.remove(patient);		
		
		if (!patientsProfile.containsKey(new Integer(patient.getId()))) {
			patientsProfile.remove(new Integer(patient.getId()));
		}
	}
	
	/**
	 * Returns chance for MTCT based on ARV
	 * @param agent: female
	 * */
	public double returnChanceOfMTC(AgentClinicalProfile profile) {
		int treatment = profile.getTypeOfTreatment();
		double chance = 100;
		switch (treatment) {
		case Settings.ARV_TREATMENT.NO_TREATMENT_CURRENTLY:
			break;
		case Settings.ARV_TREATMENT.AZT_WITH_NEVIRAPINE:
		case Settings.ARV_TREATMENT.TRIPLE_COMBINATION:
			//2%? magic number
			chance = Uniform.staticNextIntFromTo(2, 15);
			break;			
		case Settings.ARV_TREATMENT.SINGLE_DOSE_NEVIRAPINE:
			chance = Uniform.staticNextIntFromTo(50, 65);
			break;
		case Settings.ARV_TREATMENT.INFANT_TREATMENT_ONLY:
			chance = Uniform.staticNextIntFromTo(2, 15);
			break;
			default: break;			
		}
		return ((double)(chance/100)*Settings.MTCT_RATE);
	}
	
	public class AgentClinicalProfile {
		Agent agent;
		int joinTick = -1;
		int leavingTick = -1;
		double chanceofMTC = 0;
		int typeOfTreatment = Settings.ARV_TREATMENT.NO_TREATMENT_CURRENTLY;
		
		public AgentClinicalProfile(Agent _agent, int currentTick) {
			this.agent = _agent;			
			this.joinTick = currentTick;			
		}

		public void setChanceofMTC(double chanceofMTC) {			
			this.chanceofMTC = chanceofMTC;
		}

		public void setJoinTick(int joinTick) {
			this.joinTick = joinTick;
		}

		public void setLeavingTick(int leavingTick) {
			this.leavingTick = leavingTick;
		}

		public int getTypeOfTreatment() {
			return typeOfTreatment;
		}

		public void setTypeOfTreatment(int typeOfTreatment) {
			this.typeOfTreatment = typeOfTreatment;
		}

		public double getChanceofMTC() {
			return chanceofMTC;
		}

		public int getJoinTick() {
			return joinTick;
		}

		public int getLeavingTick() {
			return leavingTick;
		} 
	}

	public HashMap<Integer, AgentClinicalProfile> getPatientsProfile() {
		return patientsProfile;
	}
}