package units;

import infrastructure.Settings;
import infrastructure.Settings.GRANTS;

import java.util.ArrayList;
import java.util.HashMap;

import cern.jet.random.Uniform;
import clubs.FuneralClub;
import economics.Employee;
import economics.Nutrition;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;

import display.*;
/**
 *Class representing a household in the population. 
 *
 */
public class Household {
	/** Household Id*/
	private int id = -1;
	/**Household head */
	private Agent head = null;
	/**Children in the household */
	private ArrayList<Agent> children = new ArrayList<Agent>();
	/**Adults in the household */
	private ArrayList<Agent> adults = new ArrayList<Agent>();
	/**Seniors in the household */
	private ArrayList<Agent> seniors = new ArrayList<Agent>();
	/**List of members in the household */	
	private ArrayList<Agent> members = new ArrayList<Agent>();
	/** only maintains the household ID ... we fetch the household from the household map in the Model*/ 
	private ArrayList<Integer> extendedFamilyIDList = new ArrayList<Integer>();
	/**List of married couples in the household */
	private ArrayList<Couple> coupleList = new ArrayList<Couple>();
	/**List of current members in the household */
	private ArrayList<Agent> currentMembers = new ArrayList<Agent>();
	private Model model;
	/**Funeral club of which the household is the members */
	private FuneralClub funeralClub = new FuneralClub();
	/**Church of which the household is the members */
	private Church church;
	/**Church Id of which the household is the members */
	private int churchID = -1;
	/**Has guardians? */
	private boolean noGuardians = false;
	/** Households members on migration */	
	private int currentMembersOnMigration = 0;
	/**Is member of funeral club */
	private boolean memberFuneralClub = false;
	/**number of funeral club memberships*/
	private int defaultFuneralClubCount = 0;
	/**Household's current wealth */
	private double wealth = 0;
	/**Employees maps for this household */
	private HashMap<Integer, Employee> employees = new HashMap<Integer, Employee>();
	/**Applicants for piecework jobs */
	private ArrayList<Agent> applicants = new ArrayList<Agent>();
	/**Number of available piecework jobs */
	private int numberAvailableJobs = 0;
	/**this indicates that there was at least one death this month /tick
	we'll see how we pay for it after we get the income, support from
	the funeral club and if possible, from the extended  family*/
	private boolean bereaved = false;
	/** Are household members hungry? */
	private boolean hungry = false;
	/**Default constructor */
	public Household(){		
	}
	/** Create household in simulation.*/
	public Household(int _id, Model _model){
		this.id = _id;
		this.model = _model;		
	}
	/**Get Id */
	public int getId() {
		return this.id;		
	}
	/**
	 * This an important method that assigns head, roles, wealth and other attributes ...
	 * depending upon the logic 
	 */ 
	public void initialize() {
		/* although as reported in Gina's study, out of 20 houses, 12 had male heads
		 * and the rest female ... we're having a random pick for the head here ... 
		 * we would like to make it more parameterized ... later on */
		SimUtilities.shuffle(this.adults);
		for (Agent agent : this.members) {
			if (agent.getGender() == Settings.GENDER.FEMALE
					&& agent.getAgeGroup() != Settings.AGE_GROUP.CHILD) {
				setHead(agent);
				return;
			}
		}				
		if (!this.adults.isEmpty()) {
			setHead(this.adults.get(0));	
		}
		else if (!this.seniors.isEmpty()) {
			setHead(this.seniors.get(0));
		}
		else {
			setHead(null);
		}
	}
	/**Add agent to household*/
	public void addAgent(Agent agent) {
		if (agent == null) {
			return;
		}		
		switch (agent.getAgeGroup()){
		case Settings.AGE_GROUP.CHILD:
			children.add(agent);
			break;		
		case Settings.AGE_GROUP.ADULT:
			adults.add(agent);
			break;
		case Settings.AGE_GROUP.SENIOR:
			seniors.add(agent);
			break;
		default: break;
		}		
		agent.setHouseHoldID(this.id);
		agent.setHousehold(this);
		this.members.add(agent);
		this.currentMembers.add(agent);
	}
	/** Removes the agent from the household  */
	public void removeAgent(Agent agent) {	
		switch (agent.getAgeGroup()){
		case Settings.AGE_GROUP.CHILD:
			if (children.contains(agent)) {
				children.remove(agent);	
			}			
			break;		
		case Settings.AGE_GROUP.ADULT:
			//either married or widowed ...
			int maritalStatus = agent.getMaritalStatus();
			if (maritalStatus == Settings.MARITAL_STATUS.MARRIED && !agent.isAlive()) {
				Agent spouse = model.getAgentMap().get(new Integer(agent.getSpouseID()));							
				if (spouse != null) {
					spouse.setMaritalStatus(Settings.MARITAL_STATUS.WIDOWED);
					if (spouse.isPregnant()) {
						spouse.setSingleMother(true);
						removeCouple(agent);
					}
				}				
				makeChildrenOrphan(agent);								
			}				
			if (adults.contains(agent)) {
				adults.remove(agent);
			}			
			break;
		case Settings.AGE_GROUP.SENIOR:
			if (seniors.contains(agent)) {
				seniors.remove(agent);
			}	
			break;
		default: break;		
		}		
		this.members.remove(agent);
		this.currentMembers.remove(agent);

		if (agent.isHead()) {		
			updateHead(agent);
		}		

		if (!noGuardians) {
			if (!memberFuneralClub
					&& (wealth - Settings.FUNERALCLUB_FEE > 0)
					&& (wealth < Settings.FUNERAL_COST / 2)
					&& agent.getHealthStatus() == Settings.HEALTH_STATUS.AIDS
					&& agent.getGender() == Settings.GENDER.MALE
					&& this.head != null) {
				if (this.head == null) {
					head = null;
				}
			}
			if (this.memberFuneralClub) {
				this.funeralClub.request(this);			
			}							
			this.bereaved = true;
		}			
	}	
	/** Removes the couple for this agent from the household  */
	public void removeCouple(Agent agent) {
		Couple temp = new Couple();
		for (Couple couple : this.coupleList) {
			if (couple.getFemale() == agent ||
					couple.getMale() == agent) {
				temp = couple;
			}
		}
		this.coupleList.remove(temp);
	}
	//magic number
	/** Deduct the expenditure for the funeral */
	public void deductFuneralCost() {
		//i am really wealthy so I will have a full-fledged funeral arranged. 
		//magic number
		if (this.wealth - 2 * Settings.FUNERAL_COST > 0) {
			deductWealth((double) Settings.FUNERAL_COST);
			return;
		}
		//i m not that wealthy so I will lessen the funeral cost
		int cost = Random.uniform.nextIntFromTo(Settings.FUNERAL_COST * 3/4, 
				Settings.FUNERAL_COST);
		borrowFuneralCost(cost);
		deductWealth(cost);							
		setBereaved(false);
	}
	/**
	 * Borrow money for the funeral.
	 * @param cost
	 */
	public void borrowFuneralCost(int cost) {
		ArrayList<Integer> temp = new ArrayList<Integer>(this.extendedFamilyIDList);
		if (temp.size() == 0) {
			return;
		}		
		int contribution = cost / temp.size();
		for (Integer id : temp) {		
			Household relative = (Household) model.getHouseMap().get(id);
			if (model.getHouseholdList().contains(relative)) {
				if (relative.getWealth() > cost/2) {
					relative.deductWealth(cost);
					this.addWealth(cost);					
				}
				else if (relative.getWealth() > contribution) {
					relative.deductWealth(contribution);
					this.addWealth(contribution);
				}
			}
		}
		int diff = cost - (int) this.wealth;
		if (diff <= 0) {
			return;
		}	
		for (Integer id : temp) {		
			Household relative = (Household) model.getHouseMap().get(id);
			if (model.getHouseholdList().contains(relative)) {
				if (relative.getWealth() > diff/2) {
					relative.deductWealth(diff/2);
					this.addWealth(diff);
					if (this.wealth > cost) {
						break;
					}
				}
			}
		}		
	}
	/**
	 *orphan means 'single OR double orphan'
	 */
	public void makeChildrenOrphan(Agent agent) {
		for (Agent child : agent.getOffSprings()) {
			if (child != null) {
				if (model.getAgentList().contains(child)
						&& child.isAlive()) {
					child.setOrphan(true);
				}
			}
		}
	}	
	/**Update members when age groups change */
	public void updateAgentAgeGroup(Agent agent, int ageGroup) {
		if (ageGroup == Settings.AGE_GROUP.ADULT) {
			this.children.remove(agent);
			this.adults.add(agent);			
		}
		else if (ageGroup == Settings.AGE_GROUP.SENIOR) {
			this.adults.remove(agent);
			this.seniors.add(agent);
		}
	}
	/**Get household members */
	public ArrayList<Agent> getMembers() {
		return this.members;
	}
	/**Get household size */
	public int getSize() {
		return this.members.size();
	}
	/**Get household wealth */
	public double getWealth() {
		return this.wealth;
	}
	/**
	 * Set household members 
	 * @param _wealth
	 */
	public void setWealth(double _wealth) {
		this.wealth = _wealth;
	} 	
	/**Get household children */
	public ArrayList<Agent> getChildren(){
		return this.children;
	}
	/**Get household adults */
	public ArrayList<Agent> getAdults(){
		return this.adults;
	}
	/**Get household seniors */
	public ArrayList<Agent> getSeniors(){
		return this.seniors;
	}	
	/**Get household head */
	public Agent getHead() {
		return head;
	}
	/**
	 * Set household members 
	 * @param _head
	 */
	public void setHead(Agent _head) {
		this.head = _head;
		if (_head != null) {
			_head.setHead(true);
		}
	}
	/**Main step for the household: accumulate income, feeds members and deducts expenditure */
	public void step() {
		setHungry(false);
		double income = accumulateIncome();		
		addWealth(income);
		//feed() returns the expenditure for food which is deducted from current wealth		
//		deductWealth(feedHousehold());		//FAO version
		deductWealth(feedHouseholdWithoutFAO());
//		deductWealth(feedWithoutFarm());
		if(model.isHealthExpenditure()) {
			deductHealthExpenditure();
		}				
		deductClubsFee();
		deductEducationFee();
	}
	/** FANRPAN Working Document: Series Ref. Number: NAT SAA005 IMPACT OF HIV & AIDS ON AGRICULTURE AND FOOD
		SECURITY: THE CASE OF LIMPOPO PROVINCE IN SOUTH AFRICA 2006
	 */
	public void deductEducationFee() {
//		magic number
		int schoolExpenditure = 100;
		SimUtilities.shuffle(this.currentMembers);
		for (Agent agent : this.currentMembers) {
			if (agent.isAtSchool() 
					&& agent.getAchievedEducation().compareTo("tertiary") != 0) {
				if (this.wealth - schoolExpenditure > 0) {
					deductWealth(schoolExpenditure);
				}
				else {
					agent.setAtSchool(false);
				}
			}
		}
	}

	/**
	 * Deduct health expenditure
	 *
	 */
	public void deductHealthExpenditure() {	
		int countIlls = 0;
		for (Agent agent : this.members) {
			if(agent.getHealthStatus() == Settings.HEALTH_STATUS.AIDS
//			if((agent.getHealthStatus() == Settings.HEALTH_STATUS.HIV
//					&& currentTick - agent.getInfectedTick() > agent.getInfectedLag()
					//SJA Sept 07
//					&& agent.getAgeGroup() != Settings.AGE_GROUP.CHILD
								
			|| (agent.getHealthStatus() == Settings.HEALTH_STATUS.ILL)
			|| (agent.getHealthStatus() == Settings.HEALTH_STATUS.OLD_AGE_SICKNESS)) {
				countIlls++;
			}					
		}		
		//magic number ... this is an assumption 
		deductWealth(countIlls * Settings.HEALTH_EXPENSE);		
	}

	/**Feed household members without the FAO recommendations*/
	public int feedHouseholdWithoutFAO() {		
		//list of those hh members who've not migrated .. 
		//we need to treat them case-by-case		
		ArrayList<Agent> temp = new ArrayList<Agent>();	

		//SJA: Mar 2008 to do
		for (Agent agent : this.members) {
			if (agent.isMigrated()) {
				agent.setFeedStatus(Settings.FEED_STATUS.FULL_FED);
			}
			else if (agent.getAgeGroup() == Settings.AGE_GROUP.ADULT)  {
				temp.add(agent);
			}
		}

		int childRequirement=0, seniorRequirement = 0, adultRequirement = 0, expenditure=0;
		/* just checking off the fermi functions */

		int moneyToFeed = Random.uniform.nextIntFromTo(Settings.MIN_MONEY_TO_FEED,
				Settings.MAX_MONEY_TO_FEED);

		childRequirement = this.children.size() * moneyToFeed;
		seniorRequirement = this.seniors.size() * moneyToFeed;
		//adults who haven't migrated
		adultRequirement = temp.size() * moneyToFeed;

		int totalRequirement = childRequirement + seniorRequirement + adultRequirement;
		//------------------------------------------------------
		/* IF (the money I have is more than the food-requirement) 
		 * AND (I am not a member of any burial society) 
		 * THEN all household members are Fully-Fed.
		 */		
		/*
		 * OR
		 */
		/* IF (I am the member of a burial society) 
		 * AND (after deducting the membership fee I still have more money than the food-requirement)
		 * THEN all household members are Fully-Fed. 
		 */

		if (totalRequirement < wealth) {
			feed(Settings.FEED_STATUS.FULL_FED, Settings.FEED_STATUS.FULL_FED,
					Settings.FEED_STATUS.FULL_FED, temp);
			expenditure = totalRequirement;				
		}
		else if (childRequirement + seniorRequirement/2 + adultRequirement/2 < wealth) {
			feed(Settings.FEED_STATUS.FULL_FED, Settings.FEED_STATUS.HALF_FED,
					Settings.FEED_STATUS.HALF_FED, temp);
			expenditure = childRequirement + seniorRequirement/2 + adultRequirement/2;

			model.numHHLackingFood++;
		}
		else if (childRequirement/2 + seniorRequirement/2 + adultRequirement/2 < wealth) {
			feed(Settings.FEED_STATUS.HALF_FED, Settings.FEED_STATUS.HALF_FED,
					Settings.FEED_STATUS.HALF_FED, temp);
			expenditure = childRequirement/2 + seniorRequirement/2 + adultRequirement/2;
			
			model.numHHLackingFood++;
		}
		else {
//			feed(Settings.FEED_STATUS.HALF_FED, Settings.FEED_STATUS.HALF_FED,
//			Settings.FEED_STATUS.NOT_FED, temp);
			feed(Settings.FEED_STATUS.HALF_FED, Settings.FEED_STATUS.NOT_FED,
					Settings.FEED_STATUS.NOT_FED, temp);
			setHungry(true);
			expenditure = childRequirement;
			
			model.numHHLackingFood++;
		}
		/* IF (I am the member of a burial society) 
		 * AND (after deducting the membership fee I have less money than the food-requirement)
		 * 		AND
		 * IF (I have money to completely fulfill children food-requirement) 
		 * AND (I have the money to half-fulfill the adults food-requirement)
		 * THEN (children are Fully-Fed AND adults are Half-Fed)
		 */
		/*		
		> IF (I am the member of a burial society) AND (after deducting the membership fee I have less money than the food-requirement)
		> AND
		> IF (I have money to half-fulfill children food-requirement) AND (I have the money to half-fulfill some adults food-requirement)
		> THEN (all children are Half-Fed AND some adults are Half-Fed AND rest of the adults are Not-Fed)
		>
		IF CHANGED TO ABOVE, THEN THIS ONE COULD BE CHILDREN 1/2 FED AND ADULTS 1/4 FED.. (OR SOME EAT ELSEWHERE)
		> IF (I am the member of a burial society) AND (after deducting the membership fee I have less money than the food-requirement)
		> AND
		> IF (I have only the money to half-fulfill children food-requirement OR less than that) AND (very less money left) ?
		> //this is tricky! there is underlying assumption that children do get at least half-fed by some other sources, e.g. asking or begging etc.
		> THEN (I dont pay the membership fee) AND (all children are Half-Fed AND all adults are Not-Fed)		
		 */				
		return expenditure;
	}	

	/**Feed household members without the farming*/
	public int feedWithoutFarm() {				
		int feedStatus = Settings.FEED_STATUS.NOT_FED;
//		int requirement = this.members.size() * Settings.MONEY_TO_FEED;
		int requirement = (this.members.size() - this.currentMembersOnMigration) * 65;
		if (wealth >= requirement) {
			feedStatus = Settings.FEED_STATUS.FULL_FED;
		}
		else if ((wealth < requirement) && (wealth > requirement / 2)) {
			feedStatus = Settings.FEED_STATUS.HALF_FED;
			requirement/=4;
		}
		else {
			this.hungry = true;
		}

		for (Agent agent : this.members) {
			if (!agent.isMigrated()) {
				agent.setFeedStatus(feedStatus);
			}
			else {
				agent.setFeedStatus(Settings.FEED_STATUS.FULL_FED);
			}
		}
		return requirement;
	}	
	/**there's an underlying assumption, which is lightly supported by anecdotal accounts
	 those members of the households on migration are not included in feeding ... this could
	 be a sensitive assumption ... however, let it be like that. 
	 */
	public int feedHousehold() {		
		//list of those hh members who've not migrated .. 
		//we need to treat them case-by-case		
		ArrayList<Agent> temp = new ArrayList<Agent>();		

		for (Agent agent : this.members) {
			if (agent.isMigrated()) {
				agent.setFeedStatus(Settings.FEED_STATUS.FULL_FED);
			}
			else if (agent.getAgeGroup() == Settings.AGE_GROUP.ADULT)  {
				temp.add(agent);
			}
		}
		/** from Radar: if there's a lack of food, typically older members of the 
		 * household go without food and give food to the children
		 * we need to feed the children first ... their requirements
		 * for this model, children are first fed and then we see what
		 * can be made available for the adults ... the minimum is 50% of
		 * the adults intake and reaches steadily  
		 */		
		int childRequirement=0, seniorRequirement = 0, adultRequirement = 0, expenditure=0;
		/* just checking off the fermi functions */

		int moneyToFeed = Random.uniform.nextIntFromTo(Settings.MIN_MONEY_TO_FEED,
				Settings.MAX_MONEY_TO_FEED);

		for (Agent child : this.children) {			
//			childRequirement += returnRequirementFermi(child);
			childRequirement += (returnRequirementFAO(child)*moneyToFeed*0.01);
		}

		for (Agent senior : this.seniors) {
			seniorRequirement += (returnRequirementFAO(senior)*moneyToFeed*0.01);
		}	


		//adults who haven't migrated		
		for (Agent agent : temp) {
//			nonChildRequirement += returnRequirementFermi(agent);
//			nonChildRequirement += (returnRequirementFAO(agent)*moneyToFeed*0.01);
			adultRequirement += (returnRequirementFAO(agent)*moneyToFeed*0.01);
		}		

		int totalRequirement = childRequirement + seniorRequirement + adultRequirement;

		//------------------------------------------------------
		/* IF (the money I have is more than the food-requirement) 
		 * AND (I am not a member of any burial society) 
		 * THEN all household members are Fully-Fed.
		 */		
		/*
		 * OR
		 */
		/* IF (I am the member of a burial society) 
		 * AND (after deducting the membership fee I still have more money than the food-requirement)
		 * THEN all household members are Fully-Fed. 
		 */

		if (totalRequirement < wealth) {
			feed(Settings.FEED_STATUS.FULL_FED, Settings.FEED_STATUS.FULL_FED,
					Settings.FEED_STATUS.FULL_FED, temp);
			expenditure = totalRequirement;				
		}
		else if (childRequirement + seniorRequirement/2 + adultRequirement/2 < wealth) {
			feed(Settings.FEED_STATUS.FULL_FED, Settings.FEED_STATUS.HALF_FED,
					Settings.FEED_STATUS.HALF_FED, temp);
			expenditure = childRequirement + seniorRequirement/2 + adultRequirement/2;								
		}
		else if (childRequirement/2 + seniorRequirement/2 + adultRequirement/2 < wealth) {
			feed(Settings.FEED_STATUS.HALF_FED, Settings.FEED_STATUS.HALF_FED,
					Settings.FEED_STATUS.HALF_FED, temp);
			expenditure = childRequirement/2 + seniorRequirement/2 + adultRequirement/2;							
		}
		else {
			feed(Settings.FEED_STATUS.HALF_FED, Settings.FEED_STATUS.NOT_FED,
					Settings.FEED_STATUS.NOT_FED, temp);
			expenditure = childRequirement;							
			setHungry(true);
		}
		/* IF (I am the member of a burial society) 
		 * AND (after deducting the membership fee I have less money than the food-requirement)
		 * 		AND
		 * IF (I have money to completely fulfill children food-requirement) 
		 * AND (I have the money to half-fulfill the adults food-requirement)
		 * THEN (children are Fully-Fed AND adults are Half-Fed)
		 */
		/*		
		> IF (I am the member of a burial society) AND (after deducting the membership fee I have less money than the food-requirement)
		> AND
		> IF (I have money to half-fulfill children food-requirement) AND (I have the money to half-fulfill some adults food-requirement)
		> THEN (all children are Half-Fed AND some adults are Half-Fed AND rest of the adults are Not-Fed)
		>
		IF CHANGED TO ABOVE, THEN THIS ONE COULD BE CHILDREN 1/2 FED AND ADULTS 1/4 FED.. (OR SOME EAT ELSEWHERE)
		> IF (I am the member of a burial society) AND (after deducting the membership fee I have less money than the food-requirement)
		> AND
		> IF (I have only the money to half-fulfill children food-requirement OR less than that) AND (very less money left) ?
		> //this is tricky! there is underlying assumption that children do get at least half-fed by some other sources, e.g. asking or begging etc.
		> THEN (I dont pay the membership fee) AND (all children are Half-Fed AND all adults are Not-Fed)		
		 */				
		return expenditure;
	}
	/**
	 * Feed the household members based on the decision to feed children, 
	 * adults and seniors
	 * @param childFeedStatus
	 * @param seniorFeedStatus
	 * @param adultFeedStatus
	 * @param adults
	 */
	public void feed (int childFeedStatus, int seniorFeedStatus,
			int adultFeedStatus, ArrayList<Agent> adults){
		for (Agent child : this.children) {
			child.setFeedStatus(childFeedStatus);
		}

		for (Agent senior : this.seniors) {
			senior.setFeedStatus(seniorFeedStatus);
		}

		if (adultFeedStatus == Settings.FEED_STATUS.NOT_FED) {
			model.getHungryAgents().addAll(adults);
		}
		else {
			for (Agent adult : adults) {			
				adult.setFeedStatus(adultFeedStatus);			
			}			
		}
	}
	/**
	 * Return the FAO Adult equivalence nutrition requirement for agent
	 * @param agent
	 * @return
	 */
	public int returnRequirementFAO (Agent agent) {
		Nutrition nurition = new Nutrition();
		if (agent.getGender() == Settings.GENDER.FEMALE) {
			return nurition.requiredFemalePerCentage(agent.getAge());			
		}
		else {
			return nurition.requiredMalePerCentage(agent.getAge());
		}
	}
	/**
	 * Deducts (or supposed to deduct)the fee for the funeral club or savings 
	 * club, the household is member of ... the membership is annual here ... 
	 * so every month, some money is accumulated in the fund for clubs ... 
	 * if there's no money at a moment, the household may get money out of it
	 * ... this may force a household to pull out of a savings club 
	 */
	public void deductClubsFee() {
		/*our first priority is to be able to pay for the funeral club
		 * we're member of ... 
		 */		
		if (this.head == null) {
			return;
		}		
		//if this hh is a memeber of a funeral club ...
		if (this.memberFuneralClub) {
			if ((int)this.wealth - Settings.FUNERALCLUB_FEE > 0) {				
				defaultFuneralClubCount--;		
				if (defaultFuneralClubCount < 0) defaultFuneralClubCount = 0;			
				deductWealth((double) Settings.FUNERALCLUB_FEE);
				this.funeralClub.addContribution(Settings.FUNERALCLUB_FEE);
			}
			else {
				defaultFuneralClubCount++;				
			}			
			if (this.defaultFuneralClubCount > 0) {
				model.numHHGracePeriod++;
			}
		}		
		//now for the savings club (stokvel)
		if (this.head.isMemberSavings()) {
			if (this.wealth - Settings.STOKVEL_AMOUNT > 0) {
				//if the head is the member of a savings club
				//deduct the club fee			
				deductWealth(Settings.STOKVEL_AMOUNT);
			}
			else {
				this.head.setPullOutMembership(true);
			}
		} 
	}		
	/**
	 * Accumulated incomes from the household members
	 * @return
	 */
	public double accumulateIncome() {
		//this is the income this month 
		double income = 0;
		this.currentMembersOnMigration = 0;
		for (Agent agent : this.members) {
			//recording
			if (!agent.isLaborFit()
					&& agent.getAgeGroup() == Settings.AGE_GROUP.ADULT) {
				model.numAgentsNotLaborFit++;
			}
			if (agent.getAgeGroup() != Settings.AGE_GROUP.CHILD
					&& !agent.isMigrated()
					&& !agent.isMineJob()
					&& !agent.isPieceJob()) {
				model.numAgentsJobless++;
			}
			if (agent.isMigrated()) {
				income += agent.returnMyRemittance();
				//recording
				this.currentMembersOnMigration++;
			}
			else if (agent.isPieceJob()
					&& agent.getAgeGroup() == Settings.AGE_GROUP.ADULT) {
				income += agent.returnMyPieceWorkEarning();
				//recording
				model.numAgentsPieceWork++;
			}
			else if (agent.isMineJob()) {
				income += agent.returnMyMineJobEarning();
				//recording
				model.numAgentsOnMineJob++;
			}
			else {
				income += agent.getGrantValue();
			}			
		}	
		for (Agent agent : this.members) {
			if (agent.getGrantValue() != GRANTS.NO_GRANT) {
				model.numHHReceivingGrants++;
				break;
			}
		}
		return income;
	}
	/** Update household head */
	public boolean updateHead(Agent agent) {	
		if (this.members.isEmpty()) {
			setHead(null);
			this.noGuardians = true;
			return false;
		}
		if (agent.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED) {
			int spouseId = agent.getSpouseID();
			for (Agent member : this.members) {
				if (member.getId() == spouseId) {
					this.noGuardians = false;
					setHead(member);
					agent.setHead(false);
					return true;
				}
			}
		}
		//there's at least one adult who hasn't migrated yet.
		//agent: already removed from the children, adults, and seniors list
		//but not from members list
		SimUtilities.shuffle(this.adults);
		if (!this.adults.isEmpty()) {
			for (Agent adult : this.adults) {
//				if (!adult.getIsAlive() && !adult.isMigrated()) {
				if (adult.isAlive() && adult.getId() != agent.getId()) {
					this.noGuardians = false;
					setHead(adult);
					agent.setHead(false);
					return true;
				}
			}
		}
		SimUtilities.shuffle(this.seniors); 
		if (!this.seniors.isEmpty()) {			
			for (Agent senior : this.seniors) {
				if (senior.isAlive() && senior.getId() != agent.getId()) {
					this.noGuardians = false;
					setHead(senior);
					agent.setHead(false);
					return true;						
				}
			}			
		}	
		//child headed-household
		if (!children.isEmpty()) { 
			for (Agent child : this.children) {
				//magic number
				if (child.isAlive() && child.getAge() > 13) {
					this.noGuardians = false;
					setHead(child);
					return true;						
				}
			}	
		}
		if (agent.isAlive()) {
			return false; 
		}
		else {
			this.noGuardians = true;
			setHead(null);
			agent.setHead(false);
			return true;
		}
	}
	/**Update the status that there are no guardians */
	public boolean getNoGuardians() {
		if (this.noGuardians || this.members.size() == 0) {
			this.noGuardians = true;
		}
		return this.noGuardians;
	}
	/**
	 * Add money from funeral contributions
	 * @param money
	 */
	public void addFuneralWealth (double money) {
		addWealth(money);
	}

	/**
	 * Add money to the household wealth
	 * @param money
	 */
	public void addWealth(double money) {
		this.wealth += money;		
	}
	/**
	 * deduct money from wealth
	 * @param money
	 */
	public void deductWealth(double money) {	
		if ((this.wealth - money) >= 0) {
			this.wealth -= money;
		}		
		else {
			this.wealth = 0;
		}		
	}
	/**
	 * Find a neighbour .. in future we would like to give preference to 
	 * those to whom we have stronger/multiple ties
	 */
	public Household accommodateNeighbour() {		
		HouseholdNode householdNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(new Integer(this.id)).node;		
		ArrayList<HouseholdNode> neighbourNodes = householdNode.getToNodes();
		SimUtilities.shuffle(neighbourNodes);
		for (HouseholdNode neighbour : neighbourNodes) {
			if (neighbour.getHousehold().getId() != this.id) {
//				System.out.println("Household : " + id + " neighbour-only: " + neighbour.getHousehold().getId());						  			
				return neighbour.getHousehold();
			}
		}
		return null;
	}
	/** Get funeral club*/
	public FuneralClub getFuneralClub() {
		return funeralClub;
	}
	/** Set funeral club*/
	public void setFuneralClub(FuneralClub funeralClub) {
		this.funeralClub = funeralClub;
	}
	/** Is member of a funeral club*/
	public boolean isMemberFuneralClub() {
		return memberFuneralClub;
	}
	/** Set if member of a funeral club*/
	public void setMemberFuneralClub(boolean _isMemberFuneralClub) {
		this.memberFuneralClub = _isMemberFuneralClub;
	}   

	/** now once decided to join a funeral club, we choose which one to join */  
	public FuneralClub findFuneralClub() {
		FuneralClub club = null;

		/*first we look if some other household in our hhcluster is a member 
		 * of any funeral club ... right now, we breakaway and choose the club
		 * as soon as we find the first hh in our cluster and returns its funeral club
		 */		
		for (Integer id : this.extendedFamilyIDList) {
			if (model.getHouseMap().containsKey(id)) {
				Household relative = (Household) model.getHouseMap().get(id);
				if ((relative.isMemberFuneralClub()) && (relative.getFuneralClub() != null)
						&& model.getHouseholdList().contains(relative)) {
					club = (FuneralClub) relative.getFuneralClub();
					break;
				}			
			}
		}		
		if (club != null) {
			return club;
		}

		/* if our earlier search didnt work .. we look into the household head's friend 
		 * ... no current justification, why we're doing this ...and nothing else
		 * and this is badly written ... can be optimized ... it will slow down an
		 * already very very very slow simulation  
		 */				

		for (Agent agent : this.head.getFriends()) {
			Household household = (Household) model.getHouseMap().get(new Integer(agent.getHouseHoldID()));
			if (model.getHouseholdList().contains(household)) {
				if ((household.isMemberFuneralClub()) && (household.getFuneralClub() != null)) {
					club = (FuneralClub) household.getFuneralClub();
					break;
				}				
			}
		}
		if (club != null) {
			return club;
		}
		else {
			//join a random club ...
			ArrayList<FuneralClub> temp = new ArrayList<FuneralClub>(model.getFuneralClubs());
			SimUtilities.shuffle(temp);
			if (temp.size() > 0) {
				club = (FuneralClub) temp.get(0);
			}						
		}				
		return club;
	}

	public int getDefaultFuneralClubCount() {
		return defaultFuneralClubCount;
	}

	public void setDefaultFuneralClubCount(int defaultFuneralClubCount) {
		this.defaultFuneralClubCount = defaultFuneralClubCount;
	}

	/**Get current members on job/labor */
	public int getCurrentMembersOnLabor() {
		int count = 0;
		for (Agent member : this.members) {
			if (member.isPieceJob()) {
				count++;
			}
		}
		return count;
	}	
	/**Get current adult members */
	public int getCurrentAdultMales() {
		int count = 0;
		for (Agent adult : this.adults) {
			if (!adult.isMigrated()) {
				count++;
			}					
		}
		return count;
	}	
	/**
	 * Accommodate new members of the dissolved household
	 * @param household
	 */
	public void accommodateNewMembers(Household household) {
		ArrayList<Agent> agents = new ArrayList<Agent>(household.getMembers());			
		for (Agent agent : agents) {
			this.addAgent(agent);
			household.removeAgent(agent);
		}				
		/*When a household dissolves and accommodated, 
		 * the assests goes to the accommodating household.  ... 
		 * source: HIV/AIDS: Beyond Epidemiology*/ 
		this.addWealth(household.getWealth());
		household.setWealth(0);
	}

	/**
	 * Add couple to this household
	 * @param couple
	 */
	public void addCouple(Couple couple) {
		if (!this.coupleList.contains(couple)) {
			this.coupleList.add(couple);	
		}		
	}
	/**
	 * Remove couple from this household
	 * @param couple
	 */
	public void removeCouple(Couple couple) {
		if (this.coupleList.contains(couple)) {
			this.coupleList.remove(couple);
		}
	}

	/** Get the couples list*/
	public ArrayList<Couple> getCoupleList() {
		return coupleList;
	}

	/**
	 * Set the couples list
	 * @param coupleList
	 */
	public void setCoupleList(ArrayList<Couple> coupleList) {
		this.coupleList = coupleList;
	}
	
	/**
	 * Get list of extended family households Ids
	 * @return
	 */
	public ArrayList<Integer> getExtendedFamilyIDList() {
		return extendedFamilyIDList;
	}

	/** this is for a household who happens to be a member of an extended family...
	 * we first check if
	 * 	- any adult sibling is alive and married of the children 
	 * 	- otherwise, we will look for grandParents households
	 * 	- otherwise we will look for maternal aunts, 
	 * 	- otherwise anyone from the extened family in the village
	 *  - else return nil. 
	 */
	public Household accommodateRelative () {
		Household newHousehold = new Household();			 		
		for (Agent child : this.children) {			
			//retieve child's grandparent's (paternal or maternal) household, if alive
			newHousehold = child.getGPHousehold();
			if (newHousehold != null 
					&& newHousehold.getId() != this.id
					&& newHousehold.getId() != -1) {
				return newHousehold;
			}			
		}		
		for (Agent child : this.children) {
			newHousehold = child.getElderSiblingOrAuntHousehold();
			if (newHousehold != null 
					&& newHousehold.getId() != this.id
					&& newHousehold.getId() != -1) {
				return newHousehold;
			}
		}
		if (!this.extendedFamilyIDList.isEmpty()) {
			newHousehold = (Household) chooseExtendedFamilyHousehold();				
		}		
		if (newHousehold != null 
				&& newHousehold.getId() != this.id
				&& newHousehold.getId() != -1) { 
		}
		else {
		}		
		return newHousehold;			
	}
	/** according to the Oct 2007 validation trip by Gina and Tak, 
	 * the women said that children are accommodated/taken care of 
	 * by the family on the fathers' side - as they belong to the 
	 * father's moiety.
	 * 
	 * 	choose the household which is best suited to accomodate this household's
	 * ... currently, we choose randomly, but then we need better logics here ..
	 * 
	 * It is actually implemented in here - as we're only adding the mother's hh
	 * to our extended-family list and nothing more. 
	 */
	public Household chooseExtendedFamilyHousehold() {		
		ArrayList<Integer> temp = new ArrayList<Integer>(this.extendedFamilyIDList);
		ArrayList<Household> temp1 = new ArrayList<Household>(), 
		temp2 = new ArrayList<Household>(); 		
		Household accommodator = null;
		for (Integer id : temp) {		
			Household newHousehold = (Household) model.getHouseMap().get(id);
			if (model.getHouseholdList().contains(newHousehold)
					&& newHousehold.getId() != this.id) {
				if (isNeighbour(newHousehold)) {
					temp1.add(newHousehold);
				}
				else {
					temp2.add(newHousehold);
				}				
			}
		}
		SimUtilities.shuffle(temp1);				
		//first check for relatives who are neighbours as well
		for (Household relNeighbour : temp1) {
			if (relNeighbour.getMembers().size() < 11) {
				accommodator = relNeighbour;			
				break;
			}
		}
		if (accommodator != null) {
//			System.out.println("Household : " + id + " found a relative-neighbour: " + accommodator.getId() 
//			+ " having size < 11 ");			
			setExtendedFamilyIDList(temp);
			return accommodator;
		}
		SimUtilities.shuffle(temp2);
		//then for those in the extended family
		for (Household relative : temp2) {
			if (relative.getMembers().size() < 11) {
				accommodator = relative;
//				System.out.println("Household : " + id + " found a relative-only: " + accommodator.getId() 
//				+ " having size < 11 ");									
				break;
			}
		}			
		if (accommodator == null) {
			if (!temp1.isEmpty()) {
				accommodator = temp1.get(0);
//				System.out.println("Household : " + id + " found a relative-neighbour: " + accommodator.getId() 
//				+ " having size >= 11 ");		
			}
			else if (!temp2.isEmpty()) {
				accommodator = temp2.get(0);
//				System.out.println("Household : " + id + " found a relative-only: " + accommodator.getId() 
//				+ " having size >= 11 ");			
			}
		}		
		setExtendedFamilyIDList(temp);		
		return accommodator;
	}

	/**
	 * Check if this is neighbor
	 * @param relativeHousehold
	 * @return
	 */
	public boolean isNeighbour(Household relativeHousehold) {
		HouseholdNode householdNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(new Integer(this.id)).node;		
		ArrayList<HouseholdNode> neighbourNodes = householdNode.getToNodes();
		SimUtilities.shuffle(neighbourNodes);
		for (HouseholdNode neighbour : neighbourNodes) {
			if (neighbour.getHousehold().getId() == relativeHousehold.getId()) {
				return true;
			}
		}		
		return false;
	}	
	/**
	 * We give this new hh its relatives following the Union of both husband and wife
	 * @param husbandHousehold
	 * @param wifeHousehold
	 */
	public void populateExtendedFamily(Household husbandHousehold, Household wifeHousehold) {			
		this.extendedFamilyIDList.add(husbandHousehold.getId());
		husbandHousehold.getExtendedFamilyIDList().add(this.id);
		if (model.getHouseholdList().contains(wifeHousehold)) {
			husbandHousehold.getExtendedFamilyIDList().add(wifeHousehold.getId());		
			this.extendedFamilyIDList.add(wifeHousehold.getId());
			wifeHousehold.getExtendedFamilyIDList().add(this.id);
			wifeHousehold.getExtendedFamilyIDList().add(husbandHousehold.getId());			
			this.model.getGui().createExtendedFamilyLink(husbandHousehold, wifeHousehold);
		}
		for (Integer id : husbandHousehold.getExtendedFamilyIDList()) {
			if (model.getHouseMap().containsKey(id)) {
				/// this has to be dyadic ... ?
				Household newHousehold = (Household) model.getHouseMap().get(id);
				if (!this.extendedFamilyIDList.contains(new Integer(newHousehold.getId()))) {
					this.extendedFamilyIDList.add(id);					
				}				
			}		
		}			
		if (!model.getHouseholdList().contains(wifeHousehold)) {
			Agent wife = this.coupleList.get(0).getFemale();
			Household wifeParentHousehold = (Household) model.getHouseMap().get(wife.getMaternalHouseholdID());
			if (model.getHouseholdList().contains(wifeParentHousehold)) {
				for (Integer id : wifeParentHousehold.getExtendedFamilyIDList()) {
					if (model.getHouseMap().containsKey(id)) {
						Household newHousehold = (Household) model.getHouseMap().get(id);
						if (!this.extendedFamilyIDList.contains(new Integer(newHousehold.getId()))) {
							this.extendedFamilyIDList.add(id);	
						}				
					}		
				}	
			}
			else {
				return;
			}
		}		
		for (Integer id : wifeHousehold.getExtendedFamilyIDList()) {
			if (model.getHouseMap().containsKey(id)) {
				Household newHousehold = (Household) model.getHouseMap().get(id);
				if (!this.extendedFamilyIDList.contains(new Integer(newHousehold.getId()))) {
					this.extendedFamilyIDList.add(id);	
				}				
			}		
		}		
	}	
	public void setExtendedFamilyIDList(ArrayList<Integer> extendedFamilyID) {
		this.extendedFamilyIDList = extendedFamilyID;
	}			
	/** Using a non-linear Fermi equation ... 
	 */
	public int returnRequirementFermi(Agent agent) {
		int age = agent.getAge();
		int moneyToFeed = Random.uniform.nextIntFromTo(Settings.MIN_MONEY_TO_FEED,
				Settings.MAX_MONEY_TO_FEED); 
		//magic number
		if (age >= 18) {
			return moneyToFeed;
		}
		else {
			double req = (0.50 + (model.getFermi().returnChildFoodIntake(age)))*moneyToFeed;
			return (int) req;
		}
	}	
	/**
	 * Return the best accommodator for my dissolved household
	 * @return
	 */
	public Household returnAccommodator() {		
		Household accommodator = null;		
		accommodator = (Household) accommodateRelative();    	
		//couldn't find an extended-family household
		if (accommodator == null) {			
			accommodator = (Household) accommodateNeighbour();
		}
		return accommodator;
	}
	/**
	 * Print my household composition
	 *
	 */
	public void printhouseholdComposition() {
		System.out.println("**********Household " + this.id
				+ " : Composition at time : " + (int)model.getTickCount() + " **********");
		System.out.println("The household has : " + this.members.size() + " members with " 
				+ this.seniors.size() + " seniors, " + this.adults.size() + " adults and "
				+ this.children.size() + " children.");
		System.out.println("The household has " + this.coupleList.size() + " couples at this time.");
		System.out.println("Funeral club membership: " + memberFuneralClub);
	}
	/**
	 * Am I bereaved
	 * @return
	 */
	public boolean isBereaved() {
		return bereaved;
	}
	/**
	 * Set my bereaved flag
	 * @param isBereaved
	 */
	public void setBereaved(boolean isBereaved) {
		this.bereaved = isBereaved;
	}
	/**
	 * Return my node from the network GUI
	 * @return
	 */
	public HouseholdNode returnHouseholdNode() {
		return (HouseholdNode) model.getGui().getHouseholdNode(this);
	}
	/**
	 * Can the household has the lobola
	 * @return
	 */
	public boolean hasLebola() {
		return (this.wealth > Settings.LOBOLA_AMOUNT) ? true : false;			
	}
	/**
	 * Get my church
	 * @return
	 */
	public Church getChurch() {
		return church;
	}
	/**
	 * Set my church
	 * @param church
	 */
	public void setChurch(Church church) {
		this.church = church;
	}
	/**
	 * Announce piecework jobs
	 * @return
	 */
	public boolean announceJobs() {
		this.numberAvailableJobs = 0;
		if (this.wealth > Settings.RICH_HOUSEHOLD_CRITERIA
				//magic number
				&& Math.random() < 0.75) {
			this.numberAvailableJobs = Uniform.staticNextIntFromTo(1, 3);
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * Get applicants list for piecwork
	 * @return
	 */
	public ArrayList<Agent> getAaplicants() {
		return applicants;
	}
	/**
	 * Get number of available piecework jobs
	 * @return
	 */
	public int getNumberAvailableJobs() {
		return numberAvailableJobs;
	}
	/**
	 * Allocate jobs to applicants.
	 *
	 */
	public void allocateJob() {
		SimUtilities.shuffle(this.applicants);
		ArrayList<Agent> temp = new ArrayList<Agent>(this.applicants);
		for (Agent agent : temp) {
			if (!agent.isPieceJob() && isKin(agent)) {
				recruitEmployee(agent);
				this.numberAvailableJobs--;
				this.applicants.remove(agent);
//				System.out.println("Me: " + this.id + ". Recruiting a kin. Remaining jobs: " + this.numberAvailableJobs);
			}
			if (this.numberAvailableJobs == 0) {
				break;
			}
		}		
		temp = new ArrayList<Agent>(this.applicants);		
		if (this.numberAvailableJobs > 0) {
			for (Agent agent : temp) {
				if (!agent.isPieceJob() && isReliable(agent)) {
					recruitEmployee(agent);
					this.numberAvailableJobs--;
					this.applicants.remove(agent);
//					System.out.println("Me: " + this.id + ". Recruiting Reliable. Remaining jobs: " + this.numberAvailableJobs);
				}										
				if (this.numberAvailableJobs == 0) {
					break;
				}
			}			
		}		
		if (this.applicants.isEmpty()) {
			return;
		}			
		int i = 0;
		while (this.numberAvailableJobs > 0 && i < this.applicants.size()-1) {
			Agent agent = this.applicants.get(i); 
			if (!agent.isPieceJob()) {
				recruitEmployee(temp.get(i));
//				System.out.println("Me: " + this.id + ". Recruiting Random. Remaining jobs: " + this.numberAvailableJobs);
				this.numberAvailableJobs--;
			}
			i++;		
		}
	}
	/**
	 * Is agent a kin to my household
	 * @param agent
	 * @return
	 */
	public boolean isKin(Agent agent) {
		boolean isKin = false;
		int agentHHId = agent.getHousehold().getId();
		for (Integer id : this.extendedFamilyIDList) {
			if (id.intValue() == agentHHId) {
				//this is a kin!!!
				isKin = true;				
				break;
			}
		}
		return isKin;		
	}
	/** TODO: ... IS A STUBS  */
	public boolean isReliable(Agent agent) {
		if (this.employees.containsKey(new Integer(agent.getId()))) {
			return true;
		}
		return false;		
	}
	/** 
	 * TODO: We could also increase some pay for those working for us regularly??
	 */
	public void recruitEmployee(Agent agent) {
		Employee employee;
		Integer employeeID = new Integer(agent.getId());
		if (this.employees.containsKey(new Integer(employeeID))) {
			employee = this.employees.get(new Integer(employeeID));
			int num = employee.getNumberOfEmployments() + 1;
			employee.setNumberOfEmployments(num);
			//TODO: To implement...currently a stub
			//one possibility is to also keep track of the time ticks 
			//for this agent ... and then use it for reliability updating...
			employee.updateReliability(this);
		}
		else {
			employee = new Employee();
			employee.setEmployee(agent);
			employee.setNumberOfEmployments(1);
			employee.updateReliability(this);
			this.employees.put(employeeID, employee);
		}
		agent.setPieceJob(true);
		agent.setEmployer(this);				
	}
	/**
	 * the underlying assumption is that there is at least one person in the 
	 * household who has gone hungry ... most likely, an adult or a senior...or an 
	 * entire household 
	 *
	 */
	public void borrowFood() {
		int hungryMembers = 0;

		for (Agent member : this.members) {
			if (member.getFeedStatus() == Settings.FEED_STATUS.NOT_FED) {
				hungryMembers++;
			}
		}
		double moneyToFeed = Random.uniform.nextDoubleFromTo(Settings.MIN_MONEY_TO_FEED/2,
				3/4*Settings.MIN_MONEY_TO_FEED);
		double requirement = (double) hungryMembers * moneyToFeed, borrowedMoney = 0;		
		HouseholdNode householdNode = (HouseholdNode) model.getGui()
										.getHouseholdNodeMap().get(new Integer(this.id)).node;

		ArrayList<Household> neighbors = new ArrayList<Household>();		
		for (HouseholdNode neighborNode : new ArrayList<HouseholdNode>(householdNode.getToNodes())) {
			if (neighborNode != null) {
				neighbors.add(neighborNode.getHousehold());
			}
		}		
		SimUtilities.shuffle(neighbors);		
		for (Household neighbor : neighbors) {
			if (Math.random() < 0.20) {
				double neighborWealth = neighbor.getWealth(); 
				if (neighborWealth > requirement * 4) {
					neighbor.deductWealth(requirement);
					borrowedMoney += requirement;
				}
				else if (neighborWealth > requirement * 2) {
					neighbor.deductWealth(requirement/2);
					borrowedMoney += requirement/2;					
				}
				else if (neighborWealth > requirement) {
					neighbor.deductWealth(requirement/4);
					borrowedMoney += requirement/4;
				}				
				if (borrowedMoney < requirement) {
					break;
				}
			}			
		}		
		if (borrowedMoney < moneyToFeed) {
			addWealth(borrowedMoney); 
			return;
		}		
		if (borrowedMoney >= 0.70 * requirement) {
			for (Agent member : this.members) {
				if (member.getFeedStatus() == Settings.FEED_STATUS.NOT_FED) {
					member.setFeedStatus(Settings.FEED_STATUS.HALF_FED);
				}
			}
		}		
	}

//	public boolean isRecruitedForJob(Agent agent) {
//	boolean isRecruited = false;
//	//also to check if we have employed him before or not ...  
//	if (this.employees.isEmpty()) {
//	this.employmentStrategy = Settings.EMPLOYMENT_STRATEGIES.RANDOM_STRATEGY;
//	}
//	else {
//	this.employmentStrategy = Settings.EMPLOYMENT_STRATEGIES.RELIABILITY_STRATEGY;			
//	}

//	this.employmentStrategy = Settings.EMPLOYMENT_STRATEGIES.KIN_FIRST_STRATEGY;

//	switch (this.employmentStrategy) {
//	case Settings.EMPLOYMENT_STRATEGIES.RANDOM_STRATEGY:
//	isRecruited = agent.isPieceJob() ? false : true;
//	break;
//	case Settings.EMPLOYMENT_STRATEGIES.KIN_FIRST_STRATEGY:
//	isRecruited = kinFirstStrategy(agent);
//	break;
//	case Settings.EMPLOYMENT_STRATEGIES.RELIABILITY_STRATEGY:
//	isRecruited = reliabilityStrategy(agent);
//	break;
//	default: break;
//	}
//	return isRecruited;
//	}	

//	public boolean kinFirstStrategy(Agent agent) {
//	boolean isKin = false;
//	ArrayList<Integer> agentKins = new ArrayList<Integer>(agent.getHousehold().getExtendedFamilyID());
//	for (Integer id : this.extendedFamilyID) {
//	if (agentKins.contains(id)) {
//	//this is a kin!!!
//	isKin = true;
//	break;
//	}
//	}
//	return isKin;
//	}

//	public boolean reliabilityStrategy(Agent agent) {
//	if (this.employees.containsKey(new Integer(agent.getId()))) {
//	return true;
//	}
//	return false;
//	}

	public HashMap<Integer, Employee> getEmployee() {
		return employees;
	}
	/**
	 * Am I hungry
	 * @return
	 */
	public boolean isHungry() {
		return hungry;
	}
	/**
	 * Set flag for hungry
	 * @param hungry
	 */
	public void setHungry(boolean hungry) {
		this.hungry = hungry;
	}
	/**
	 * Get my current members
	 * @return
	 */
	public ArrayList<Agent> getCurrentMembers() {
		return currentMembers;
	}
	/**
	 * Set my current members
	 * @param currentMembers
	 */
	public void setCurrentMembers(ArrayList<Agent> currentMembers) {
		this.currentMembers = currentMembers;
	}
	/**
	 * Get my church Id
	 * @return
	 */
	public int getChurchID() {
		return churchID;
	}
	/**
	 * Set my church ID
	 * @param churchID
	 */
	public void setChurchID(int churchID) {
		this.churchID = churchID;
	}
}