package units;

import infrastructure.Settings;

import java.util.ArrayList;
import java.util.HashMap;

import cern.jet.random.Normal;
import cern.jet.random.Uniform;


import uchicago.src.sim.util.SimUtilities;

/**
 * This is the abstract mine - which corresponds to the 
 * platinum mine in the region. 
 * 
 *  */
public class Mine {
	/**The record of employees at the mine for renumeration purposes */ 
	private HashMap<Integer, Record> employeeRecord = new HashMap<Integer, Record>();
	/**Skilled workers at the mine */
	private ArrayList<Agent> skilledWorkers = new ArrayList<Agent>();
	/**Unskilled workers at the mine */
	private ArrayList<Agent> unSkilledWorkers = new ArrayList<Agent>();
	/**Trainee workers at the mine */
	private ArrayList<Agent> trainees = new ArrayList<Agent>();
	//magic number
	/**Number of maximum unskilled workers at the mine at a time*/
	int unskilledCapacity = Settings.MINE.UNSKILLED_CAPACITY;
	/**Number of maximum unskilled workers at the mine at a time*/
	int skilledCapacity = Settings.MINE.SKILLED_CAPACITY;
	/**The starting month of the mine*/
	private int startTick = -1;
	private int trainingStartTick = -1;
	/**The stopping month of the mine*/
	private int stopTick = -1;
	/**flag that tells if the mine is currently functioning or has stopped 
	 * or yet to be started*/
	private boolean functioning = false;
	
	/**
	 * Creates the mine and register it to the model.
	 */
	public Mine() {		
		//magic number - between 15th and 24th year
		this.startTick = Uniform.staticNextIntFromTo(Settings.MINE.START_TICK_RANGE-15,
				Settings.MINE.START_TICK_RANGE+15);
		//magic number - 
		this.trainingStartTick = Uniform.staticNextIntFromTo(this.startTick+12,
				this.startTick+18);
		//magic number
		this.stopTick =  this.startTick + 
				Uniform.staticNextIntFromTo(Settings.MINE.MINE_WORKING_MONTHS-60,
						Settings.MINE.MINE_WORKING_MONTHS+60);								
	}
	
	/**
	 * Stop the functioning of the mine
	 *
	 */
	public void stop() {
		functioning = false;
		removeAllWorkers();
		//magic number
		this.startTick = this.stopTick + Uniform.staticNextIntFromTo(36, 72);
				 
	}

	/**
	 * Handles job applications from the villagers and outsiders
	 * @param currentTick: current simulation month
	 * @param jobApplicants: agents applying for the job at the mine
	 */
	public void handleJobApplications(int currentTick, ArrayList<Agent> jobApplicants) {		
		SimUtilities.shuffle(jobApplicants);
		for (Agent agent : jobApplicants) {
			if (shouldEmploy(agent)) {
				employAgent(currentTick, agent);
			}
		}
	}

/*	public void laborTraining() {
		int countUpgrades = 25;
		ArrayList<Agent> temp = new ArrayList<Agent>(this.unSkilledWorkers);
		for (Agent worker : temp) {
			if (countUpgrades > 0) {
				if (shouldUpgradeWorker(worker)) {
					this.unSkilledWorkers.remove(worker);
					this.skilledWorkers.add(worker);
					countUpgrades--;	
				}				
			}
		}
	}	*/	
	/**
	 * Decide if we want to upgrade the status of this mine worker
	 */
	public boolean shouldUpgradeWorker(Agent worker) {
		if (this.skilledWorkers.size() < this.skilledCapacity) {
			return false;
		}
		Integer id = new Integer(worker.getId());
		Record record = (Record) employeeRecord.get(id);
		//now a year working as unskilled
		if (record.timeRecruitment > 6) {
			record.employmentStatus = "skilled";
			return true;
		}
		return false;
		
	}

	/**
	 * Removes all workers from the mine.
	 *
	 */
	public void removeAllWorkers() {
		for (Agent worker : this.unSkilledWorkers) {
			worker.setMineJob(false);
		}
		for (Agent worker : this.skilledWorkers) {
			worker.setMineJob(false);
		}
		this.unSkilledWorkers.clear();
		this.skilledWorkers.clear();		
	}

	/**
	 * Removes agent from the mine
	 * @param agent
	 */
	public void removeMineWorker(Agent agent) {
		if (this.skilledWorkers.contains(agent)) {
			this.skilledWorkers.remove(agent);
		}
		else if (this.unSkilledWorkers.contains(agent)) {
			this.unSkilledWorkers.remove(agent);
		}
	}
	
	/**
	 * Employ the agent at the currentTick (simulation time step)
	 * @param currentTick
	 * @param agent
	 */
	private void employAgent(int currentTick, Agent agent) {
		String status = "";
		if (agent.isMineTrained()
				&& (this.skilledWorkers.size() < this.skilledCapacity)
				&& !this.skilledWorkers.contains(agent)){
			this.skilledWorkers.add(agent);
			status = "skilled";
		}
		else if ((this.unSkilledWorkers.size() < this.unskilledCapacity)
				&& !this.unSkilledWorkers.contains(agent)){
			this.unSkilledWorkers.add(agent);
			status = "unskilled";
		}
		agent.setMineJob(true);
		//cleared. 
		agent.setNumMonthsOutsiderArrival(0);
		Record record = new Record(agent,currentTick, 1, status);
		if (!employeeRecord.containsKey(new Integer(agent.getId()))) {
			employeeRecord.put(new Integer(agent.getId()), record);
		}
	}

	/**
	 * Returns the employess renumeration.
	 * @param employee
	 * @return
	 */
	public int returnEmployeeRenumeration(Agent employee) {
		Record record = (Record) employeeRecord.get(new Integer(employee.getId()));
		if (record.employmentStatus.compareTo("unskilled") == 0) {
//			return (int) Normal.staticNextDouble(250, 50);
			//magic number - gina says minimum 900R
			return (int) Normal.staticNextDouble(1000, 250);
		}
		else {
//			return (int) Normal.staticNextDouble(400, 100);	
			return (int) Normal.staticNextDouble(2000, 5000);
		}		
	}

	/**
	 * policy for employing an agent. Depends upon their 
	 * skills and any other characteristics/backgrounds that might 
	 * be needed
	 */
	private boolean shouldEmploy(Agent agent) {
/*		if (a!agent.isLaborFit()
				|| !agent.isAlive()
				|| agent.getAgeGroup() != Settings.AGE_GROUP.ADULT
				|| agent.getGender() != Settings.GENDER.MALE) {
			return false;
		}
		else {
			return true;
		}
*/	
		if (agent.isMineTrained()) {
			if (this.skilledWorkers.size() < this.skilledCapacity) {
				return true;
			}
		}
		else {
			if (this.unSkilledWorkers.size() < this.unskilledCapacity) {
				return true;
			}
		}
		return false;
	}

	public ArrayList<Agent> getSkilledWorkers() {
		return skilledWorkers;
	}

	public ArrayList<Agent> getUnSkilledWorkers() {
		return unSkilledWorkers;
	}

	public ArrayList<Agent> getTrainees() {
		return trainees;
	}

	public int getStartTick() {
		return startTick;
	}

	public void setStartTick(int startTick) {
		this.startTick = startTick;
	}

	public int getTrainingStartTick() {
		return trainingStartTick;
	}

	public void setTrainingStartTick(int trainingStartTick) {
		this.trainingStartTick = trainingStartTick;
	}

	/**
	 * Maintain record of mine workers at the mine
	 * 
	 *
	 */
	public class Record {
		public Record(Agent _employee, int _tr, int _numRec, String _status) {
			employee = _employee;
			timeRecruitment = _tr;
			numRecruitments = _numRec;
			employmentStatus = _status;			
		}
		Agent employee = new Agent();
		int timeRecruitment = -1;
		int numRecruitments = -1;
		String employmentStatus = "";
	}

	public int getStopTick() {
		return stopTick;
	}

	public void setStopTick(int stopTick) {
		this.stopTick = stopTick;
	}

	public boolean isFunctioning() {
		return functioning;
	}

	public void setFunctioning(boolean functioning) {
		this.functioning = functioning;
	}
}
